@@ -10,6 +10,7 @@
 #include "simgrid/simix.h"        /* semaphores for the barrier */
 #include "xbt.h"                /* calloc, printf */
 #include "instr/instr_private.h"
+#include <xbt/replay.h>
 
 /** @addtogroup MSG_examples
  *
@@ -25,7 +26,7 @@
  *  C file, and the events to react to in a separate text file.
  *  Declare a function handling each of the events that you want to
  *  accept in your trace files, register them using \ref
- *  MSG_action_register in your main, and then use \ref
+ *  xbt_replay_action_register in your main, and then use \ref
  *  MSG_action_trace_run to launch the simulation. You can either
  *  have one trace file containing all your events, or a file per
  *  simulated process. Check the tesh files in the example directory
@@ -53,7 +54,7 @@ typedef struct  {
   xbt_dynar_t isends; /* of msg_comm_t */
   /* Used to implement irecv+wait */
   xbt_dynar_t irecvs; /* of msg_comm_t */
-  xbt_dynar_t tasks; /* of m_task_t */
+  xbt_dynar_t tasks; /* of msg_task_t */
 } s_process_globals_t, *process_globals_t;
 
 /* Helper function */
@@ -148,7 +149,7 @@ static void action_recv(const char *const *action)
 {
   char *name = NULL;
   char mailbox_name[250];
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   double clock = MSG_get_clock();
 
   sprintf(mailbox_name, "%s_%s", action[2],
@@ -164,7 +165,7 @@ static void action_recv(const char *const *action)
 #endif
 
   XBT_DEBUG("Receiving: %s", name);
-  MSG_error_t res = MSG_task_receive(&task, mailbox_name);
+  msg_error_t res = MSG_task_receive(&task, mailbox_name);
   //  MSG_task_receive(&task, MSG_process_get_name(MSG_process_self()));
   XBT_VERB("%s %f", name, MSG_get_clock() - clock);
 
@@ -199,7 +200,7 @@ static void action_Irecv(const char *const *action)
 
   sprintf(mailbox, "%s_%s", action[2],
           MSG_process_get_name(MSG_process_self()));
-  m_task_t t=NULL;
+  msg_task_t t=NULL;
   xbt_dynar_push(globals->tasks,&t);
   msg_comm_t c =
       MSG_task_irecv(
@@ -220,7 +221,7 @@ static void action_Irecv(const char *const *action)
 static void action_wait(const char *const *action)
 {
   char *name = NULL;
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   msg_comm_t comm;
   double clock = MSG_get_clock();
   process_globals_t globals = (process_globals_t) MSG_process_get_data(MSG_process_self());
@@ -240,7 +241,7 @@ static void action_wait(const char *const *action)
   XBT_DEBUG("Entering %s", name);
   comm = xbt_dynar_pop_as(globals->irecvs,msg_comm_t);
   MSG_comm_wait(comm,-1);
-  task = xbt_dynar_pop_as(globals->tasks,m_task_t);
+  task = xbt_dynar_pop_as(globals->tasks,msg_task_t);
   MSG_comm_destroy(comm);
   MSG_task_destroy(task);
 
@@ -295,105 +296,105 @@ static void action_barrier(const char *const *action)
 
 static void action_reduce(const char *const *action)
 {
-	int i;
-	char *reduce_identifier;
-	char mailbox[80];
-	double comm_size = parse_double(action[2]);
-	double comp_size = parse_double(action[3]);
-	m_task_t comp_task = NULL;
-	const char *process_name;
-	double clock = MSG_get_clock();
+  int i;
+  char *reduce_identifier;
+  char mailbox[80];
+  double comm_size = parse_double(action[2]);
+  double comp_size = parse_double(action[3]);
+  msg_task_t comp_task = NULL;
+  const char *process_name;
+  double clock = MSG_get_clock();
 
-	process_globals_t counters = (process_globals_t) MSG_process_get_data(MSG_process_self());
+  process_globals_t counters = (process_globals_t) MSG_process_get_data(MSG_process_self());
 
-	xbt_assert(communicator_size, "Size of Communicator is not defined, "
-			"can't use collective operations");
+  xbt_assert(communicator_size, "Size of Communicator is not defined, "
+      "can't use collective operations");
 
-	process_name = MSG_process_get_name(MSG_process_self());
+  process_name = MSG_process_get_name(MSG_process_self());
 
-	reduce_identifier = bprintf("reduce_%d", counters->reduce_counter++);
+  reduce_identifier = bprintf("reduce_%d", counters->reduce_counter++);
 
-	if (!strcmp(process_name, "p0")) {
-		XBT_DEBUG("%s: %s is the Root", reduce_identifier, process_name);
+  if (!strcmp(process_name, "p0")) {
+    XBT_DEBUG("%s: %s is the Root", reduce_identifier, process_name);
 
-		msg_comm_t *comms = xbt_new0(msg_comm_t,communicator_size-1);
-	    m_task_t *tasks = xbt_new0(m_task_t,communicator_size-1);
-	    for (i = 1; i < communicator_size; i++) {
-	      sprintf(mailbox, "%s_p%d_p0", reduce_identifier, i);
-	      comms[i-1] = MSG_task_irecv(&(tasks[i-1]),mailbox);
-	    }
-	    MSG_comm_waitall(comms,communicator_size-1,-1);
-	    for (i = 1; i < communicator_size; i++) {
-	    	MSG_comm_destroy(comms[i-1]);
-	    	MSG_task_destroy(tasks[i-1]);
-	    }
-	    free(tasks);
+    msg_comm_t *comms = xbt_new0(msg_comm_t,communicator_size-1);
+      msg_task_t *tasks = xbt_new0(msg_task_t,communicator_size-1);
+      for (i = 1; i < communicator_size; i++) {
+        sprintf(mailbox, "%s_p%d_p0", reduce_identifier, i);
+        comms[i-1] = MSG_task_irecv(&(tasks[i-1]),mailbox);
+      }
+      MSG_comm_waitall(comms,communicator_size-1,-1);
+      for (i = 1; i < communicator_size; i++) {
+        MSG_comm_destroy(comms[i-1]);
+        MSG_task_destroy(tasks[i-1]);
+      }
+      free(tasks);
 
-	    comp_task = MSG_task_create("reduce_comp", comp_size, 0, NULL);
-	    XBT_DEBUG("%s: computing 'reduce_comp'", reduce_identifier);
-	    MSG_task_execute(comp_task);
-	    MSG_task_destroy(comp_task);
-	    XBT_DEBUG("%s: computed", reduce_identifier);
+      comp_task = MSG_task_create("reduce_comp", comp_size, 0, NULL);
+      XBT_DEBUG("%s: computing 'reduce_comp'", reduce_identifier);
+      MSG_task_execute(comp_task);
+      MSG_task_destroy(comp_task);
+      XBT_DEBUG("%s: computed", reduce_identifier);
 
-	} else {
-		XBT_DEBUG("%s: %s sends", reduce_identifier, process_name);
-		sprintf(mailbox, "%s_%s_p0", reduce_identifier, process_name);
-	    XBT_DEBUG("put on %s", mailbox);
-	    MSG_task_send(MSG_task_create(reduce_identifier, 0, comm_size, NULL),
-	                  mailbox);
-	}
+  } else {
+    XBT_DEBUG("%s: %s sends", reduce_identifier, process_name);
+    sprintf(mailbox, "%s_%s_p0", reduce_identifier, process_name);
+      XBT_DEBUG("put on %s", mailbox);
+      MSG_task_send(MSG_task_create(reduce_identifier, 0, comm_size, NULL),
+                    mailbox);
+  }
 
-	XBT_VERB("%s %f", xbt_str_join_array(action, " "), MSG_get_clock() - clock);
-	free(reduce_identifier);
+  XBT_VERB("%s %f", xbt_str_join_array(action, " "), MSG_get_clock() - clock);
+  free(reduce_identifier);
 }
 
 static void action_bcast(const char *const *action)
 {
-	int i;
-	char *bcast_identifier;
-	char mailbox[80];
-	double comm_size = parse_double(action[2]);
-	m_task_t task = NULL;
-	const char *process_name;
-	double clock = MSG_get_clock();
+  int i;
+  char *bcast_identifier;
+  char mailbox[80];
+  double comm_size = parse_double(action[2]);
+  msg_task_t task = NULL;
+  const char *process_name;
+  double clock = MSG_get_clock();
 
-	process_globals_t counters = (process_globals_t) MSG_process_get_data(MSG_process_self());
+  process_globals_t counters = (process_globals_t) MSG_process_get_data(MSG_process_self());
 
-	xbt_assert(communicator_size, "Size of Communicator is not defined, "
-			"can't use collective operations");
+  xbt_assert(communicator_size, "Size of Communicator is not defined, "
+      "can't use collective operations");
 
-	process_name = MSG_process_get_name(MSG_process_self());
+  process_name = MSG_process_get_name(MSG_process_self());
 
-	bcast_identifier = bprintf("bcast_%d", counters->bcast_counter++);
+  bcast_identifier = bprintf("bcast_%d", counters->bcast_counter++);
 
-	if (!strcmp(process_name, "p0")) {
-		XBT_DEBUG("%s: %s is the Root", bcast_identifier, process_name);
+  if (!strcmp(process_name, "p0")) {
+    XBT_DEBUG("%s: %s is the Root", bcast_identifier, process_name);
 
-	    msg_comm_t *comms = xbt_new0(msg_comm_t,communicator_size-1);
+      msg_comm_t *comms = xbt_new0(msg_comm_t,communicator_size-1);
 
-	    for (i = 1; i < communicator_size; i++) {
-	      sprintf(mailbox, "%s_p0_p%d", bcast_identifier, i);
-	      comms[i-1] =
-	          MSG_task_isend(MSG_task_create(mailbox,0,comm_size,NULL),
-	              mailbox);
-	    }
-	    MSG_comm_waitall(comms,communicator_size-1,-1);
-		for (i = 1; i < communicator_size; i++)
-	       MSG_comm_destroy(comms[i-1]);
-	    free(comms);
+      for (i = 1; i < communicator_size; i++) {
+        sprintf(mailbox, "%s_p0_p%d", bcast_identifier, i);
+        comms[i-1] =
+            MSG_task_isend(MSG_task_create(mailbox,0,comm_size,NULL),
+                mailbox);
+      }
+      MSG_comm_waitall(comms,communicator_size-1,-1);
+    for (i = 1; i < communicator_size; i++)
+         MSG_comm_destroy(comms[i-1]);
+      free(comms);
 
-	    XBT_DEBUG("%s: all messages sent by %s have been received",
-	           bcast_identifier, process_name);
+      XBT_DEBUG("%s: all messages sent by %s have been received",
+             bcast_identifier, process_name);
 
-	} else {
-	    sprintf(mailbox, "%s_p0_%s", bcast_identifier, process_name);
-	    MSG_task_receive(&task, mailbox);
-	    MSG_task_destroy(task);
-	    XBT_DEBUG("%s: %s has received", bcast_identifier, process_name);
-	}
+  } else {
+      sprintf(mailbox, "%s_p0_%s", bcast_identifier, process_name);
+      MSG_task_receive(&task, mailbox);
+      MSG_task_destroy(task);
+      XBT_DEBUG("%s: %s has received", bcast_identifier, process_name);
+  }
 
-	XBT_VERB("%s %f", xbt_str_join_array(action, " "), MSG_get_clock() - clock);
-	free(bcast_identifier);
+  XBT_VERB("%s %f", xbt_str_join_array(action, " "), MSG_get_clock() - clock);
+  free(bcast_identifier);
 }
 
 
@@ -419,7 +420,7 @@ static void action_allReduce(const char *const *action) {
   char mailbox[80];
   double comm_size = parse_double(action[2]);
   double comp_size = parse_double(action[3]);
-  m_task_t task = NULL, comp_task = NULL;
+  msg_task_t task = NULL, comp_task = NULL;
   const char *process_name;
   double clock = MSG_get_clock();
 
@@ -436,7 +437,7 @@ static void action_allReduce(const char *const *action) {
     XBT_DEBUG("%s: %s is the Root", allreduce_identifier, process_name);
 
     msg_comm_t *comms = xbt_new0(msg_comm_t,communicator_size-1);
-    m_task_t *tasks = xbt_new0(m_task_t,communicator_size-1);
+    msg_task_t *tasks = xbt_new0(msg_task_t,communicator_size-1);
     for (i = 1; i < communicator_size; i++) {
       sprintf(mailbox, "%s_p%d_p0", allreduce_identifier, i);
       comms[i-1] = MSG_task_irecv(&(tasks[i-1]),mailbox);
@@ -502,7 +503,7 @@ static void action_compute(const char *const *action)
 {
   char *name = NULL;
   const char *amout = action[2];
-  m_task_t task = MSG_task_create(name, parse_double(amout), 0, NULL);
+  msg_task_t task = MSG_task_create(name, parse_double(amout), 0, NULL);
   double clock = MSG_get_clock();
 
   if (XBT_LOG_ISENABLED(actions, xbt_log_priority_verbose))
@@ -523,7 +524,7 @@ static void action_init(const char *const *action)
   process_globals_t globals = (process_globals_t) calloc(1, sizeof(s_process_globals_t));
   globals->isends = xbt_dynar_new(sizeof(msg_comm_t),NULL);
   globals->irecvs = xbt_dynar_new(sizeof(msg_comm_t),NULL);
-  globals->tasks  = xbt_dynar_new(sizeof(m_task_t),NULL);
+  globals->tasks  = xbt_dynar_new(sizeof(msg_task_t),NULL);
   MSG_process_set_data(MSG_process_self(),globals);
 
 }
@@ -545,10 +546,10 @@ static void action_finalize(const char *const *action)
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* Check the given arguments */
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file [action_files]\n",
            argv[0]);
@@ -568,27 +569,26 @@ int main(int argc, char *argv[])
   MSG_launch_application(argv[2]);
 
   /*   Action registration */
-  MSG_action_register("init",     action_init);
-  MSG_action_register("finalize", action_finalize);
-  MSG_action_register("comm_size",action_comm_size);
-  MSG_action_register("send",     action_send);
-  MSG_action_register("Isend",    action_Isend);
-  MSG_action_register("recv",     action_recv);
-  MSG_action_register("Irecv",    action_Irecv);
-  MSG_action_register("wait",     action_wait);
-  MSG_action_register("barrier",  action_barrier);
-  MSG_action_register("bcast",    action_bcast);
-  MSG_action_register("reduce",   action_reduce);
-  MSG_action_register("allReduce",action_allReduce);
-  MSG_action_register("sleep",    action_sleep);
-  MSG_action_register("compute",  action_compute);
+  xbt_replay_action_register("init",     action_init);
+  xbt_replay_action_register("finalize", action_finalize);
+  xbt_replay_action_register("comm_size",action_comm_size);
+  xbt_replay_action_register("send",     action_send);
+  xbt_replay_action_register("Isend",    action_Isend);
+  xbt_replay_action_register("recv",     action_recv);
+  xbt_replay_action_register("Irecv",    action_Irecv);
+  xbt_replay_action_register("wait",     action_wait);
+  xbt_replay_action_register("barrier",  action_barrier);
+  xbt_replay_action_register("bcast",    action_bcast);
+  xbt_replay_action_register("reduce",   action_reduce);
+  xbt_replay_action_register("allReduce",action_allReduce);
+  xbt_replay_action_register("sleep",    action_sleep);
+  xbt_replay_action_register("compute",  action_compute);
 
 
   /* Actually do the simulation using MSG_action_trace_run */
   res = MSG_action_trace_run(argv[3]);  // it's ok to pass a NULL argument here
 
   XBT_INFO("Simulation time %g", MSG_get_clock());
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
