@@ -28,7 +28,7 @@ static int surf_parse_bypass_platform(void)
 
   XBT_DEBUG("<AS>");
   SURFXML_BUFFER_SET(AS_id, "AS0");
-  SURFXML_BUFFER_SET(AS_routing, "Full");
+  A_surfxml_AS_routing = A_surfxml_AS_routing_Full;
   SURFXML_START_TAG(AS);
 
   XBT_DEBUG("<host id=\"host A\" power=\"100000000.00\"/>");
@@ -72,7 +72,7 @@ static int surf_parse_bypass_platform(void)
   SURFXML_BUFFER_SET(route_dst, "host B");
   A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_YES;
   SURFXML_START_TAG(route);
-  XBT_DEBUG("	<link:ctn id=\"LinkA\"/>");
+  XBT_DEBUG("  <link:ctn id=\"LinkA\"/>");
   SURFXML_BUFFER_SET(link_ctn_id, "LinkA");
   A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
   SURFXML_START_TAG(link_ctn);
@@ -150,14 +150,14 @@ static int surf_parse_bypass_application(void)
 
 int master(int argc, char *argv[]);
 int slave(int argc, char *argv[]);
-MSG_error_t test_all(void);
+msg_error_t test_all(void);
 
 /** Emitter function  */
 int master(int argc, char *argv[])
 {
   int slaves_count = 0;
-  m_host_t *slaves = NULL;
-  m_task_t *todo = NULL;
+  msg_host_t *slaves = NULL;
+  msg_task_t *todo = NULL;
   int number_of_tasks = 0;
   double task_comp_size = 0;
   double task_comm_size = 0;
@@ -174,7 +174,7 @@ int master(int argc, char *argv[])
   {                             /*  Task creation */
     char sprintf_buffer[64];
 
-    todo = xbt_new0(m_task_t, number_of_tasks);
+    todo = xbt_new0(msg_task_t, number_of_tasks);
 
     for (i = 0; i < number_of_tasks; i++) {
       sprintf(sprintf_buffer, "Task_%d", i);
@@ -186,7 +186,7 @@ int master(int argc, char *argv[])
 
   {                             /* Process organisation */
     slaves_count = argc - 4;
-    slaves = xbt_new0(m_host_t, slaves_count);
+    slaves = xbt_new0(msg_host_t, slaves_count);
 
     for (i = 4; i < argc; i++) {
       slaves[i - 4] = MSG_get_host_by_name(argv[i]);
@@ -220,7 +220,7 @@ int master(int argc, char *argv[])
       ("All tasks have been dispatched. Let's tell everybody the computation is over.");
   for (i = 0; i < slaves_count; i++)
     MSG_task_send(MSG_task_create("finalize", 0, 0, FINALIZE),
-    		MSG_host_get_name(slaves[i]));
+        MSG_host_get_name(slaves[i]));
 
   XBT_INFO("Goodbye now!");
   free(slaves);
@@ -233,7 +233,7 @@ int slave(int argc, char *argv[])
 {
   XBT_INFO("I'm a slave");
   while (1) {
-    m_task_t task = NULL;
+    msg_task_t task = NULL;
     int a;
     a = MSG_task_receive(&task, MSG_host_get_name(MSG_host_self()));
     if (a == MSG_OK) {
@@ -256,9 +256,9 @@ int slave(int argc, char *argv[])
 }                               /* end_of_slave */
 
 /** Test function */
-MSG_error_t test_all(void)
+msg_error_t test_all(void)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /*  Simulation setting */
   surf_parse = surf_parse_bypass_platform;
@@ -279,11 +279,10 @@ MSG_error_t test_all(void)
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   res = test_all();
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
