@@ -7,25 +7,25 @@
 
 #include <simgrid_config.h> /* HAVE_MC ? */
 #include <xbt.h>
+#include "xbt/automaton.h"
 
 #ifndef SIMGRID_MODELCHECKER_H
 #define SIMGRID_MODELCHECKER_H
 
 #ifdef HAVE_MC
 
-extern int _surf_do_model_check;
-#define MC_IS_ENABLED _surf_do_model_check
+extern int _surf_do_model_check; /* please don't use directly: we inline MC_is_active, but that's what you should use */
+
+#define MC_is_active() _surf_do_model_check
 
 XBT_PUBLIC(void) MC_assert(int);
-XBT_PUBLIC(void) MC_assert_stateful(int);
 XBT_PUBLIC(int) MC_random(int min, int max);
-XBT_PUBLIC(void) MC_diff(void);
+XBT_PUBLIC(void) MC_automaton_new_propositional_symbol(const char* id, void* fct);
 
 #else
 
-#define MC_IS_ENABLED 0
 #define MC_assert(a) xbt_assert(a)
-#define MC_assert_stateful(a) xbt_assert(a)
+#define MC_is_active() 0
 
 #endif
 
