@@ -19,9 +19,8 @@ SG_BEGIN_DECL()
 
 
 /* ******************************** Host ************************************ */
-/** @defgroup m_datatypes_management_details Details on SIMIX datatypes */
 /** @brief Host datatype
-    @ingroup m_datatypes_management
+    @ingroup simix_host_management
 
     A <em>location</em> (or <em>host</em>) is any possible place where
     a process may run. Thus it is represented as a <em>physical
@@ -52,8 +51,17 @@ typedef enum {
 typedef struct s_smx_timer* smx_timer_t;
 
 /* ******************************** Synchro ************************************ */
+/**
+ * \ingroup simix_synchro_management
+ */
 typedef struct s_smx_mutex *smx_mutex_t;
+/**
+ * \ingroup simix_synchro_management
+ */
 typedef struct s_smx_cond *smx_cond_t;
+/**
+ * \ingroup simix_synchro_management
+ */
 typedef struct s_smx_sem *smx_sem_t;
 
 /********************************** File *************************************/
@@ -67,7 +75,7 @@ typedef struct s_smx_action *smx_action_t; /* FIXME: replace by specialized acti
 
 /* ****************************** Process *********************************** */
 /** @brief Process datatype
-    @ingroup m_datatypes_management
+    @ingroup simix_process_management
 
     A processt may be defined as a <em>code</em>, with some <em>private
     data</em>, executing in a <em>location</em>.
@@ -96,19 +104,29 @@ typedef void (*smx_creation_func_t) ( /* process */ smx_process_t*,
                                       /* kill_time */ double,
                                       /* argc */ int,
                                       /* argv */ char**,
-                                      /* props */ xbt_dict_t);
+                                      /* props */ xbt_dict_t,
+                                      /* auto_restart */ int);
 
 
 /******************************* Networking ***********************************/
+/**
+ * \ingroup simix_rdv_management
+ */
 typedef struct s_smx_rvpoint *smx_rdv_t;
 
+XBT_PUBLIC(void*) SIMIX_comm_get_src_data(smx_action_t action);
+XBT_PUBLIC(void*) SIMIX_comm_get_dst_data(smx_action_t action);
+
 /******************************** Context *************************************/
 typedef struct s_smx_context *smx_context_t;
 typedef struct s_smx_context_factory *smx_context_factory_t;
 
 /* Process creation/destruction callbacks */
 typedef void (*void_pfn_smxprocess_t) (smx_process_t);
-
+/* Process kill */
+typedef void (*void_pfn_smxprocess_t_smxprocess_t) (smx_process_t, smx_process_t);
+/* for auto-restart function */
+typedef void (*void_pfn_smxhost_t) (smx_host_t);
 
 /* The following function pointer types describe the interface that any context
    factory should implement */
@@ -164,7 +182,7 @@ typedef struct s_smx_context {
   void *data;   /* Here SIMIX stores the smx_process_t containing the context */
   char **argv;
   int argc;
-  int iwannadie:1;
+  unsigned iwannadie:1;
 } s_smx_ctx_base_t;
 
 /* methods of this class */
@@ -200,12 +218,11 @@ XBT_PUBLIC(void) SIMIX_context_set_parallel_mode(e_xbt_parmap_mode_t mode);
 /********************************** Global ************************************/
 /* Initialization and exit */
 XBT_PUBLIC(void) SIMIX_global_init(int *argc, char **argv);
-XBT_PUBLIC(void) SIMIX_clean(void);
 
 
 XBT_PUBLIC(void) SIMIX_function_register_process_cleanup(void_pfn_smxprocess_t function);
 XBT_PUBLIC(void) SIMIX_function_register_process_create(smx_creation_func_t function);
-XBT_PUBLIC(void) SIMIX_function_register_process_kill(void_pfn_smxprocess_t function);
+XBT_PUBLIC(void) SIMIX_function_register_process_kill(void_pfn_smxprocess_t_smxprocess_t function);
 
 /* Simulation execution */
 XBT_PUBLIC(void) SIMIX_run(void);    
@@ -252,6 +269,8 @@ XBT_PUBLIC(void*) SIMIX_process_self_get_data(smx_process_t self);
 XBT_PUBLIC(smx_context_t) SIMIX_process_get_context(smx_process_t);
 XBT_PUBLIC(void) SIMIX_process_set_context(smx_process_t p,smx_context_t c);
 XBT_PUBLIC(int) SIMIX_process_has_pending_comms(smx_process_t process);
+XBT_PUBLIC(void) SIMIX_process_on_exit_runall(smx_process_t process);
+XBT_PUBLIC(void) SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data);
 
 /****************************** Communication *********************************/
 XBT_PUBLIC(void) SIMIX_comm_set_copy_data_callback(void (*callback) (smx_action_t, void*, size_t));
@@ -312,7 +331,8 @@ XBT_PUBLIC(void) simcall_process_create(smx_process_t *process,
                                           const char *hostname,
                                           double kill_time,
                                           int argc, char **argv,
-                                          xbt_dict_t properties);
+                                          xbt_dict_t properties,
+                                          int auto_restart);
 
 XBT_PUBLIC(void) simcall_process_kill(smx_process_t process);
 XBT_PUBLIC(void) simcall_process_killall(void);
@@ -320,7 +340,7 @@ XBT_PUBLIC(void) simcall_process_killall(void);
 /* Process handling */
 XBT_PUBLIC(void) simcall_process_cleanup(smx_process_t process);
 XBT_PUBLIC(void) simcall_process_change_host(smx_process_t process,
-					       smx_host_t dest);
+                 smx_host_t dest);
 XBT_PUBLIC(void) simcall_process_suspend(smx_process_t process);
 XBT_PUBLIC(void) simcall_process_resume(smx_process_t process);
 
@@ -333,7 +353,9 @@ XBT_PUBLIC(const char *) simcall_process_get_name(smx_process_t process);
 XBT_PUBLIC(int) simcall_process_is_suspended(smx_process_t process);
 XBT_PUBLIC(xbt_dict_t) simcall_process_get_properties(smx_process_t host);
 XBT_PUBLIC(void) simcall_process_set_kill_time(smx_process_t process, double kill_time);
-
+XBT_PUBLIC(void) simcall_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data);
+XBT_PUBLIC(void) simcall_process_auto_restart_set(smx_process_t process, int auto_restart);
+XBT_PUBLIC(smx_process_t) simcall_process_restart(smx_process_t process);
 /* Sleep control */
 XBT_PUBLIC(e_smx_state_t) simcall_process_sleep(double duration);
 
@@ -345,6 +367,8 @@ XBT_PUBLIC(void) simcall_rdv_destroy(smx_rdv_t rvp);
 XBT_PUBLIC(smx_rdv_t) simcall_rdv_get_by_name(const char *name);
 XBT_PUBLIC(int) simcall_rdv_comm_count_by_host(smx_rdv_t rdv, smx_host_t host);
 XBT_PUBLIC(smx_action_t) simcall_rdv_get_head(smx_rdv_t rdv);
+XBT_PUBLIC(smx_process_t) simcall_rdv_get_receiver(smx_rdv_t rdv);
+XBT_PUBLIC(void) simcall_rdv_set_receiver(smx_rdv_t rdv , smx_process_t process);
 
 XBT_PUBLIC(xbt_dict_t) SIMIX_get_rdv_points(void);
 
@@ -374,7 +398,8 @@ XBT_PUBLIC(smx_action_t) simcall_comm_irecv(smx_rdv_t rdv, void *dst_buff,
                                               void *data);
 
 XBT_PUBLIC(void) simcall_comm_destroy(smx_action_t comm);
-
+XBT_PUBLIC(smx_action_t) simcall_comm_iprobe(smx_rdv_t rdv, int src, int tag,
+                                int (*match_fun)(void *, void *, smx_action_t), void *data);
 XBT_PUBLIC(void) simcall_comm_cancel(smx_action_t comm);
 
 /* FIXME: waitany is going to be a vararg function, and should take a timeout */
@@ -429,11 +454,23 @@ XBT_PUBLIC(void) simcall_sem_acquire_timeout(smx_sem_t sem,
 XBT_PUBLIC(unsigned int) simcall_sem_acquire_any(xbt_dynar_t sems);
 XBT_PUBLIC(int) simcall_sem_get_capacity(smx_sem_t sem);
 
-XBT_PUBLIC(size_t) simcall_file_read(const char* storage, void* ptr, size_t size, size_t nmemb, smx_file_t stream);
-XBT_PUBLIC(size_t) simcall_file_write(const char* storage, const void* ptr, size_t size, size_t nmemb, smx_file_t stream);
+XBT_PUBLIC(double) simcall_file_read(void* ptr, size_t size, size_t nmemb, smx_file_t stream);
+XBT_PUBLIC(size_t) simcall_file_write(const void* ptr, size_t size, size_t nmemb, smx_file_t stream);
 XBT_PUBLIC(smx_file_t) simcall_file_open(const char* storage, const char* path, const char* mode);
-XBT_PUBLIC(int) simcall_file_close(const char* storage, smx_file_t fp);
-XBT_PUBLIC(int) simcall_file_stat(const char* storage, smx_file_t fd, s_file_stat_t *buf);
+XBT_PUBLIC(int) simcall_file_close(smx_file_t fp);
+XBT_PUBLIC(int) simcall_file_stat(smx_file_t fd, s_file_stat_t *buf);
+XBT_PUBLIC(int) simcall_file_unlink(smx_file_t fd);
+XBT_PUBLIC(xbt_dict_t) simcall_file_ls(const char* mount, const char* path);
+
+/************************** AS router   **********************************/
+XBT_PUBLIC(xbt_dict_t) SIMIX_asr_get_properties(const char *name);
+/************************** AS router simcalls ***************************/
+XBT_PUBLIC(xbt_dict_t) simcall_asr_get_properties(const char *name);
+
+/************************** New API simcalls **********************************/
+/* TUTORIAL: New API                                                          */
+/******************************************************************************/
+XBT_PUBLIC(int) simcall_new_api_fct(const char* param1, double param2);
 
 SG_END_DECL()
 #endif                          /* _SIMIX_SIMIX_H */
