@@ -28,9 +28,7 @@ typedef struct xbt_automaton {
 typedef struct xbt_automaton* xbt_automaton_t;
 
 typedef struct xbt_exp_label{
-  /* fixme: "or", "and", and "not" are reserved keywords in C++ */
-  /* enum{or=0, and=1, not=2, predicat=3, one=4} type; */
-  int type;
+  enum{or=0, and=1, not=2, predicat=3, one=4} type;
   union{
     struct{
       struct xbt_exp_label* left_exp;
@@ -61,7 +59,8 @@ typedef struct xbt_propositional_symbol{
 typedef struct xbt_propositional_symbol* xbt_propositional_symbol_t;
 
 
-XBT_PUBLIC(xbt_automaton_t) xbt_automaton_new_automaton(void);
+XBT_PUBLIC(xbt_automaton_t) xbt_automaton_new(void);
+XBT_PUBLIC(void) xbt_automaton_load(xbt_automaton_t automaton, const char *file);
 
 XBT_PUBLIC(xbt_state_t) xbt_automaton_new_state(xbt_automaton_t a, int type, char* id);
 
