@@ -14,25 +14,46 @@
 #include "simgrid/simix.h"
 #include "simgrid/modelchecker.h" /* our public interface (and definition of HAVE_MC) */
 #include "xbt/automaton.h"
+#include "xbt/dynar.h"
 
+#define STD_HEAP_SIZE   20480000        /* Maximum size of the system's heap */
 
 SG_BEGIN_DECL()
 
+extern char*_surf_mc_property_file; /* fixme: better location? */
+
+extern xbt_dynar_t mc_comparison_ignore;
+extern xbt_dynar_t stacks_areas;
+
 /********************************* Global *************************************/
-XBT_PUBLIC(void) MC_init_safety_stateless(void);
-XBT_PUBLIC(void) MC_init_safety_stateful(void);
+void _mc_cfg_cb_reduce(const char *name, int pos);
+void _mc_cfg_cb_checkpoint(const char *name, int pos);
+void _mc_cfg_cb_property(const char *name, int pos);
+
+XBT_PUBLIC(void) MC_do_the_modelcheck_for_real(void);
+
+XBT_PUBLIC(void) MC_init_safety(void);
 XBT_PUBLIC(void) MC_exit(void);
 XBT_PUBLIC(void) MC_exit_liveness(void);
 XBT_PUBLIC(void) MC_modelcheck(void);
-XBT_PUBLIC(void) MC_modelcheck_stateful(void);
-XBT_PUBLIC(void) MC_modelcheck_liveness(xbt_automaton_t a, char *prgm);
+XBT_PUBLIC(void) MC_modelcheck_liveness(void);
 XBT_PUBLIC(void) MC_process_clock_add(smx_process_t, double);
 XBT_PUBLIC(double) MC_process_clock_get(smx_process_t);
+void MC_automaton_load(const char *file);
+
+XBT_PUBLIC(void) MC_ignore(void *address, size_t size);
+void MC_new_stack_area(void *stack, char *name, void *context);
 
 /********************************* Memory *************************************/
 XBT_PUBLIC(void) MC_memory_init(void);  /* Initialize the memory subsystem */
 XBT_PUBLIC(void) MC_memory_exit(void);
 
+/********************************* Snapshot comparison test *************************************/
+void MC_test_heap_comparison(void);
+
+/* Trigger for state equality detection (check potential cycle in application) */
+void MC_compare(void);
+
 SG_END_DECL()
 
 #endif                          /* _MC_MC_H */
