@@ -79,16 +79,16 @@ typedef struct surf_action {
   double cost;                  /**< cost        */
   double priority;              /**< priority (1.0 by default) */
   double max_duration;          /**< max_duration (may fluctuate until
-				   the task is completed) */
+           the task is completed) */
   double remains;               /**< How much of that cost remains to
-				 * be done in the currently running task */
+         * be done in the currently running task */
 #ifdef HAVE_LATENCY_BOUND_TRACKING
   int latency_limited;               /**< Set to 1 if is limited by latency, 0 otherwise */
 #endif
 
   double start;                 /**< start time  */
   double finish;                /**< finish time : this is modified during the run
-				 * and fluctuates until the task is completed */
+         * and fluctuates until the task is completed */
   void *data;                   /**< for your convenience */
   int refcount;
   surf_model_t model_type;
@@ -97,6 +97,7 @@ typedef struct surf_action {
 #endif
   surf_file_t file;        /**< surf_file_t for storage model */
   s_file_stat_t stat;        /**< surf_file_t for storage model */
+  xbt_dict_t ls_dict;
 } s_surf_action_t;
 
 typedef struct surf_action_lmm {
@@ -152,6 +153,16 @@ typedef struct surf_action_state {
 typedef struct s_routing_platf s_routing_platf_t, *routing_platf_t;
 XBT_PUBLIC_DATA(routing_platf_t) routing_platf;
 
+/*******************************************
+ *  TUTORIAL: New model
+ *  New model extension public
+ *  Public functions specific to a New model.
+ */
+typedef struct surf_new_model_extension_public {
+  surf_action_t(*fct) ();
+  void* (*create_resource) ();
+} s_surf_model_extension_new_model_t;
+/*******************************************/
 
 /** \ingroup SURF_models
  *  \brief Private data available on all models
@@ -218,12 +229,14 @@ typedef struct surf_network_model_extension_public {
  */
 
 typedef struct surf_storage_model_extension_public {
-  surf_action_t(*open) (void *storage, const char* path, const char* mode);
+  surf_action_t(*open) (void *storage, const char* mount, const char* path, const char* mode);
   surf_action_t(*close) (void *storage, surf_file_t fp);
-  surf_action_t(*read) (void *storage, void* ptr, size_t size, size_t nmemb, surf_file_t stream);
+  surf_action_t(*read) (void *storage, void* ptr, double size, size_t nmemb, surf_file_t stream);
   surf_action_t(*write) (void *storage, const void* ptr, size_t size, size_t nmemb, surf_file_t stream);
   surf_action_t(*stat) (void *storage, surf_file_t stream);
-  void* (*create_resource) (const char* id, const char* model,const char* type_id);
+  surf_action_t(*unlink) (void *storage, surf_file_t stream);
+  surf_action_t(*ls) (void *storage, const char *path);
+  void* (*create_resource) (const char* id, const char* model, const char* type_id, const char *content);
 } s_surf_model_extension_storage_t;
 
      /** \ingroup SURF_models
@@ -233,7 +246,7 @@ typedef struct surf_storage_model_extension_public {
       */
 typedef struct surf_workstation_model_extension_public {
   surf_action_t(*execute) (void *workstation, double size);                                /**< Execute a computation amount on a workstation
-                    									and create the corresponding action */
+                                      and create the corresponding action */
   surf_action_t(*sleep) (void *workstation, double duration);                              /**< Make a workstation sleep during a given duration */
   e_surf_resource_state_t(*get_state) (void *workstation);                                      /**< Return the CPU state of a workstation */
   double (*get_speed) (void *workstation, double load);                                    /**< Return the speed of a workstation */
@@ -252,10 +265,13 @@ typedef struct surf_workstation_model_extension_public {
   double (*get_link_bandwidth) (const void *link);                                         /**< Return the current bandwidth of a network link */
   double (*get_link_latency) (const void *link);                                           /**< Return the current latency of a network link */
   surf_action_t(*open) (void *workstation, const char* storage, const char* path, const char* mode);
-  surf_action_t(*close) (void *workstation, const char* storage, surf_file_t fp);
-  surf_action_t(*read) (void *workstation, const char* storage, void* ptr, size_t size, size_t nmemb, surf_file_t stream);
-  surf_action_t(*write) (void *workstation, const char* storage, const void* ptr, size_t size, size_t nmemb, surf_file_t stream);
-  surf_action_t(*stat) (void *workstation, const char* storage, surf_file_t stream);
+  surf_action_t(*close) (void *workstation, surf_file_t fp);
+  surf_action_t(*read) (void *workstation, void* ptr, size_t size, size_t nmemb, surf_file_t stream);
+  surf_action_t(*write) (void *workstation, const void* ptr, size_t size, size_t nmemb, surf_file_t stream);
+  surf_action_t(*stat) (void *workstation, surf_file_t stream);
+  surf_action_t(*unlink) (void *workstation, surf_file_t stream);
+  surf_action_t(*ls) (void *workstation, const char* mount, const char *path);
+
   int (*link_shared) (const void *link);
    xbt_dict_t(*get_properties) (const void *resource);
   void* (*link_create_resource) (const char *name,
@@ -299,7 +315,7 @@ typedef struct surf_model {
 
   double (*action_get_start_time) (surf_action_t action);     /**< Return the start time of an action */
   double (*action_get_finish_time) (surf_action_t action);     /**< Return the finish time of an action */
-  int (*action_unref) (surf_action_t action);     /**< Specify that we don't use that action anymore */
+  int (*action_unref) (surf_action_t action);     /**< Specify that we don't use that action anymore. Returns true if the action was destroyed and false if someone still has references on it. */
   void (*action_cancel) (surf_action_t action);     /**< Cancel a running action */
   void (*action_recycle) (surf_action_t action);     /**< Recycle an action */
   void (*action_data_set) (surf_action_t action, void *data);     /**< Set the user data of an action */
@@ -316,6 +332,8 @@ typedef struct surf_model {
   int (*get_latency_limited) (surf_action_t action);     /**< Return 1 if action is limited by latency, 0 otherwise */
 #endif
 
+  void (*gap_remove) (surf_action_lmm_t action);
+
   surf_model_private_t model_private;
 
   union extension {
@@ -323,6 +341,10 @@ typedef struct surf_model {
     s_surf_model_extension_network_t network;
     s_surf_model_extension_storage_t storage;
     s_surf_model_extension_workstation_t workstation;
+    /*******************************************/
+    /* TUTORIAL: New model                     */
+    s_surf_model_extension_new_model_t new_model;
+    /*******************************************/
   } extension;
 } s_surf_model_t;
 
@@ -330,10 +352,10 @@ surf_model_t surf_model_init(void);
 void surf_model_exit(surf_model_t model);
 
 static inline void *surf_cpu_resource_by_name(const char *name) {
-	return xbt_lib_get_or_null(host_lib, name, SURF_CPU_LEVEL);
+  return xbt_lib_get_or_null(host_lib, name, SURF_CPU_LEVEL);
 }
 static inline void *surf_workstation_resource_by_name(const char *name){
-	return xbt_lib_get_or_null(host_lib, name, SURF_WKS_LEVEL);
+  return xbt_lib_get_or_null(host_lib, name, SURF_WKS_LEVEL);
 }
 static inline void *surf_storage_resource_by_name(const char *name){
     return xbt_lib_get_or_null(storage_lib, name, SURF_STORAGE_LEVEL);
@@ -565,11 +587,6 @@ XBT_PUBLIC(void) surf_network_model_init_Vegas(void);
 XBT_PUBLIC_DATA(s_surf_model_description_t)
     surf_network_model_description[];
 
-
-
-
-
-
 /** \ingroup SURF_models
  *  \brief The storage model
  */
@@ -581,12 +598,6 @@ XBT_PUBLIC(void) surf_storage_model_init_default(void);
  */
 XBT_PUBLIC_DATA(s_surf_model_description_t) surf_storage_model_description[];
 
-
-
-
-
-
-
 /** \ingroup SURF_models
  *  \brief The workstation model
  *
@@ -646,6 +657,13 @@ XBT_PUBLIC(void) surf_workstation_model_init_ptask_L07(void);
 XBT_PUBLIC_DATA(s_surf_model_description_t)
     surf_workstation_model_description[];
 
+/*******************************************
+ *  TUTORIAL: New model
+ */
+XBT_PUBLIC(void) surf_new_model_init_default(void);
+XBT_PUBLIC_DATA(s_surf_model_description_t) surf_new_model_description[];
+/*******************************************/
+
 /** \ingroup SURF_models
  *  \brief List of initialized models
  */
@@ -727,8 +745,12 @@ XBT_PUBLIC_DATA(xbt_dict_t) trace_connect_list_latency;
 
 XBT_PUBLIC(double) get_cpu_power(const char *power);
 
+XBT_PUBLIC(xbt_dict_t) get_as_router_properties(const char* name);
+
 int surf_get_nthreads(void);
 void surf_set_nthreads(int nthreads);
 
+void surf_watched_hosts(void);
+
 SG_END_DECL()
 #endif                          /* _SURF_SURF_H */
