@@ -53,7 +53,7 @@ static xbt_dynar_t instr_dict_to_dynar (xbt_dict_t filter)
  *  that is being declared is random. This function has no effect
  *  if a category with the same name has been already declared.
  *
- * See \ref tracing_tracing for details on how to trace
+ * See \ref tracing for details on how to trace
  * the (categorized) resource utilization.
  *
  *  \param category The name of the new tracing category to be created.
@@ -74,11 +74,11 @@ void TRACE_category(const char *category)
  *  can be specified using "0.7 0.7 0.7" as color. This function has
  *  no effect if a category with the same name has been already declared.
  *
- * See \ref tracing_tracing for details on how to trace
+ * See \ref tracing for details on how to trace
  * the (categorized) resource utilization.
  *
  *  \param category The name of the new tracing category to be created.
- *  \param color The color of the category (see \ref tracing_tracing to
+ *  \param color The color of the category (see \ref tracing to
  *  know how to correctly specify the color)
  *
  *  \see MSG_task_set_category, SD_task_set_category
@@ -123,7 +123,7 @@ void TRACE_category_with_color (const char *category, const char *color)
  * This function should be used to get categories that were already
  * declared with #TRACE_category or with #TRACE_category_with_color.
  *
- * See \ref tracing_tracing for details on how to trace
+ * See \ref tracing for details on how to trace
  * the (categorized) resource utilization.
  *
  * \return A dynar with the declared categories, must be freed with xbt_dynar_free.
@@ -156,15 +156,81 @@ void TRACE_declare_mark(const char *mark_type)
   /* safe switch */
   if (!TRACE_is_enabled()) return;
 
-  if (!mark_type) return;
+  /* if platform is not traced, we don't allow marks */
+  if (!TRACE_needs_platform()) return;
+
+  if (!mark_type) THROWF (tracing_error, 1, "mark_type is NULL");
 
   //check if mark_type is already declared
   char *created = xbt_dict_get_or_null(declared_marks, mark_type);
-  if (created) return;
-  xbt_dict_set (declared_marks, mark_type, xbt_strdup("1"), NULL);
+  if (created) {
+    THROWF (tracing_error, 1, "mark_type with name (%s) is already declared", mark_type);
+  }
 
   XBT_DEBUG("MARK,declare %s", mark_type);
-  PJ_type_event_new(mark_type, NULL, PJ_type_get_root());
+  PJ_type_event_new(mark_type, PJ_type_get_root());
+  xbt_dict_set (declared_marks, mark_type, xbt_strdup("1"), NULL);
+}
+
+/** \ingroup TRACE_mark
+ * \brief Declare a new colored value for a previously declared mark type.
+ *
+ * This function declares a new colored value for a Paje event
+ * type in the trace file that can be used by
+ * simulators to declare application-level
+ * marks. This function is independent of
+ * which API is used in SimGrid. The color needs to be
+ * a string with three numbers separated by spaces in the range [0,1].
+ * A light-gray color can be specified using "0.7 0.7 0.7" as color.
+ * If a NULL color is provided, the color used will be white ("1 1 1").
+ *
+ * \param mark_type The name of the new type.
+ * \param mark_value The name of the new value for this type.
+ * \param mark_color The color of the new value for this type.
+ *
+ * \see TRACE_mark
+ */
+void TRACE_declare_mark_value_with_color (const char *mark_type, const char *mark_value, const char *mark_color)
+{
+  /* safe switch */
+  if (!TRACE_is_enabled()) return;
+
+  /* if platform is not traced, we don't allow marks */
+  if (!TRACE_needs_platform()) return;
+
+  if (!mark_type) THROWF (tracing_error, 1, "mark_type is NULL");
+  if (!mark_value) THROWF (tracing_error, 1, "mark_value is NULL");
+
+  type_t type = PJ_type_get (mark_type, PJ_type_get_root());
+  if (!type){
+    THROWF (tracing_error, 1, "mark_type with name (%s) is not declared", mark_type);
+  }
+
+  char white[INSTR_DEFAULT_STR_SIZE] = "1.0 1.0 1.0";
+  if (!mark_color) mark_color = white;
+
+  XBT_DEBUG("MARK,declare_value %s %s %s", mark_type, mark_value, mark_color);
+  PJ_value_new (mark_value, mark_color, type);
+}
+
+/** \ingroup TRACE_mark
+ * \brief Declare a new value for a previously declared mark type.
+ *
+ * This function declares a new value for a Paje event
+ * type in the trace file that can be used by
+ * simulators to declare application-level
+ * marks. This function is independent of
+ * which API is used in SimGrid. Calling this function is the same
+ * as calling \ref TRACE_declare_mark_value_with_color with a NULL color.
+ *
+ * \param mark_type The name of the new type.
+ * \param mark_value The name of the new value for this type.
+ *
+ * \see TRACE_mark
+ */
+void TRACE_declare_mark_value (const char *mark_type, const char *mark_value)
+{
+  TRACE_declare_mark_value_with_color (mark_type, mark_value, NULL);
 }
 
 /**
@@ -190,21 +256,20 @@ void TRACE_mark(const char *mark_type, const char *mark_value)
   /* safe switch */
   if (!TRACE_is_enabled()) return;
 
-  if (!mark_type || !mark_value) return;
+  /* if platform is not traced, we don't allow marks */
+  if (!TRACE_needs_platform()) return;
+
+  if (!mark_type) THROWF (tracing_error, 1, "mark_type is NULL");
+  if (!mark_value) THROWF (tracing_error, 1, "mark_value is NULL");
 
   //check if mark_type is already declared
-  char *created = xbt_dict_get_or_null(declared_marks, mark_type);
-  if (created) return;
-
-  XBT_DEBUG("MARK %s %s", mark_type, mark_value);
   type_t type = PJ_type_get (mark_type, PJ_type_get_root());
-  if (type == NULL){
-    THROWF (tracing_error, 1, "mark_type with name (%s) not declared before", mark_type);
+  if (!type){
+    THROWF (tracing_error, 1, "mark_type with name (%s) is not declared", mark_type);
   }
+
   val_t value = PJ_value_get (mark_value, type);
-  if (value == NULL){
-    value = PJ_value_new (mark_value, NULL, type);
-  }
+  XBT_DEBUG("MARK %s %s", mark_type, mark_value);
   new_pajeNewEvent (MSG_get_clock(), PJ_container_get_root(), type, value);
 }
 
@@ -829,17 +894,13 @@ void TRACE_host_state_declare_value (const char *state, const char *value, const
  *  \param state The name of the state previously declared.
  *  \param value The new value of the state.
  *
- *  \see TRACE_host_state_declare, TRACE_host_push_state, TRACE_host_pop_state
+ *  \see TRACE_host_state_declare, TRACE_host_push_state, TRACE_host_pop_state, TRACE_host_reset_state
  */
 void TRACE_host_set_state (const char *host, const char *state, const char *value)
 {
   container_t container = PJ_container_get(host);
   type_t type = PJ_type_get (state, container->type);
-  val_t val = PJ_value_get (value, type);
-  if (val == NULL){
-    //if user didn't declare a value with a color, user a NULL color
-    PJ_value_new (value, NULL, type);
-  }
+  val_t val = PJ_value_get_or_new (value, NULL, type); /* if user didn't declare a value with a color, user a NULL color */
   new_pajeSetState(MSG_get_clock(), container, type, val);
 }
 
@@ -852,17 +913,13 @@ void TRACE_host_set_state (const char *host, const char *state, const char *valu
  *  \param state The name of the state previously declared.
  *  \param value The value to be pushed.
  *
- *  \see TRACE_host_state_declare, TRACE_host_set_state, TRACE_host_pop_state
+ *  \see TRACE_host_state_declare, TRACE_host_set_state, TRACE_host_pop_state, TRACE_host_reset_state
  */
 void TRACE_host_push_state (const char *host, const char *state, const char *value)
 {
   container_t container = PJ_container_get(host);
   type_t type = PJ_type_get (state, container->type);
-  val_t val = PJ_value_get (value, type);
-  if (val == NULL){
-    //if user didn't declare a value with a color, user a NULL color
-    PJ_value_new (value, NULL, type);
-  }
+  val_t val = PJ_value_get_or_new (value, NULL, type); /* if user didn't declare a value with a color, user a NULL color */
   new_pajePushState(MSG_get_clock(), container, type, val);
 }
 
@@ -874,7 +931,7 @@ void TRACE_host_push_state (const char *host, const char *state, const char *val
  *  \param host The name of the host to be considered.
  *  \param state The name of the state to be popped.
  *
- *  \see TRACE_host_state_declare, TRACE_host_set_state, TRACE_host_push_state
+ *  \see TRACE_host_state_declare, TRACE_host_set_state, TRACE_host_push_state, TRACE_host_reset_state
  */
 void TRACE_host_pop_state (const char *host, const char *state)
 {
@@ -883,6 +940,23 @@ void TRACE_host_pop_state (const char *host, const char *state)
   new_pajePopState(MSG_get_clock(), container, type);
 }
 
+/** \ingroup TRACE_user_variables
+ *  \brief Reset the state of a given host.
+ *
+ *  Clear all previous values of a user state.
+ *
+ *  \param host The name of the host to be considered.
+ *  \param state The name of the state to be cleared.
+ *
+ *  \see TRACE_host_state_declare, TRACE_host_set_state, TRACE_host_push_state, TRACE_host_pop_state
+ */
+void TRACE_host_reset_state (const char *host, const char *state)
+{
+  container_t container = PJ_container_get(host);
+  type_t type = PJ_type_get (state, container->type);
+  new_pajeResetState(MSG_get_clock(), container, type);
+}
+
 /** \ingroup TRACE_API
  *  \brief Get Paje container types that can be mapped to the nodes of a graph.
  *
@@ -911,4 +985,22 @@ xbt_dynar_t TRACE_get_edge_types (void)
   return instr_dict_to_dynar (trivaEdgeTypes);
 }
 
+/** \ingroup TRACE_API
+ *  \brief Pauses all tracing activities.
+ *  \see TRACE_resume
+ */
+void TRACE_pause (void)
+{
+  instr_pause_tracing();
+}
+
+/** \ingroup TRACE_API
+ *  \brief Resumes all tracing activities.
+ *  \see TRACE_pause
+ */
+void TRACE_resume (void)
+{
+  instr_resume_tracing();
+}
+
 #endif /* HAVE_TRACING */
