@@ -24,6 +24,27 @@
 #include "simix/smx_private.h"
 #include "xbt/graph_private.h"
 
+typedef enum {
+  PAJE_DefineContainerType,
+  PAJE_DefineVariableType,
+  PAJE_DefineStateType,
+  PAJE_DefineEventType,
+  PAJE_DefineLinkType,
+  PAJE_DefineEntityValue,
+  PAJE_CreateContainer,
+  PAJE_DestroyContainer,
+  PAJE_SetVariable,
+  PAJE_AddVariable,
+  PAJE_SubVariable,
+  PAJE_SetState,
+  PAJE_PushState,
+  PAJE_PopState,
+  PAJE_ResetState,
+  PAJE_StartLink,
+  PAJE_EndLink,
+  PAJE_NewEvent
+} e_event_type;
+
 typedef enum {
   TYPE_VARIABLE,
   TYPE_LINK,
@@ -79,8 +100,10 @@ extern xbt_dict_t user_host_variables;
 extern xbt_dict_t user_link_variables;
 extern double TRACE_last_timestamp_to_dump;
 
+/* instr_paje_header.c */
+void TRACE_header(int basic);
+
 /* from paje.c */
-void TRACE_paje_create_header(void);
 void TRACE_paje_start(void);
 void TRACE_paje_end(void);
 void TRACE_paje_dump_buffer (int force);
@@ -98,33 +121,35 @@ void new_pajeSubVariable (double timestamp, container_t container, type_t type,
 void new_pajeSetState (double timestamp, container_t container, type_t type, val_t value);
 void new_pajePushState (double timestamp, container_t container, type_t type, val_t value);
 void new_pajePopState (double timestamp, container_t container, type_t type);
+void new_pajeResetState (double timestamp, container_t container, type_t type);
 void new_pajeStartLink (double timestamp, container_t container, type_t type, container_t sourceContainer, const char *value, const char *key);
 void new_pajeEndLink (double timestamp, container_t container, type_t type, container_t destContainer, const char *value, const char *key);
 void new_pajeNewEvent (double timestamp, container_t container, type_t type, val_t value);
 
 /* declaration of instrumentation functions from msg_task_instr.c */
-void TRACE_msg_set_task_category(m_task_t task, const char *category);
-void TRACE_msg_task_create(m_task_t task);
-void TRACE_msg_task_execute_start(m_task_t task);
-void TRACE_msg_task_execute_end(m_task_t task);
-void TRACE_msg_task_destroy(m_task_t task);
+void TRACE_msg_set_task_category(msg_task_t task, const char *category);
+void TRACE_msg_task_create(msg_task_t task);
+void TRACE_msg_task_execute_start(msg_task_t task);
+void TRACE_msg_task_execute_end(msg_task_t task);
+void TRACE_msg_task_destroy(msg_task_t task);
 void TRACE_msg_task_get_start(void);
-void TRACE_msg_task_get_end(double start_time, m_task_t task);
-int TRACE_msg_task_put_start(m_task_t task);    //returns TRUE if the task_put_end must be called
+void TRACE_msg_task_get_end(double start_time, msg_task_t task);
+int TRACE_msg_task_put_start(msg_task_t task);    //returns TRUE if the task_put_end must be called
 void TRACE_msg_task_put_end(void);
 
 /* declaration of instrumentation functions from msg_process_instr.c */
-char *instr_process_id (m_process_t proc, char *str, int len);
+char *instr_process_id (msg_process_t proc, char *str, int len);
 char *instr_process_id_2 (const char *process_name, int process_pid, char *str, int len);
-void TRACE_msg_process_change_host(m_process_t process, m_host_t old_host,
-                                   m_host_t new_host);
-void TRACE_msg_process_create (const char *process_name, int process_pid, m_host_t host);
-void TRACE_msg_process_kill(m_process_t process);
-void TRACE_msg_process_suspend(m_process_t process);
-void TRACE_msg_process_resume(m_process_t process);
-void TRACE_msg_process_sleep_in(m_process_t process);   //called from msg/gos.c
-void TRACE_msg_process_sleep_out(m_process_t process);
-void TRACE_msg_process_end(m_process_t process);
+void TRACE_msg_process_change_host(msg_process_t process, msg_host_t old_host,
+                                   msg_host_t new_host);
+void TRACE_msg_process_create (const char *process_name, int process_pid, msg_host_t host);
+void TRACE_msg_process_destroy (const char *process_name, int process_pid, msg_host_t host);
+void TRACE_msg_process_kill(msg_process_t process);
+void TRACE_msg_process_suspend(msg_process_t process);
+void TRACE_msg_process_resume(msg_process_t process);
+void TRACE_msg_process_sleep_in(msg_process_t process);   //called from msg/gos.c
+void TRACE_msg_process_sleep_out(msg_process_t process);
+void TRACE_msg_process_end(msg_process_t process);
 
 /* from surf_instr.c */
 void TRACE_surf_alloc(void);
@@ -146,6 +171,9 @@ void TRACE_smpi_init(int rank);
 void TRACE_smpi_finalize(int rank);
 void TRACE_smpi_collective_in(int rank, int root, const char *operation);
 void TRACE_smpi_collective_out(int rank, int root, const char *operation);
+void TRACE_smpi_computing_init(int rank);
+void TRACE_smpi_computing_out(int rank);
+void TRACE_smpi_computing_in(int rank);
 void TRACE_smpi_ptp_in(int rank, int src, int dst, const char *operation);
 void TRACE_smpi_ptp_out(int rank, int src, int dst, const char *operation);
 void TRACE_smpi_send(int rank, int src, int dst);
@@ -160,19 +188,29 @@ int TRACE_platform(void);
 int TRACE_is_configured(void);
 int TRACE_smpi_is_enabled(void);
 int TRACE_smpi_is_grouped(void);
+int TRACE_smpi_is_computing(void);
 int TRACE_categorized (void);
 int TRACE_uncategorized (void);
 int TRACE_msg_process_is_enabled(void);
 int TRACE_buffer (void);
 int TRACE_onelink_only (void);
 int TRACE_disable_destroy (void);
+int TRACE_basic (void);
+char *TRACE_get_comment (void);
+char *TRACE_get_comment_file (void);
 char *TRACE_get_filename(void);
 char *TRACE_get_triva_uncat_conf (void);
 char *TRACE_get_triva_cat_conf (void);
+char *TRACE_get_viva_uncat_conf (void);
+char *TRACE_get_viva_cat_conf (void);
 void TRACE_global_init(int *argc, char **argv);
 void TRACE_help(int detailed);
 void TRACE_generate_triva_uncat_conf (void);
 void TRACE_generate_triva_cat_conf (void);
+void TRACE_generate_viva_uncat_conf (void);
+void TRACE_generate_viva_cat_conf (void);
+void instr_pause_tracing (void);
+void instr_resume_tracing (void);
 
 /* from resource_utilization.c */
 void TRACE_surf_host_set_utilization(const char *resource,
@@ -208,7 +246,7 @@ void PJ_type_alloc (void);
 void PJ_type_release (void);
 type_t PJ_type_get_root (void);
 type_t PJ_type_container_new (const char *name, type_t father);
-type_t PJ_type_event_new (const char *name, const char *color, type_t father);
+type_t PJ_type_event_new (const char *name, type_t father);
 type_t PJ_type_variable_new (const char *name, const char *color, type_t father);
 type_t PJ_type_link_new (const char *name, type_t father, type_t source, type_t dest);
 type_t PJ_type_state_new (const char *name, type_t father);
@@ -219,6 +257,7 @@ void PJ_type_free_all (void);
 
 /* instr_paje_values.c */
 val_t PJ_value_new (const char *name, const char *color, type_t father);
+val_t PJ_value_get_or_new (const char *name, const char *color, type_t father);
 val_t PJ_value_get (const char *name, const type_t father);
 void PJ_value_free (val_t value);
 
