@@ -6,6 +6,7 @@
 #ifndef MC_PRIVATE_H
 #define MC_PRIVATE_H
 
+#include "simgrid_config.h"
 #include <stdio.h>
 #include <sys/mman.h>
 #include "mc/mc.h"
@@ -19,6 +20,7 @@
 #include "xbt/hash.h"
 #include "msg/msg.h"
 #include "msg/datatypes.h"
+#include "xbt/strbuff.h"
 
 /****************************** Snapshots ***********************************/
 
@@ -32,15 +34,20 @@ typedef struct s_mc_mem_region{
 typedef struct s_mc_snapshot{
   unsigned int num_reg;
   mc_mem_region_t *regions;
+  xbt_dynar_t stacks;
 } s_mc_snapshot_t, *mc_snapshot_t;
 
-extern char *prog_name;
+typedef struct s_mc_snapshot_stack{
+  xbt_strbuff_t local_variables;
+  void *stack_pointer;
+}s_mc_snapshot_stack_t, *mc_snapshot_stack_t;
 
 void MC_take_snapshot(mc_snapshot_t);
 void MC_take_snapshot_liveness(mc_snapshot_t s);
 void MC_restore_snapshot(mc_snapshot_t);
 void MC_free_snapshot(mc_snapshot_t);
 
+
 /********************************* MC Global **********************************/
 extern double *mc_time;
 
@@ -49,16 +56,13 @@ extern double *mc_time;
 #define MAX_DEPTH_LIVENESS 500
 
 int MC_deadlock_check(void);
-void MC_replay(xbt_fifo_t stack);
+void MC_replay(xbt_fifo_t stack, int start);
 void MC_replay_liveness(xbt_fifo_t stack, int all_stack);
 void MC_wait_for_requests(void);
 void MC_get_enabled_processes();
 void MC_show_deadlock(smx_simcall_t req);
-void MC_show_deadlock_stateful(smx_simcall_t req);
-void MC_show_stack_safety_stateless(xbt_fifo_t stack);
-void MC_dump_stack_safety_stateless(xbt_fifo_t stack);
-void MC_show_stack_safety_stateful(xbt_fifo_t stack);
-void MC_dump_stack_safety_stateful(xbt_fifo_t stack);
+void MC_show_stack_safety(xbt_fifo_t stack);
+void MC_dump_stack_safety(xbt_fifo_t stack);
 
 /********************************* Requests ***********************************/
 int MC_request_depend(smx_simcall_t req1, smx_simcall_t req2);
@@ -95,6 +99,7 @@ typedef struct mc_state {
   s_smx_simcall_t executed_req;         /* The executed request of the state */
   int req_num;                      /* The request number (in the case of a
                                        multi-request like waitany ) */
+  mc_snapshot_t system_state;      /* Snapshot of system state */
 } s_mc_state_t, *mc_state_t;
 
 extern xbt_fifo_t mc_stack_safety_stateless;
@@ -137,22 +142,24 @@ void MC_print_statistics_pairs(mc_stats_pair_t);
 
 extern void *std_heap;
 extern void *raw_heap;
-#define STD_HEAP_SIZE   20480000        /* Maximum size of the system's heap */
+
 
 /* FIXME: Horrible hack! because the mmalloc library doesn't provide yet of */
 /* an API to query about the status of a heap, we simply call mmstats and */
 /* because I now how does structure looks like, then I redefine it here */
 
-struct mstats {
-  size_t bytes_total;           /* Total size of the heap. */
-  size_t chunks_used;           /* Chunks allocated by the user. */
-  size_t bytes_used;            /* Byte total of user-allocated chunks. */
-  size_t chunks_free;           /* Chunks in the free list. */
-  size_t bytes_free;            /* Byte total of chunks in the free list. */
-};
+/* struct mstats { */
+/*   size_t bytes_total;           /\* Total size of the heap. *\/ */
+/*   size_t chunks_used;           /\* Chunks allocated by the user. *\/ */
+/*   size_t bytes_used;            /\* Byte total of user-allocated chunks. *\/ */
+/*   size_t chunks_free;           /\* Chunks in the free list. *\/ */
+/*   size_t bytes_free;            /\* Byte total of chunks in the free list. *\/ */
+/* }; */
 
 #define MC_SET_RAW_MEM    mmalloc_set_current_heap(raw_heap)
-#define MC_UNSET_RAW_MEM    mmalloc_set_current_heap(std_heap)
+#define MC_UNSET_RAW_MEM  mmalloc_set_current_heap(std_heap)
+
+extern int raw_mem_set;
 
 /******************************* MEMORY MAPPINGS ***************************/
 /* These functions and data structures implements a binary interface for   */
@@ -181,34 +188,37 @@ typedef struct s_memory_map {
 } s_memory_map_t, *memory_map_t;
 
 memory_map_t get_memory_map(void);
+void free_memory_map(memory_map_t map);
+void get_libsimgrid_plt_section(void);
+void get_binary_plt_section(void);
 
+extern void *start_data_libsimgrid;
+
+/********************************** DPOR for safety  **************************************/
+typedef enum {
+  e_mc_reduce_unset,
+  e_mc_reduce_none,
+  e_mc_reduce_dpor
+} e_mc_reduce_t;
+
+extern e_mc_reduce_t mc_reduce_kind;
 
-/********************************** DPOR for safety (stateless) **************************************/
 void MC_dpor_init(void);
 void MC_dpor(void);
 void MC_dpor_exit(void);
-
-/***** DPOR for safety (stateful) **** */
-
-typedef struct s_mc_state_with_snapshot{
-  mc_snapshot_t system_state;
-  mc_state_t graph_state;
-}s_mc_state_ws_t, *mc_state_ws_t;
-
-extern xbt_fifo_t mc_stack_safety_stateful;
-
-void MC_init_stateful(void);
-void MC_modelcheck_stateful(void);
-mc_state_ws_t new_state_ws(mc_snapshot_t s, mc_state_t gs);
-void MC_dpor_stateful_init(void);
-void MC_dpor_stateful(void);
-void MC_exit_stateful(void);
+void MC_init_safety(void);
 
 
 /********************************** Double-DFS for liveness property**************************************/
 
+extern xbt_fifo_t mc_stack_liveness;
 extern mc_snapshot_t initial_snapshot_liveness;
-extern xbt_automaton_t automaton;
+extern xbt_automaton_t _mc_property_automaton;
+extern int compare;
+extern void *start_plt_libsimgrid;
+extern void *end_plt_libsimgrid;
+extern void *start_plt_binary;
+extern void *end_plt_binary;
 
 typedef struct s_mc_pair{
   mc_snapshot_t system_state;
@@ -217,54 +227,21 @@ typedef struct s_mc_pair{
 }s_mc_pair_t, *mc_pair_t;
 
 typedef struct s_mc_pair_reached{
+  int nb;
   xbt_state_t automaton_state;
   xbt_dynar_t prop_ato;
   mc_snapshot_t system_state;
-  //xbt_dict_t rdv_points;
 }s_mc_pair_reached_t, *mc_pair_reached_t;
 
-typedef struct s_mc_pair_visited{
-  xbt_state_t automaton_state;
-  xbt_dynar_t prop_ato;
-  mc_snapshot_t system_state;
-  int search_cycle;
-}s_mc_pair_visited_t, *mc_pair_visited_t;
-
-typedef struct s_mc_pair_visited_hash{
-  xbt_state_t automaton_state;
-  xbt_dynar_t prop_ato;
-  unsigned int *hash_regions;
-  int search_cycle;
-}s_mc_pair_visited_hash_t, *mc_pair_visited_hash_t;
-
-typedef struct s_mc_pair_reached_hash{
-  xbt_state_t automaton_state;
-  xbt_dynar_t prop_ato;
-  unsigned int *hash_regions;
-}s_mc_pair_reached_hash_t, *mc_pair_reached_hash_t;
-
-
-
 int MC_automaton_evaluate_label(xbt_exp_label_t l);
 mc_pair_t new_pair(mc_snapshot_t sn, mc_state_t sg, xbt_state_t st);
 
 int reached(xbt_state_t st);
 void set_pair_reached(xbt_state_t st);
-int reached_hash(xbt_state_t st);
-void set_pair_reached_hash(xbt_state_t st);
 int snapshot_compare(mc_snapshot_t s1, mc_snapshot_t s2);
-int data_program_region_compare(void *d1, void *d2, size_t size);
-int data_libsimgrid_region_compare(void *d1, void *d2, size_t size);
 void MC_pair_delete(mc_pair_t pair);
 void MC_exit_liveness(void);
 mc_state_t MC_state_pair_new(void);
-int visited(xbt_state_t st, int search_cycle);
-void set_pair_visited(xbt_state_t st, int search_cycle);
-int visited_hash(xbt_state_t st, int search_cycle);
-void set_pair_visited_hash(xbt_state_t st, int search_cycle);
-unsigned int hash_region(char *str, int str_len);
-int rdv_points_compare(xbt_dict_t d1, xbt_dict_t d2);
-
 
 /* **** Double-DFS stateless **** */
 
@@ -274,8 +251,6 @@ typedef struct s_mc_pair_stateless{
   int requests;
 }s_mc_pair_stateless_t, *mc_pair_stateless_t;
 
-extern xbt_fifo_t mc_stack_liveness;
-
 mc_pair_stateless_t new_pair_stateless(mc_state_t sg, xbt_state_t st, int r);
 void MC_ddfs_init(void);
 void MC_ddfs(int search_cycle);
@@ -283,5 +258,111 @@ void MC_show_stack_liveness(xbt_fifo_t stack);
 void MC_dump_stack_liveness(xbt_fifo_t stack);
 void MC_pair_stateless_delete(mc_pair_stateless_t pair);
 
+/********************************** Configuration of MC **************************************/
+extern xbt_fifo_t mc_stack_safety;
+
+extern int _surf_mc_checkpoint;
+extern char* _surf_mc_property_file;
+
+/****** Core dump ******/
+
+int create_dump(int pair);
+
+/****** Local variables with DWARF ******/
+
+typedef enum {
+  e_dw_loclist,
+  e_dw_register,
+  e_dw_bregister_op,
+  e_dw_lit,
+  e_dw_fbregister_op,
+  e_dw_piece,
+  e_dw_arithmetic,
+  e_dw_plus_uconst,
+  e_dw_compose,
+  e_dw_deref,
+  e_dw_uconstant,
+  e_dw_sconstant,
+  e_dw_unsupported
+} e_dw_location_type;
+
+typedef struct s_dw_location{
+  e_dw_location_type type;
+  union{
+    
+    xbt_dynar_t loclist;
+    
+    int reg;
+    
+    struct{
+      unsigned int reg;
+      int offset;
+    }breg_op;
+
+    unsigned int lit;
+
+    int fbreg_op;
+
+    int piece;
+
+    unsigned short int deref_size;
+
+    xbt_dynar_t compose;
+
+    char *arithmetic;
+
+    struct{
+      int bytes;
+      long unsigned int value;
+    }uconstant;
+
+    struct{
+      int bytes;
+      long signed int value;
+    }sconstant;
+
+    unsigned int plus_uconst;
+
+  }location;
+}s_dw_location_t, *dw_location_t;
+
+typedef struct s_dw_location_entry{
+  long lowpc;
+  long highpc;
+  dw_location_t location;
+}s_dw_location_entry_t, *dw_location_entry_t;
+
+typedef struct s_dw_local_variable{
+  char *name;
+  dw_location_t location;
+}s_dw_local_variable_t, *dw_local_variable_t;
+
+typedef struct s_dw_frame{
+  char *name;
+  void *low_pc;
+  void *high_pc;
+  dw_location_t frame_base;
+  xbt_dict_t variables;
+  unsigned long int start;
+  unsigned long int end;
+}s_dw_frame_t, *dw_frame_t;
+
+/* FIXME : implement free functions for each structure */
+
+extern xbt_dict_t mc_local_variables;
+
+typedef struct s_variable_value{
+  char *type;
+  
+  union{
+    void *address;
+    long int res;
+  }value;
+}s_variable_value_t, *variable_value_t;
+
+void MC_get_local_variables(const char *elf_file, xbt_dict_t location_list, xbt_dict_t *variables);
+void print_local_variables(xbt_dict_t list);
+char *get_libsimgrid_path(void);
+xbt_dict_t MC_get_location_list(const char *elf_file);
 
 #endif
