@@ -11,36 +11,34 @@
 #include "xbt/log.h"
 #include "xbt/virtu.h"
 #include "xbt/ex.h"             /* ex_backtrace_display */
+#include "xbt/replay.h"
 
 XBT_LOG_NEW_CATEGORY(msg, "All MSG categories");
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(msg_kernel, msg,
                                 "Logging specific to MSG (kernel)");
 
 MSG_Global_t msg_global = NULL;
+static void MSG_exit(void);
 
 /********************************* MSG **************************************/
 
-/** \ingroup msg_simulation
- * \brief Initialize some MSG internal data.
+/**
+ * \ingroup msg_simulation
+ * \brief Initialize MSG with less verifications
+ * You should use the MSG_init() function instead. Failing to do so may turn into PEBKAC some day. You've been warned.
  */
-void MSG_global_init_args(int *argc, char **argv)
-{
-  MSG_global_init(argc, argv);
-}
+void MSG_init_nocheck(int *argc, char **argv) {
 
-/** \ingroup msg_simulation
- * \brief Initialize some MSG internal data.
- */
-void MSG_global_init(int *argc, char **argv)
-{
 #ifdef HAVE_TRACING
   TRACE_global_init(argc, argv);
 #endif
 
   xbt_getpid = MSG_process_self_PID;
   if (!msg_global) {
-    SIMIX_global_init(argc, argv);
+    s_msg_vm_t vm; // to compute the offset
 
+    SIMIX_global_init(argc, argv);
+    
     msg_global = xbt_new0(s_MSG_Global_t, 1);
 
 #ifdef MSG_USE_DEPRECATED
@@ -50,20 +48,30 @@ void MSG_global_init(int *argc, char **argv)
     msg_global->sent_msg = 0;
     msg_global->task_copy_callback = NULL;
     msg_global->process_data_cleanup = NULL;
+    msg_global->vms = xbt_swag_new(xbt_swag_offset(vm,all_vms_hookup));
 
     /* initialization of the action module */
     _MSG_action_init();
 
     SIMIX_function_register_process_create(MSG_process_create_from_SIMIX);
     SIMIX_function_register_process_cleanup(MSG_process_cleanup_from_SIMIX);
-    SIMIX_function_register_process_kill(MSG_process_kill_from_SIMIX);
+
+    sg_platf_postparse_add_cb(MSG_post_create_environment);
   }
+  
+  if(MC_is_active()){
+    /* Ignore total amount of messages sent during the simulation for heap comparison */
+    MC_ignore(&(msg_global->sent_msg), sizeof(msg_global->sent_msg));
+  }
+
 #ifdef HAVE_TRACING
   TRACE_start();
 #endif
 
   XBT_DEBUG("ADD MSG LEVELS");
   MSG_HOST_LEVEL = xbt_lib_add_level(host_lib, (void_f_pvoid_t) __MSG_host_destroy);
+
+  atexit(MSG_exit);
 }
 
 #ifdef MSG_USE_DEPRECATED
@@ -75,7 +83,7 @@ void MSG_global_init(int *argc, char **argv)
    parameter.
  * \param number the number of channel in the simulation. It has to be >0
  */
-MSG_error_t MSG_set_channel_number(int number)
+msg_error_t MSG_set_channel_number(int number)
 {
   XBT_WARN("DEPRECATED! Please use aliases instead");
   xbt_assert((msg_global)
@@ -105,47 +113,15 @@ int MSG_get_channel_number(void)
 /** \ingroup msg_simulation
  * \brief Launch the MSG simulation
  */
-MSG_error_t MSG_main(void)
+msg_error_t MSG_main(void)
 {
   /* Clean IO before the run */
   fflush(stdout);
   fflush(stderr);
 
-  if (MC_IS_ENABLED) {
-    MC_modelcheck();
-  }
-  else {
-    SIMIX_run();
-  }
-  return MSG_OK;
-}
-
-MSG_error_t MSG_main_stateful(void)
-{
-  /* Clean IO before the run */
-  fflush(stdout);
-  fflush(stderr);
-
-  if (MC_IS_ENABLED) {
-    MC_modelcheck_stateful();
-  }
-  else {
-    SIMIX_run();
-  }
-  return MSG_OK;
-}
-
-
-MSG_error_t MSG_main_liveness(xbt_automaton_t a, char *prgm)
-{
-  /* Clean IO before the run */
-  fflush(stdout);
-  fflush(stderr);
-
-  if (MC_IS_ENABLED) {
-    MC_modelcheck_liveness(a, prgm);
-  }
-  else {
+  if (MC_is_active()) {
+    MC_do_the_modelcheck_for_real();
+  } else {
     SIMIX_run();
   }
   return MSG_OK;
@@ -171,19 +147,14 @@ int MSG_process_killall(int reset_PIDs)
 
 }
 
-/** \ingroup msg_simulation
- * \brief Clean the MSG simulation
- */
-MSG_error_t MSG_clean(void)
-{
-  XBT_DEBUG("Closing MSG");
+static void MSG_exit(void) {
+  if (msg_global==NULL)
+    return;
 
 #ifdef HAVE_TRACING
   TRACE_surf_release();
 #endif
 
-  MSG_process_killall(0);
-
   /* initialization of the action module */
   _MSG_action_exit();
 
@@ -191,12 +162,9 @@ MSG_error_t MSG_clean(void)
   TRACE_end();
 #endif
 
-  SIMIX_clean();
-
+  xbt_swag_free(msg_global->vms);
   free(msg_global);
   msg_global = NULL;
-
-  return MSG_OK;
 }
 
 
@@ -212,3 +180,9 @@ unsigned long int MSG_get_sent_msg()
 {
   return msg_global->sent_msg;
 }
+
+#ifdef MSG_USE_DEPRECATED
+msg_error_t MSG_clean(void) {
+  return MSG_OK;
+}
+#endif
