@@ -16,7 +16,6 @@
 #include "xbt/dict.h"
 #include "xbt/config.h"
 #include "instr/instr_private.h"
-
 SG_BEGIN_DECL()
 
 /**************** datatypes **********************************/
@@ -28,9 +27,9 @@ typedef struct simdata_task {
   smx_action_t comm;            /* SIMIX modeling of communication */
   double message_size;          /* Data size */
   double computation_amount;    /* Computation size */
-  m_process_t sender;
-  m_process_t receiver;
-  m_host_t source;
+  msg_process_t sender;
+  msg_process_t receiver;
+  msg_host_t source;
   double priority;
   double rate;
   int isused;  /* Indicates whether the task is used in SIMIX currently */
@@ -58,18 +57,21 @@ typedef struct simdata_gpu_task {
 /******************************* Process *************************************/
 
 typedef struct simdata_process {
-  m_host_t m_host;              /* the host on which the process is running */
+  msg_host_t m_host;              /* the host on which the process is running */
   int PID;                      /* used for debugging purposes */
   int PPID;                     /* The parent PID */
-  m_host_t put_host;            /* used for debugging purposes */
+  msg_host_t put_host;            /* used for debugging purposes */
 #ifdef MSG_USE_DEPRECATED
   m_channel_t put_channel;      /* used for debugging purposes */
 #endif
   smx_action_t waiting_action;
-  m_task_t waiting_task;
+  msg_task_t waiting_task;
   char **argv;                  /* arguments table if any */
   int argc;                     /* arguments number if any */
-  MSG_error_t last_errno;       /* the last value returned by a MSG_function */
+  msg_error_t last_errno;       /* the last value returned by a MSG_function */
+
+  msg_vm_t vm;                 /* virtual machine the process is in */
+
   void* data;                   /* user data */
 } s_simdata_process_t, *simdata_process_t;
 
@@ -77,7 +79,7 @@ typedef struct process_arg {
   const char *name;
   xbt_main_func_t code;
   void *data;
-  m_host_t m_host;
+  msg_host_t m_host;
   int argc;
   char **argv;
   double kill_time;
@@ -85,11 +87,24 @@ typedef struct process_arg {
 
 typedef struct msg_comm {
   smx_action_t s_comm;          /* SIMIX communication object encapsulated (the same for both processes) */
-  m_task_t task_sent;           /* task sent (NULL for the receiver) */
-  m_task_t *task_received;      /* where the task will be received (NULL for the sender) */
-  MSG_error_t status;           /* status of the communication once finished */
+  msg_task_t task_sent;           /* task sent (NULL for the receiver) */
+  msg_task_t *task_received;      /* where the task will be received (NULL for the sender) */
+  msg_error_t status;           /* status of the communication once finished */
 } s_msg_comm_t;
 
+typedef enum {
+  msg_vm_state_suspended, msg_vm_state_running, msg_vm_state_migrating
+} e_msg_vm_state_t;
+
+typedef struct msg_vm {
+  s_xbt_swag_hookup_t all_vms_hookup;
+  s_xbt_swag_hookup_t host_vms_hookup;
+  xbt_dynar_t processes;
+  e_msg_vm_state_t state;
+  msg_host_t location;
+  int coreAmount;
+} s_msg_vm_t;
+
 /************************** Global variables ********************************/
 typedef struct MSG_Global {
   xbt_fifo_t host;
@@ -99,8 +114,9 @@ typedef struct MSG_Global {
   int PID;
   int session;
   unsigned long int sent_msg;   /* Total amount of messages sent during the simulation */
-  void (*task_copy_callback) (m_task_t task, m_process_t src, m_process_t dst);
+  void (*task_copy_callback) (msg_task_t task, msg_process_t src, msg_process_t dst);
   void_f_pvoid_t process_data_cleanup;
+  xbt_swag_t vms;
 } s_MSG_Global_t, *MSG_Global_t;
 
 /*extern MSG_Global_t msg_global;*/
@@ -121,8 +137,8 @@ XBT_PUBLIC_DATA(MSG_Global_t) msg_global;
 #  define MSG_RETURN(val) return(val)
 #endif
 
-m_host_t __MSG_host_create(smx_host_t workstation);
-void __MSG_host_destroy(m_host_t host);
+msg_host_t __MSG_host_create(smx_host_t workstation);
+void __MSG_host_destroy(msg_host_t host);
 
 void __MSG_display_process_status(void);
 
@@ -130,12 +146,13 @@ void MSG_process_cleanup_from_SIMIX(smx_process_t smx_proc);
 void MSG_process_create_from_SIMIX(smx_process_t *process, const char *name,
                                    xbt_main_func_t code, void *data,
                                    const char *hostname, double kill_time,  int argc,
-                                   char **argv, xbt_dict_t properties);
-void MSG_process_kill_from_SIMIX(smx_process_t p);
+                                   char **argv, xbt_dict_t properties, int auto_restart);
 void MSG_comm_copy_data_from_SIMIX(smx_action_t comm, void* buff, size_t buff_size);
 
 void _MSG_action_init(void);
 void _MSG_action_exit(void);
 
+void MSG_post_create_environment(void);
+
 SG_END_DECL()
 #endif
