@@ -97,6 +97,118 @@ SD_task_t SD_task_create(const char *name, void *data, double amount)
   return task;
 }
 
+static XBT_INLINE SD_task_t SD_task_create_sized(const char *name,
+                                                 void *data, double amount,
+                                                 int ws_count)
+{
+  SD_task_t task = SD_task_create(name, data, amount);
+  task->communication_amount = xbt_new0(double, ws_count * ws_count);
+  task->computation_amount = xbt_new0(double, ws_count);
+  task->workstation_nb = ws_count;
+  task->workstation_list = xbt_new0(SD_workstation_t, ws_count);
+  return task;
+}
+
+/** @brief create a end-to-end communication task that can then be auto-scheduled
+ *
+ * Auto-scheduling mean that the task can be used with SD_task_schedulev(). This
+ * allows to specify the task costs at creation, and decouple them from the
+ * scheduling process where you just specify which resource should deliver the
+ * mandatory power.
+ *
+ * A end-to-end communication must be scheduled on 2 hosts, and the amount
+ * specified at creation is sent from hosts[0] to hosts[1].
+ */
+SD_task_t SD_task_create_comm_e2e(const char *name, void *data,
+                                  double amount)
+{
+  SD_task_t res = SD_task_create_sized(name, data, amount, 2);
+  res->communication_amount[2] = amount;
+  res->kind = SD_TASK_COMM_E2E;
+  return res;
+}
+
+/** @brief create a sequential computation task that can then be auto-scheduled
+ *
+ * Auto-scheduling mean that the task can be used with SD_task_schedulev(). This
+ * allows to specify the task costs at creation, and decouple them from the
+ * scheduling process where you just specify which resource should deliver the
+ * mandatory power.
+ *
+ * A sequential computation must be scheduled on 1 host, and the amount
+ * specified at creation to be run on hosts[0].
+ *
+ * \param name the name of the task (can be \c NULL)
+ * \param data the user data you want to associate with the task (can be \c NULL)
+ * \param amount amount of compute work to be done by the task
+ * \return the new SD_TASK_COMP_SEQ typed task
+ */
+SD_task_t SD_task_create_comp_seq(const char *name, void *data,
+                                  double amount)
+{
+  SD_task_t res = SD_task_create_sized(name, data, amount, 1);
+  res->computation_amount[0] = amount;
+  res->kind = SD_TASK_COMP_SEQ;
+  return res;
+}
+
+/** @brief create a parallel computation task that can then be auto-scheduled
+ *
+ * Auto-scheduling mean that the task can be used with SD_task_schedulev(). This
+ * allows to specify the task costs at creation, and decouple them from the
+ * scheduling process where you just specify which resource should deliver the
+ * mandatory power.
+ *
+ * A parallel computation can be scheduled on any number of host.
+ * The underlying speedup model is Amdahl's law. 
+ * To be auto-scheduled, \see SD_task_distribute_comp_amdhal has to be called 
+ * first.
+ * \param name the name of the task (can be \c NULL)
+ * \param data the user data you want to associate with the task (can be \c NULL)
+ * \param amount amount of compute work to be done by the task
+ * \param alpha purely serial fraction of the work to be done (in [0.;1.[)
+ * \return the new task
+ */
+SD_task_t SD_task_create_comp_par_amdahl(const char *name, void *data,
+                                  double amount, double alpha)
+{
+  xbt_assert(alpha < 1. && alpha >= 0.,
+              "Invalid parameter: alpha must be in [0.;1.[");
+  
+  SD_task_t res = SD_task_create(name, data, amount);
+  res->alpha = alpha;
+  res->kind = SD_TASK_COMP_PAR_AMDAHL;
+  return res;
+}
+
+/** @brief create a complex data redistribution task that can then be 
+ * auto-scheduled
+ *
+ * Auto-scheduling mean that the task can be used with SD_task_schedulev(). 
+ * This allows to specify the task costs at creation, and decouple them from 
+ * the scheduling process where you just specify which resource should 
+ * communicate. 
+ *
+ * A data redistribution can be scheduled on any number of host.
+ * The assumed distribution is a 1D block distribution. Each host owns the same
+ * share of the \see amount. 
+ * To be auto-scheduled, \see SD_task_distribute_comm_mxn_1d_block has to be 
+ * called first.
+ * \param name the name of the task (can be \c NULL)
+ * \param data the user data you want to associate with the task (can be
+ * \c NULL)
+ * \param amount amount of data to redistribute by the task
+ * \return the new task
+ */
+SD_task_t SD_task_create_comm_par_mxn_1d_block(const char *name, void *data,
+											   double amount)
+{
+  SD_task_t res = SD_task_create(name, data, amount);
+  res->workstation_list=NULL;
+  res->kind = SD_TASK_COMM_PAR_MXN_1D_BLOCK;
+  return res;
+}
+
 /**
  * \brief Destroys a task.
  *
@@ -127,13 +239,13 @@ void SD_task_destroy(SD_task_t task)
   xbt_free(task->communication_amount);
   xbt_free(task->computation_amount);
 
-  xbt_mallocator_release(sd_global->task_mallocator,task);
-  sd_global->task_number--;
-
 #ifdef HAVE_TRACING
   if (task->category) xbt_free(task->category);
 #endif
 
+  xbt_mallocator_release(sd_global->task_mallocator,task);
+  sd_global->task_number--;
+
   XBT_DEBUG("Task destroyed.");
 }
 
@@ -164,6 +276,28 @@ void SD_task_set_data(SD_task_t task, void *data)
   task->data = data;
 }
 
+/**
+ * \brief Sets the rate of a task
+ *
+ * This will change the network bandwidth a task can use. This rate
+ * depends on both the nominal bandwidth on the route onto which the task is
+ * scheduled (\see SD_task_get_current_bandwidth) and the amount of data to
+ * transfer.
+ *
+ * To divide the nominal bandwidth by 2, the rate then has to be :
+ *    rate = bandwidth/(2*amount)
+ *
+ * \param task a \see SD_TASK_COMM_E2E task (end-to-end communication)
+ * \param rate the new rate you want to associate with this task.
+ */
+void SD_task_set_rate(SD_task_t task, double rate)
+{
+  xbt_assert(task->kind == SD_TASK_COMM_E2E,
+             "The rate can be modified for end-to-end communications only.");
+
+  task->rate = rate;
+}
+
 /**
  * \brief Returns the state of a task
  *
@@ -367,6 +501,9 @@ void SD_task_dump(SD_task_t task)
     case SD_TASK_COMP_SEQ:
       XBT_INFO("  - kind: sequential computation");
       break;
+    case SD_TASK_COMP_PAR_AMDAHL:
+      XBT_INFO("  - kind: parallel computation following Amdahl's law");
+      break;
     default:
       XBT_INFO("  - (unknown kind %d)", task->kind);
     }
@@ -450,7 +587,7 @@ void SD_task_dependency_add(const char *name, void *data, SD_task_t src,
       && !__SD_task_is_scheduled_or_runnable(src) && !__SD_task_is_running(src))
     THROWF(arg_error, 0,
            "Task '%s' must be SD_NOT_SCHEDULED, SD_SCHEDULABLE, SD_SCHEDULED or SD_RUNNABLE"
-	   " or SD_RUNNING",
+     " or SD_RUNNING",
            SD_task_get_name(src));
 
   if (!__SD_task_is_not_scheduled(dst) && !__SD_task_is_schedulable(dst)
@@ -836,8 +973,12 @@ void SD_task_unschedule(SD_task_t task)
            SD_task_get_name(task));
 
   if (__SD_task_is_scheduled_or_runnable(task)  /* if the task is scheduled or runnable */
-      &&task->kind == SD_TASK_NOT_TYPED)        /* Don't free scheduling data for typed tasks */
+      && ((task->kind == SD_TASK_COMP_PAR_AMDAHL) ||
+          (task->kind == SD_TASK_COMM_PAR_MXN_1D_BLOCK))) { /* Don't free scheduling data for typed tasks */
     __SD_task_destroy_scheduling_data(task);
+    task->workstation_list=NULL;
+    task->workstation_nb = 0;
+  }
 
   if (__SD_task_is_running(task))       /* the task should become SD_FAILED */
     surf_workstation_model->action_cancel(task->surf_action);
@@ -902,13 +1043,15 @@ void __SD_task_really_run(SD_task_t task)
 
   /* start the task */
 
-  /* we have to create a Surf workstation array instead of the SimDag workstation array */
+  /* we have to create a Surf workstation array instead of the SimDag
+   * workstation array */
   surf_workstations = xbt_new(void *, task->workstation_nb);
 
   for (i = 0; i < task->workstation_nb; i++)
     surf_workstations[i] = task->workstation_list[i]->surf_workstation;
 
-  /* It's allowed to pass a NULL vector as cost to mean vector of 0.0 (easing user's life). Let's deal with it */
+  /* It's allowed to pass a NULL vector as cost to mean vector of 0.0 (easing
+   * user's life). Let's deal with it */
 #define cost_or_zero(array,pos) ((array)?(array)[pos]:0.0)
 
   task->surf_action = NULL;
@@ -1182,7 +1325,9 @@ void __SD_task_just_done(SD_task_t task)
   xbt_free(candidates);
 }
 
-/* Remove all dependencies associated with a task. This function is called when the task is destroyed.
+/* 
+ * Remove all dependencies associated with a task. This function is called 
+ * when the task is destroyed.
  */
 static void __SD_task_remove_dependencies(SD_task_t task)
 {
@@ -1236,56 +1381,28 @@ double SD_task_get_finish_time(SD_task_t task)
   else
     return task->finish_time;
 }
-
-static XBT_INLINE SD_task_t SD_task_create_sized(const char *name,
-                                                 void *data, double amount,
-                                                 int ws_count)
+/** @brief Blah
+ *
+ */
+void SD_task_distribute_comp_amdhal(SD_task_t task, int ws_count)
 {
-  SD_task_t task = SD_task_create(name, data, amount);
-  task->communication_amount = xbt_new0(double, ws_count * ws_count);
+  int i;
+  xbt_assert(task->kind == SD_TASK_COMP_PAR_AMDAHL,
+              "Task %s is not a SD_TASK_COMP_PAR_AMDAHL typed task."
+              "Cannot use this function.",
+              SD_task_get_name(task));  
+              
   task->computation_amount = xbt_new0(double, ws_count);
+  task->communication_amount = xbt_new0(double, ws_count * ws_count);
   task->workstation_nb = ws_count;
   task->workstation_list = xbt_new0(SD_workstation_t, ws_count);
-  return task;
-}
+  
+  for(i=0;i<ws_count;i++){
+    task->computation_amount[i] = 
+      (task->alpha + (1 - task->alpha)/ws_count) * task->amount;
+  }
+} 
 
-/** @brief create a end-to-end communication task that can then be auto-scheduled
- *
- * Auto-scheduling mean that the task can be used with SD_task_schedulev(). This
- * allows to specify the task costs at creation, and decorelate them from the
- * scheduling process where you just specify which resource should deliver the
- * mandatory power.
- *
- * A end-to-end communication must be scheduled on 2 hosts, and the amount
- * specified at creation is sent from hosts[0] to hosts[1].
- */
-SD_task_t SD_task_create_comm_e2e(const char *name, void *data,
-                                  double amount)
-{
-  SD_task_t res = SD_task_create_sized(name, data, amount, 2);
-  res->communication_amount[2] = amount;
-  res->kind = SD_TASK_COMM_E2E;
-  return res;
-}
-
-/** @brief create a sequential computation task that can then be auto-scheduled
- *
- * Auto-scheduling mean that the task can be used with SD_task_schedulev(). This
- * allows to specify the task costs at creation, and decorelate them from the
- * scheduling process where you just specify which resource should deliver the
- * mandatory power.
- *
- * A sequential computation must be scheduled on 1 host, and the amount
- * specified at creation to be run on hosts[0].
- */
-SD_task_t SD_task_create_comp_seq(const char *name, void *data,
-                                  double amount)
-{
-  SD_task_t res = SD_task_create_sized(name, data, amount, 1);
-  res->computation_amount[0] = amount;
-  res->kind = SD_TASK_COMP_SEQ;
-  return res;
-}
 
 /** @brief Auto-schedules a task.
  *
@@ -1309,18 +1426,25 @@ SD_task_t SD_task_create_comp_seq(const char *name, void *data,
 void SD_task_schedulev(SD_task_t task, int count,
                        const SD_workstation_t * list)
 {
-  int i;
+  int i, j;
   SD_dependency_t dep;
   unsigned int cpt;
   xbt_assert(task->kind != 0,
               "Task %s is not typed. Cannot automatically schedule it.",
               SD_task_get_name(task));
   switch (task->kind) {
+  case SD_TASK_COMP_PAR_AMDAHL:
+    SD_task_distribute_comp_amdhal(task, count);
   case SD_TASK_COMM_E2E:
   case SD_TASK_COMP_SEQ:
     xbt_assert(task->workstation_nb == count,"Got %d locations, but were expecting %d locations",count,task->workstation_nb);
     for (i = 0; i < count; i++)
       task->workstation_list[i] = list[i];
+    if (SD_task_get_kind(task)== SD_TASK_COMP_SEQ && !task->computation_amount){
+      /*This task has failed and is rescheduled. Reset the computation amount*/
+      task->computation_amount = xbt_new0(double, 1);
+      task->computation_amount[0] = task->remains;
+    }
     SD_task_do_schedule(task);
     break;
   default:
@@ -1335,6 +1459,7 @@ void SD_task_schedulev(SD_task_t task, int count,
           task->communication_amount[2]);
 
   }
+
   /* Iterate over all childs and parent being COMM_E2E to say where I am located (and start them if runnable) */
   if (task->kind == SD_TASK_COMP_SEQ) {
     XBT_VERB("Schedule computation task %s on %s. It costs %.f flops",
@@ -1364,7 +1489,6 @@ void SD_task_schedulev(SD_task_t task, int count,
       SD_task_t after = dep->dst;
       if (after->kind == SD_TASK_COMM_E2E) {
         after->workstation_list[0] = task->workstation_list[0];
-        //J-N : Why did you comment on these line (this comment add a bug I think)?
         if (after->workstation_list[1]
             && (__SD_task_is_not_scheduled(after)
                 || __SD_task_is_schedulable(after))) {
@@ -1380,6 +1504,133 @@ void SD_task_schedulev(SD_task_t task, int count,
       }
     }
   }
+  /* Iterate over all childs and parent being MXN_1D_BLOC to say where I am located (and start them if runnable) */
+  if (task->kind == SD_TASK_COMP_PAR_AMDAHL) {
+    XBT_VERB("Schedule computation task %s on %d workstations. %.f flops"
+             " will be distributed following Amdahl'Law",
+          SD_task_get_name(task), task->workstation_nb,
+          task->computation_amount[0]);
+    xbt_dynar_foreach(task->tasks_before, cpt, dep) {
+      SD_task_t before = dep->src;
+      if (before->kind == SD_TASK_COMM_PAR_MXN_1D_BLOCK){
+        if (!before->workstation_list){
+          XBT_VERB("Sender side of Task %s is not scheduled yet. Fill the workstation list with receiver side",
+             SD_task_get_name(before));
+          before->workstation_list = xbt_new0(SD_workstation_t, count);
+          before->workstation_nb = count;
+          for (i=0;i<count;i++)
+            before->workstation_list[i] = task->workstation_list[i];
+        } else {
+          int src_nb, dst_nb;
+          double src_start, src_end, dst_start, dst_end;
+          src_nb = before->workstation_nb;
+          dst_nb = count;
+          before->workstation_list = (SD_workstation_t*) xbt_realloc(
+             before->workstation_list,
+             (before->workstation_nb+count)*sizeof(s_SD_workstation_t));
+          for(i=0; i<count; i++)
+            before->workstation_list[before->workstation_nb+i] =
+               task->workstation_list[i];
+
+          before->workstation_nb += count;
+
+          before->computation_amount = xbt_new0(double,
+                                                before->workstation_nb);
+          before->communication_amount = xbt_new0(double,
+                                                  before->workstation_nb*
+                                                  before->workstation_nb);
+
+          for(i=0;i<src_nb;i++){
+            src_start = i*before->amount/src_nb;
+            src_end = src_start + before->amount/src_nb;
+            for(j=0; j<dst_nb; j++){
+              dst_start = j*before->amount/dst_nb;
+              dst_end = dst_start + before->amount/dst_nb;
+              XBT_VERB("(%s->%s): (%.2f, %.2f)-> (%.2f, %.2f)",
+                  SD_workstation_get_name(before->workstation_list[i]),
+                  SD_workstation_get_name(before->workstation_list[src_nb+j]),
+                  src_start, src_end, dst_start, dst_end);
+              if ((src_end <= dst_start) || (dst_end <= src_start)) {
+                before->communication_amount[i*(src_nb+dst_nb)+src_nb+j]=0.0;
+              } else {
+                before->communication_amount[i*(src_nb+dst_nb)+src_nb+j] =
+                  MIN(src_end, dst_end) - MAX(src_start, dst_start);
+              }
+              XBT_VERB("==> %.2f",
+                 before->communication_amount[i*(src_nb+dst_nb)+src_nb+j]);
+            }
+          }
+
+          if (__SD_task_is_schedulable(before) ||
+              __SD_task_is_not_scheduled(before)) {
+            SD_task_do_schedule(before);
+            XBT_VERB
+              ("Auto-Schedule redistribution task %s. Send %.f bytes from %d hosts to %d hosts.",
+                  SD_task_get_name(before),before->amount, src_nb, dst_nb);
+            }
+        }
+      }
+    }
+    xbt_dynar_foreach(task->tasks_after, cpt, dep) {
+      SD_task_t after = dep->dst;
+      if (after->kind == SD_TASK_COMM_PAR_MXN_1D_BLOCK){
+        if (!after->workstation_list){
+          XBT_VERB("Receiver side of Task %s is not scheduled yet. Fill the workstation list with sender side",
+              SD_task_get_name(after));
+          after->workstation_list = xbt_new0(SD_workstation_t, count);
+          after->workstation_nb = count;
+          for (i=0;i<count;i++)
+            after->workstation_list[i] = task->workstation_list[i];
+        } else {
+          int src_nb, dst_nb;
+          double src_start, src_end, dst_start, dst_end;
+          src_nb = count;
+          dst_nb = after->workstation_nb;
+          after->workstation_list = (SD_workstation_t*) xbt_realloc(
+            after->workstation_list,
+            (after->workstation_nb+count)*sizeof(s_SD_workstation_t));
+          for(i=after->workstation_nb - 1; i>=0; i--)
+            after->workstation_list[count+i] = after->workstation_list[i];
+          for(i=0; i<count; i++)
+            after->workstation_list[i] = task->workstation_list[i];
+
+          after->workstation_nb += count;
+
+          after->computation_amount = xbt_new0(double, after->workstation_nb);
+          after->communication_amount = xbt_new0(double,
+                                                 after->workstation_nb*
+                                                 after->workstation_nb);
+
+          for(i=0;i<src_nb;i++){
+            src_start = i*after->amount/src_nb;
+            src_end = src_start + after->amount/src_nb;
+            for(j=0; j<dst_nb; j++){
+              dst_start = j*after->amount/dst_nb;
+              dst_end = dst_start + after->amount/dst_nb;
+              XBT_VERB("(%d->%d): (%.2f, %.2f)-> (%.2f, %.2f)",
+                  i, j, src_start, src_end, dst_start, dst_end);
+              if ((src_end <= dst_start) || (dst_end <= src_start)) {
+                after->communication_amount[i*(src_nb+dst_nb)+src_nb+j]=0.0;
+              } else {
+                after->communication_amount[i*(src_nb+dst_nb)+src_nb+j] =
+                   MIN(src_end, dst_end)- MAX(src_start, dst_start);
+              }
+              XBT_VERB("==> %.2f",
+                 after->communication_amount[i*(src_nb+dst_nb)+src_nb+j]);
+            }
+          }
+
+          if (__SD_task_is_schedulable(after) ||
+              __SD_task_is_not_scheduled(after)) {
+            SD_task_do_schedule(after);
+            XBT_VERB
+            ("Auto-Schedule redistribution task %s. Send %.f bytes from %d hosts to %d hosts.",
+              SD_task_get_name(after),after->amount, src_nb, dst_nb);
+          }
+         }
+      }
+    }
+  }
 }
 
 /** @brief autoschedule a task on a list of workstations
