@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -13,7 +13,6 @@
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(simix_host, simix,
                                 "Logging specific to SIMIX (hosts)");
 
-
 static void SIMIX_execution_finish(smx_action_t action);
 
 /**
@@ -67,7 +66,7 @@ void SIMIX_host_destroy(void *h)
     SIMIX_display_process_status();
     THROWF(arg_error, 0, "%s", msg);
   }
-
+  xbt_dynar_free(&host->auto_restart_processes);
   xbt_swag_free(host->process_list);
 
   /* Clean host structure */
@@ -90,7 +89,7 @@ xbt_dict_t SIMIX_host_get_dict(void)
   void **host = NULL;
 
   xbt_lib_foreach(host_lib, cursor, name, host){
-	  if(host[SIMIX_HOST_LEVEL])
+    if(host[SIMIX_HOST_LEVEL])
             xbt_dict_set(host_dict,name,host[SIMIX_HOST_LEVEL], NULL);
   }
   return host_dict;
@@ -176,6 +175,107 @@ void* SIMIX_host_get_data(smx_host_t host)
 
   return host->data;
 }
+void _SIMIX_host_free_process_arg(void *);
+void _SIMIX_host_free_process_arg(void *data)
+{
+  smx_process_arg_t arg = *(void**)data;
+  xbt_free(arg->name);
+  xbt_free(arg);
+}
+/**
+ * \brief Add a process to the list of the processes that the host will restart when it comes back
+ * This function add a process to the list of the processes that will be restarted when the host comes
+ * back. It is expected that this function is called when the host is down.
+ * The processes will only be restarted once, meaning that you will have to register the process
+ * again to restart the process again.
+ */
+void SIMIX_host_add_auto_restart_process(smx_host_t host,
+                                         const char *name,
+                                         xbt_main_func_t code,
+                                         void *data,
+                                         const char *hostname,
+                                         double kill_time,
+                                         int argc, char **argv,
+                                         xbt_dict_t properties,
+                                         int auto_restart)
+{
+  if (!host->auto_restart_processes) {
+    host->auto_restart_processes = xbt_dynar_new(sizeof(smx_process_arg_t),_SIMIX_host_free_process_arg);
+  }
+  smx_process_arg_t arg = xbt_new(s_smx_process_arg_t,1);
+  arg->name = xbt_strdup(name);
+  arg->code = code;
+  arg->data = data;
+  arg->hostname = hostname;
+  arg->kill_time = kill_time;
+  arg->argc = argc;
+
+  arg->argv = xbt_new(char*,argc + 1);
+
+  int i;
+  for (i = 0; i < argc; i++) {
+    arg->argv[i] = xbt_strdup(argv[i]);
+  }
+  arg->argv[argc] = NULL;
+
+  arg->properties = properties;
+  arg->auto_restart = auto_restart;
+
+  if( SIMIX_host_get_state(host) == SURF_RESOURCE_OFF
+      && !xbt_dict_get_or_null(watched_hosts_lib,host->name)){
+    xbt_dict_set(watched_hosts_lib,host->name,host,NULL);
+    XBT_DEBUG("Have push host %s to watched_hosts_lib because state == SURF_RESOURCE_OFF",host->name);
+  }
+  xbt_dynar_push_as(host->auto_restart_processes,smx_process_arg_t,arg);
+}
+/**
+ * \brief Restart the list of processes that have been registered to the host
+ */
+void SIMIX_host_restart_processes(smx_host_t host)
+{
+  unsigned int cpt;
+  smx_process_arg_t arg;
+  xbt_dynar_foreach(host->auto_restart_processes,cpt,arg) {
+
+    smx_process_t process;
+
+    XBT_DEBUG("Restarting Process %s(%s) right now", arg->argv[0], arg->hostname);
+    if (simix_global->create_process_function) {
+      simix_global->create_process_function(&process,
+                                            arg->argv[0],
+                                            arg->code,
+                                            NULL,
+                                            arg->hostname,
+                                            arg->kill_time,
+                                            arg->argc,
+                                            arg->argv,
+                                            arg->properties,
+                                            arg->auto_restart);
+    }
+    else {
+      simcall_process_create(&process,
+                                            arg->argv[0],
+                                            arg->code,
+                                            NULL,
+                                            arg->hostname,
+                                            arg->kill_time,
+                                            arg->argc,
+                                            arg->argv,
+                                            arg->properties,
+                                            arg->auto_restart);
+
+    }
+  }
+  xbt_dynar_reset(host->auto_restart_processes);
+}
+
+void SIMIX_host_autorestart(smx_host_t host)
+{
+  if(simix_global->autorestart)
+    simix_global->autorestart(host);
+  else
+    xbt_die("No function for simix_global->autorestart");
+}
 
 void SIMIX_host_set_data(smx_host_t host, void *data)
 {
@@ -201,10 +301,10 @@ smx_action_t SIMIX_host_execute(const char *name, smx_host_t host,
 #endif
 
   /* set surf's action */
-  if (!MC_IS_ENABLED) {
+  if (!MC_is_active()) {
     action->execution.surf_exec =
       surf_workstation_model->extension.workstation.execute(host->host,
-	  computation_amount);
+    computation_amount);
     surf_workstation_model->action_data_set(action->execution.surf_exec, action);
     surf_workstation_model->set_priority(action->execution.surf_exec, priority);
   }
@@ -239,11 +339,11 @@ smx_action_t SIMIX_host_parallel_execute( const char *name,
     workstation_list[i] = host_list[i]->host;
 
   /* set surf's action */
-  if (!MC_IS_ENABLED) {
+  if (!MC_is_active()) {
     action->execution.surf_exec =
       surf_workstation_model->extension.workstation.
       execute_parallel_task(host_nb, workstation_list, computation_amount,
-	                    communication_amount, rate);
+                      communication_amount, rate);
 
     surf_workstation_model->action_data_set(action->execution.surf_exec, action);
   }
@@ -256,13 +356,11 @@ void SIMIX_host_execution_destroy(smx_action_t action)
 {
   XBT_DEBUG("Destroy action %p", action);
 
-  xbt_free(action->name);
-
   if (action->execution.surf_exec) {
     surf_workstation_model->action_unref(action->execution.surf_exec);
     action->execution.surf_exec = NULL;
   }
-
+  xbt_free(action->name);
   xbt_mallocator_release(simix_global->action_mallocator, action);
 }
 
@@ -306,7 +404,7 @@ void SIMIX_pre_host_execution_wait(smx_simcall_t simcall)
   simcall->issuer->waiting_action = action;
 
   /* set surf's action */
-  if (MC_IS_ENABLED) {
+  if (MC_is_active()) {
     action->state = SIMIX_DONE;
     SIMIX_execution_finish(action);
     return;
@@ -340,12 +438,13 @@ void SIMIX_execution_finish(smx_action_t action)
 
       case SIMIX_DONE:
         /* do nothing, action done */
-	XBT_DEBUG("SIMIX_execution_finished: execution successful");
+  XBT_DEBUG("SIMIX_execution_finished: execution successful");
         break;
 
       case SIMIX_FAILED:
         XBT_DEBUG("SIMIX_execution_finished: host '%s' failed", simcall->issuer->smx_host->name);
-        SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
+        simcall->issuer->context->iwannadie = 1;
+        //SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
         break;
 
       case SIMIX_CANCELED:
@@ -357,7 +456,13 @@ void SIMIX_execution_finish(smx_action_t action)
         xbt_die("Internal error in SIMIX_execution_finish: unexpected action state %d",
             (int)action->state);
     }
-    simcall->issuer->waiting_action = NULL;
+    /* check if the host is down */
+    if (surf_workstation_model->extension.
+        workstation.get_state(simcall->issuer->smx_host->host) != SURF_RESOURCE_ON) {
+      simcall->issuer->context->iwannadie = 1;
+    }
+
+    simcall->issuer->waiting_action =    NULL;
     simcall->host_execution_wait.result = action->state;
     SIMIX_simcall_answer(simcall);
   }
@@ -368,14 +473,19 @@ void SIMIX_execution_finish(smx_action_t action)
 
 void SIMIX_post_host_execute(smx_action_t action)
 {
-  /* FIXME: check if the host running the action failed or not*/
-  /*if(surf_workstation_model->extension.workstation.get_state(action->host->host))*/
-
-  /* If the host running the action didn't fail, then the action was canceled */
-  if (surf_workstation_model->action_state_get(action->execution.surf_exec) == SURF_ACTION_FAILED)
-     action->state = SIMIX_CANCELED;
-  else
-     action->state = SIMIX_DONE;
+  if (action->type == SIMIX_ACTION_EXECUTE && /* FIMXE: handle resource failure
+                                               * for parallel tasks too */
+      surf_workstation_model->extension.workstation.get_state(action->execution.host->host) == SURF_RESOURCE_OFF) {
+    /* If the host running the action failed, notice it so that the asking
+     * process can be killed if it runs on that host itself */
+    action->state = SIMIX_FAILED;
+  } else if (surf_workstation_model->action_state_get(action->execution.surf_exec) == SURF_ACTION_FAILED) {
+    /* If the host running the action didn't fail, then the action was
+     * canceled */
+    action->state = SIMIX_CANCELED;
+  } else {
+    action->state = SIMIX_DONE;
+  }
 
   if (action->execution.surf_exec) {
     surf_workstation_model->action_unref(action->execution.surf_exec);
