@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -118,6 +118,8 @@ void SIMIX_process_empty_trash(void)
 
     xbt_fifo_free(process->comms);
 
+    xbt_dynar_free(&process->on_exit);
+
     free(process->name);
     free(process);
   }
@@ -142,9 +144,34 @@ void SIMIX_create_maestro_process()
   simix_global->maestro_process = maestro;
   return;
 }
+/**
+ * \brief Stops a process.
+ *
+ * Stops the process, execute all the registered on_exit functions,
+ * register it to the list of the process to restart if needed
+ * and stops its context.
+ */
+void SIMIX_process_stop(smx_process_t arg) {
+  /* execute the on_exit functions */
+  SIMIX_process_on_exit_runall(arg);
+  /* Add the process to the list of process to restart, only if
+   * the host is down
+   */
+  if (arg->auto_restart && !SIMIX_host_get_state(arg->smx_host)) {
+    SIMIX_host_add_auto_restart_process(arg->smx_host,arg->name,arg->code, arg->data,
+                                        arg->smx_host->name,
+                                        arg->kill_time,
+                                        arg->argc,arg->argv,arg->properties,
+                                        arg->auto_restart);
+  }
+  XBT_DEBUG("Process %s (%s) is dead",arg->name,arg->smx_host->name);
+  /* stop the context */
+  SIMIX_context_stop(arg->context);
+}
 
 /**
  * \brief Same as SIMIX_process_create() but with only one argument (used by timers).
+ * This function frees the argument.
  * \return the process created
  */
 smx_process_t SIMIX_process_create_from_wrapper(smx_process_arg_t args) {
@@ -159,8 +186,9 @@ smx_process_t SIMIX_process_create_from_wrapper(smx_process_arg_t args) {
       args->kill_time,
       args->argc,
       args->argv,
-      args->properties);
-
+      args->properties,
+      args->auto_restart);
+  xbt_free(args);
   return process;
 }
 
@@ -180,7 +208,8 @@ void SIMIX_process_create(smx_process_t *process,
                           const char *hostname,
                           double kill_time,
                           int argc, char **argv,
-                          xbt_dict_t properties) {
+                          xbt_dict_t properties,
+                          int auto_restart) {
 
   *process = NULL;
   smx_host_t host = SIMIX_host_get_by_name(hostname);
@@ -195,7 +224,6 @@ void SIMIX_process_create(smx_process_t *process,
     *process = xbt_new0(s_smx_process_t, 1);
 
     xbt_assert(((code != NULL) && (host != NULL)), "Invalid parameters");
-
     /* Process data */
     (*process)->pid = simix_process_maxpid++;
     (*process)->name = xbt_strdup(name);
@@ -203,10 +231,17 @@ void SIMIX_process_create(smx_process_t *process,
     (*process)->data = data;
     (*process)->comms = xbt_fifo_new();
     (*process)->simcall.issuer = *process;
+    /* Process data for auto-restart */
+    (*process)->auto_restart = auto_restart;
+    (*process)->code = code;
+    (*process)->argc = argc;
+    (*process)->argv = argv;
+    (*process)->kill_time = kill_time;
+
 
     XBT_VERB("Create context %s", (*process)->name);
     (*process)->context = SIMIX_context_new(code, argc, argv,
-    	simix_global->cleanup_process_function, *process);
+      simix_global->cleanup_process_function, *process);
 
     (*process)->running_ctx = xbt_new(xbt_running_ctx_t, 1);
     XBT_RUNNING_CTX_INITIALIZE((*process)->running_ctx);
@@ -260,8 +295,9 @@ void SIMIX_process_runall(void)
  * or directly for SIMIX internal purposes.
  *
  * \param process poor victim
+ * \param issuer the process which has sent the PROCESS_KILL. Important to not schedule twice the same process.
  */
-void SIMIX_process_kill(smx_process_t process) {
+void SIMIX_process_kill(smx_process_t process, smx_process_t issuer) {
 
   XBT_DEBUG("Killing process %s on %s", process->name, process->smx_host->name);
 
@@ -282,25 +318,35 @@ void SIMIX_process_kill(smx_process_t process) {
 
       case SIMIX_ACTION_COMMUNICATE:
         xbt_fifo_remove(process->comms, process->waiting_action);
-        SIMIX_comm_destroy(process->waiting_action);
+        SIMIX_comm_cancel(process->waiting_action);
         break;
 
-      case SIMIX_ACTION_SLEEP:
-	SIMIX_process_sleep_destroy(process->waiting_action);
-	break;
+        case SIMIX_ACTION_SLEEP:
+          SIMIX_process_sleep_destroy(process->waiting_action);
+          break;
 
-      case SIMIX_ACTION_SYNCHRO:
-	SIMIX_synchro_stop_waiting(process, &process->simcall);
-	SIMIX_synchro_destroy(process->waiting_action);
-	break;
+        case SIMIX_ACTION_SYNCHRO:
+          SIMIX_synchro_stop_waiting(process, &process->simcall);
+          SIMIX_synchro_destroy(process->waiting_action);
+          break;
+
+        case SIMIX_ACTION_IO:
+          SIMIX_io_destroy(process->waiting_action);
+          break;
+
+        /* **************************************/
+        /* TUTORIAL: New API                    */
+        case SIMIX_ACTION_NEW_API:
+          SIMIX_new_api_destroy(process->waiting_action);
+          break;
+        /* **************************************/
 
-      case SIMIX_ACTION_IO:
-        SIMIX_io_destroy(process->waiting_action);
-        break;
     }
   }
-  if(!xbt_dynar_member(simix_global->process_to_run, &(process)))
+  if(!xbt_dynar_member(simix_global->process_to_run, &(process)) && process != issuer) {
     xbt_dynar_push_as(simix_global->process_to_run, smx_process_t, process);
+  }
+
 }
 
 /**
@@ -313,7 +359,7 @@ void SIMIX_process_killall(smx_process_t issuer)
 
   while ((p = xbt_swag_extract(simix_global->process_list))) {
     if (p != issuer) {
-      SIMIX_process_kill(p);
+      SIMIX_process_kill(p,issuer);
     }
   }
 
@@ -323,7 +369,7 @@ void SIMIX_process_killall(smx_process_t issuer)
 }
 
 void SIMIX_process_change_host(smx_process_t process,
-			       smx_host_t dest)
+             smx_host_t dest)
 {
   xbt_assert((process != NULL), "Invalid parameters");
   xbt_swag_remove(process, process->smx_host->process_list);
@@ -533,7 +579,7 @@ xbt_dict_t SIMIX_process_get_properties(smx_process_t process)
 
 void SIMIX_pre_process_sleep(smx_simcall_t simcall)
 {
-  if (MC_IS_ENABLED) {
+  if (MC_is_active()) {
     MC_process_clock_add(simcall->issuer, simcall->process_sleep.duration);
     simcall->process_sleep.result = SIMIX_DONE;
     SIMIX_simcall_answer(simcall);
@@ -582,7 +628,8 @@ void SIMIX_post_process_sleep(smx_action_t action)
 
     switch(surf_workstation_model->action_state_get(action->sleep.surf_sleep)){
       case SURF_ACTION_FAILED:
-        state = SIMIX_SRC_HOST_FAILURE;
+        simcall->issuer->context->iwannadie = 1;
+        //SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
         break;
 
       case SURF_ACTION_DONE:
@@ -593,9 +640,14 @@ void SIMIX_post_process_sleep(smx_action_t action)
         THROW_IMPOSSIBLE;
         break;
     }
+    if (surf_workstation_model->extension.
+        workstation.get_state(simcall->issuer->smx_host->host) != SURF_RESOURCE_ON) {
+      simcall->issuer->context->iwannadie = 1;
+    }
     simcall->process_sleep.result = state;
     simcall->issuer->waiting_action = NULL;
     SIMIX_simcall_answer(simcall);
+
   }
   SIMIX_process_sleep_destroy(action);
 }
@@ -636,9 +688,14 @@ void SIMIX_process_yield(smx_process_t self)
   /* Ok, maestro returned control to us */
   XBT_DEBUG("Control returned to me: '%s'", self->name);
 
+  if (self->new_host) {
+    SIMIX_process_change_host(self, self->new_host);
+    self->new_host = NULL;
+  }
+
   if (self->context->iwannadie){
     XBT_DEBUG("I wanna die!");
-    SIMIX_context_stop(self->context);
+    SIMIX_process_stop(self);
   }
 
   if(self->suspended) {
@@ -652,11 +709,6 @@ void SIMIX_process_yield(smx_process_t self)
     self->doexception = 0;
     SMX_THROW();
   }
-  
-  if (self->new_host) {
-    SIMIX_process_change_host(self, self->new_host);
-    self->new_host = NULL;
-  }
 }
 
 /* callback: context fetching */
@@ -669,7 +721,7 @@ xbt_running_ctx_t *SIMIX_process_get_running_context(void)
 void SIMIX_process_exception_terminate(xbt_ex_t * e)
 {
   xbt_ex_display(e);
-  abort();
+  xbt_abort();
 }
 
 smx_context_t SIMIX_process_get_context(smx_process_t p) {
@@ -693,13 +745,13 @@ xbt_dynar_t SIMIX_process_get_runnable(void)
  */
 smx_process_t SIMIX_process_from_PID(int PID)
 {
-	smx_process_t proc;
-	xbt_swag_foreach(proc, simix_global->process_list)
-	{
-	 if(proc->pid == PID)
-	 return proc;
-	}
-	return NULL;
+  smx_process_t proc;
+  xbt_swag_foreach(proc, simix_global->process_list)
+  {
+   if(proc->pid == PID)
+   return proc;
+  }
+  return NULL;
 }
 
 /** @brief returns a dynar containg all currently existing processes */
@@ -711,3 +763,83 @@ xbt_dynar_t SIMIX_processes_as_dynar(void) {
   }
   return res;
 }
+void SIMIX_process_on_exit_runall(smx_process_t process) {
+  s_smx_process_exit_fun_t exit_fun;
+
+  while (!xbt_dynar_is_empty(process->on_exit)) {
+    exit_fun = xbt_dynar_pop_as(process->on_exit,s_smx_process_exit_fun_t);
+    (exit_fun.fun)(exit_fun.arg);
+  }
+}
+void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data) {
+  xbt_assert(process, "current process not found: are you in maestro context ?");
+
+  if (!process->on_exit) {
+    process->on_exit = xbt_dynar_new(sizeof(s_smx_process_exit_fun_t), NULL);
+  }
+
+  s_smx_process_exit_fun_t exit_fun = {fun, data};
+
+  xbt_dynar_push_as(process->on_exit,s_smx_process_exit_fun_t,exit_fun);
+}
+/**
+ * \brief Sets the auto-restart status of the process.
+ * If set to 1, the process will be automatically restarted when its host
+ * comes back.
+ */
+void SIMIX_process_auto_restart_set(smx_process_t process, int auto_restart) {
+  process->auto_restart = auto_restart;
+}
+/**
+ * \brief Restart a process.
+ * Restart a process, starting it again from the beginning.
+ */
+smx_process_t SIMIX_process_restart(smx_process_t process, smx_process_t issuer) {
+  XBT_DEBUG("Restarting process %s on %s", process->name, process->smx_host->name);
+  //retrieve the arguments of the old process
+  //FIXME: Factorise this with SIMIX_host_add_auto_restart_process ?
+  s_smx_process_arg_t arg;
+  arg.code = process->code;
+  arg.hostname = process->smx_host->name;
+  arg.kill_time = process->kill_time;
+  arg.argc = process->argc;
+  arg.data = process->data;
+  int i;
+  arg.argv = xbt_new(char*,process->argc + 1);
+  for (i = 0; i < arg.argc; i++) {
+    arg.argv[i] = xbt_strdup(process->argv[i]);
+  }
+  arg.argv[process->argc] = NULL;
+  arg.properties = NULL;
+  arg.auto_restart = process->auto_restart;
+  //kill the old process
+  SIMIX_process_kill(process,issuer);
+  //start the new process
+  smx_process_t new_process;
+  if (simix_global->create_process_function) {
+    simix_global->create_process_function(&new_process,
+                                          arg.argv[0],
+                                          arg.code,
+                                          arg.data,
+                                          arg.hostname,
+                                          arg.kill_time,
+                                          arg.argc,
+                                          arg.argv,
+                                          arg.properties,
+                                          arg.auto_restart);
+  }
+  else {
+    simcall_process_create(&new_process,
+                                          arg.argv[0],
+                                          arg.code,
+                                          arg.data,
+                                          arg.hostname,
+                                          arg.kill_time,
+                                          arg.argc,
+                                          arg.argv,
+                                          arg.properties,
+                                          arg.auto_restart);
+
+  }
+  return new_process;
+}
