@@ -100,7 +100,8 @@ static void SIMIX_synchro_finish(smx_action_t action)
       break;
 
     case SIMIX_FAILED:
-      SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
+        simcall->issuer->context->iwannadie = 1;
+//      SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
       break;
 
     default:
@@ -175,8 +176,8 @@ int SIMIX_mutex_trylock(smx_mutex_t mutex, smx_process_t issuer)
 {
   XBT_IN("(%p, %p)",mutex,issuer);
   if (mutex->locked){
-	  XBT_OUT();
-	  return 0;
+    XBT_OUT();
+    return 0;
   }
 
   mutex->locked = 1;
@@ -201,8 +202,8 @@ void SIMIX_mutex_unlock(smx_mutex_t mutex, smx_process_t issuer)
 
   /* If the mutex is not owned by the issuer do nothing */
   if (issuer != mutex->owner){
-	  XBT_OUT();
-	  return;
+    XBT_OUT();
+    return;
   }
 
   if (xbt_swag_size(mutex->sleeping) > 0) {
