@@ -6,8 +6,8 @@
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 /*
-	commit: e2d6799c4182f00443b3013aadb1c2412372460f
-	This commit retrieves the old implementation of CPU_TI with multi-levels.
+  commit: e2d6799c4182f00443b3013aadb1c2412372460f
+  This commit retrieves the old implementation of CPU_TI with multi-levels.
 */
 
 #include "surf_private.h"
@@ -73,12 +73,12 @@ static surf_cpu_ti_trace_t surf_cpu_ti_trace_new(tmgr_trace_t power_trace)
   trace = xbt_new0(s_surf_cpu_ti_trace_t, 1);
   trace->time_points =
       xbt_malloc0(sizeof(double) *
-                  (xbt_dynar_length(power_trace->event_list) + 1));
+                  (xbt_dynar_length(power_trace->s_list.event_list) + 1));
   trace->integral =
       xbt_malloc0(sizeof(double) *
-                  (xbt_dynar_length(power_trace->event_list) + 1));
-  trace->nb_points = xbt_dynar_length(power_trace->event_list);
-  xbt_dynar_foreach(power_trace->event_list, cpt, val) {
+                  (xbt_dynar_length(power_trace->s_list.event_list) + 1));
+  trace->nb_points = xbt_dynar_length(power_trace->s_list.event_list);
+  xbt_dynar_foreach(power_trace->s_list.event_list, cpt, val) {
     trace->time_points[i] = time;
     trace->integral[i] = integral;
     integral += val.delta * val.value;
@@ -93,10 +93,10 @@ static surf_cpu_ti_trace_t surf_cpu_ti_trace_new(tmgr_trace_t power_trace)
 /**
 * \brief Creates a new integration trace from a tmgr_trace_t
 *
-* \param	power_trace		CPU availability trace
-* \param	value					Percentage of CPU power available (useful to fixed tracing)
-* \param	spacing				Initial spacing
-* \return	Integration trace structure
+* \param  power_trace    CPU availability trace
+* \param  value          Percentage of CPU power available (useful to fixed tracing)
+* \param  spacing        Initial spacing
+* \return  Integration trace structure
 */
 static surf_cpu_ti_tgmr_t cpu_ti_parse_trace(tmgr_trace_t power_trace,
                                              double value)
@@ -116,8 +116,8 @@ static surf_cpu_ti_tgmr_t cpu_ti_parse_trace(tmgr_trace_t power_trace,
   }
 
   /* only one point available, fixed trace */
-  if (xbt_dynar_length(power_trace->event_list) == 1) {
-    xbt_dynar_get_cpy(power_trace->event_list, 0, &val);
+  if (xbt_dynar_length(power_trace->s_list.event_list) == 1) {
+    xbt_dynar_get_cpy(power_trace->s_list.event_list, 0, &val);
     trace->type = TRACE_FIXED;
     trace->value = val.value;
     return trace;
@@ -127,7 +127,7 @@ static surf_cpu_ti_tgmr_t cpu_ti_parse_trace(tmgr_trace_t power_trace,
   trace->power_trace = power_trace;
 
   /* count the total time of trace file */
-  xbt_dynar_foreach(power_trace->event_list, cpt, val) {
+  xbt_dynar_foreach(power_trace->s_list.event_list, cpt, val) {
     total_time += val.delta;
   }
   trace->trace = surf_cpu_ti_trace_new(power_trace);
@@ -172,10 +172,10 @@ static void* cpu_ti_create_resource(const char *name, double power_peak,
   if (state_trace)
     cpu->state_event =
         tmgr_history_add_trace(history, state_trace, 0.0, 0, cpu);
-  if (power_trace && xbt_dynar_length(power_trace->event_list) > 1) {
+  if (power_trace && xbt_dynar_length(power_trace->s_list.event_list) > 1) {
     /* add a fake trace event if periodicity == 0 */
-    xbt_dynar_get_cpy(power_trace->event_list,
-                      xbt_dynar_length(power_trace->event_list) - 1, &val);
+    xbt_dynar_get_cpy(power_trace->s_list.event_list,
+                      xbt_dynar_length(power_trace->s_list.event_list) - 1, &val);
     if (val.delta == 0) {
       empty_trace = tmgr_empty_trace_new();
       cpu->power_event =
@@ -192,13 +192,13 @@ static void* cpu_ti_create_resource(const char *name, double power_peak,
 static void parse_cpu_ti_init(sg_platf_host_cbarg_t host)
 {
   cpu_ti_create_resource(host->id,
-			  host->power_peak,
-			  host->power_scale,
-			  host->power_trace,
-			  host->core_amount,
-			  host->initial_state,
-			  host->state_trace,
-			  host->properties);
+        host->power_peak,
+        host->power_scale,
+        host->power_trace,
+        host->core_amount,
+        host->initial_state,
+        host->state_trace,
+        host->properties);
 
 }
 
@@ -244,10 +244,10 @@ static void add_traces_cpu_ti(void)
     cpu->avail_trace = cpu_ti_parse_trace(trace, cpu->power_scale);
 
     /* add a fake trace event if periodicity == 0 */
-    if (trace && xbt_dynar_length(trace->event_list) > 1) {
+    if (trace && xbt_dynar_length(trace->s_list.event_list) > 1) {
       s_tmgr_event_t val;
-      xbt_dynar_get_cpy(trace->event_list,
-                        xbt_dynar_length(trace->event_list) - 1, &val);
+      xbt_dynar_get_cpy(trace->s_list.event_list,
+                        xbt_dynar_length(trace->s_list.event_list) - 1, &val);
       if (val.delta == 0) {
         tmgr_trace_t empty_trace;
         empty_trace = tmgr_empty_trace_new();
@@ -308,8 +308,8 @@ static void cpu_ti_action_state_set(surf_action_t action,
 /**
 * \brief Update the remaining amount of actions
 *
-* \param	cpu		Cpu on which the actions are running
-* \param	now		Current time
+* \param  cpu    Cpu on which the actions are running
+* \param  now    Current time
 */
 static void cpu_ti_update_remaining_amount(cpu_ti_t cpu, double now)
 {
@@ -365,8 +365,8 @@ static void cpu_ti_update_remaining_amount(cpu_ti_t cpu, double now)
 /**
 * \brief Update the finish date of action if necessary
 *
-* \param	cpu		Cpu on which the actions are running
-* \param	now		Current time
+* \param  cpu    Cpu on which the actions are running
+* \param  now    Current time
 */
 static void cpu_ti_update_action_finish_date(cpu_ti_t cpu, double now)
 {
@@ -494,6 +494,8 @@ static void cpu_ti_update_resource_state(void *id,
   cpu_ti_t cpu = id;
   surf_action_cpu_ti_t action;
 
+  surf_watched_hosts();
+
   if (event_type == cpu->power_event) {
     tmgr_trace_t power_trace;
     surf_cpu_ti_tgmr_t trace;
@@ -506,8 +508,8 @@ static void cpu_ti_update_resource_state(void *id,
     xbt_swag_insert(cpu, cpu_ti_modified_cpu);
 
     power_trace = cpu->avail_trace->power_trace;
-    xbt_dynar_get_cpy(power_trace->event_list,
-                      xbt_dynar_length(power_trace->event_list) - 1, &val);
+    xbt_dynar_get_cpy(power_trace->s_list.event_list,
+                      xbt_dynar_length(power_trace->s_list.event_list) - 1, &val);
     /* free old trace */
     surf_cpu_ti_free_tmgr(cpu->avail_trace);
     cpu->power_scale = val.value;
@@ -698,9 +700,9 @@ static double cpu_ti_get_speed(void *cpu, double load)
 /**
 * \brief Auxiliary function to update the CPU power scale.
 *
-*	This function uses the trace structure to return the power scale at the determined time a.
-* \param trace		Trace structure to search the updated power scale
-* \param a				Time
+*  This function uses the trace structure to return the power scale at the determined time a.
+* \param trace    Trace structure to search the updated power scale
+* \param a        Time
 * \return CPU power scale
 */
 static double surf_cpu_ti_get_power_scale(surf_cpu_ti_tgmr_t trace,
@@ -714,7 +716,7 @@ static double surf_cpu_ti_get_power_scale(surf_cpu_ti_tgmr_t trace,
   point =
       surf_cpu_ti_binary_search(trace->trace->time_points, reduced_a, 0,
                                 trace->trace->nb_points - 1);
-  xbt_dynar_get_cpy(trace->power_trace->event_list, point, &val);
+  xbt_dynar_get_cpy(trace->power_trace->s_list.event_list, point, &val);
   return val.value;
 }
 
@@ -734,12 +736,12 @@ static void cpu_ti_finalize(void)
   char *key;
 
   xbt_lib_foreach(host_lib, cursor, key, cpu){
-	  if(cpu[SURF_CPU_LEVEL])
-	  {
-		    cpu_ti_t CPU = cpu[SURF_CPU_LEVEL];
-		    xbt_swag_free(CPU->action_set);
-		    surf_cpu_ti_free_tmgr(CPU->avail_trace);
-	  }
+    if(cpu[SURF_CPU_LEVEL])
+    {
+        cpu_ti_t CPU = cpu[SURF_CPU_LEVEL];
+        xbt_swag_free(CPU->action_set);
+        surf_cpu_ti_free_tmgr(CPU->avail_trace);
+    }
   }
 
   surf_model_exit(surf_cpu_model);
@@ -818,8 +820,8 @@ void surf_cpu_model_init_ti()
 * the cyclic effect.
 *
 * \param trace Trace structure.
-* \param a			Begin of interval
-* \param b			End of interval
+* \param a      Begin of interval
+* \param b      End of interval
 * \return the integrate value. -1 if an error occurs.
 */
 static double surf_cpu_ti_integrate_trace(surf_cpu_ti_tgmr_t trace,
@@ -879,12 +881,12 @@ static double surf_cpu_ti_integrate_trace(surf_cpu_ti_tgmr_t trace,
 
 /**
  * \brief Auxiliary function to calculate the integral between a and b.
- * 		It simply calculates the integral at point a and b and returns the difference 
- * 	between them.
- * \param trace		Trace structure
- * \param a				Initial point
- * \param b	Final point
- * \return	Integral
+ *     It simply calculates the integral at point a and b and returns the difference 
+ *   between them.
+ * \param trace    Trace structure
+ * \param a        Initial point
+ * \param b  Final point
+ * \return  Integral
 */
 static double surf_cpu_ti_integrate_trace_simple(surf_cpu_ti_trace_t trace,
                                                  double a, double b)
@@ -896,9 +898,9 @@ static double surf_cpu_ti_integrate_trace_simple(surf_cpu_ti_trace_t trace,
 
 /**
  * \brief Auxiliary function to calculate the integral at point a.
- * \param trace		Trace structure
- * \param a				point
- * \return	Integral
+ * \param trace    Trace structure
+ * \param a        point
+ * \return  Integral
 */
 static double surf_cpu_ti_integrate_trace_simple_point(surf_cpu_ti_trace_t
                                                        trace, double a)
@@ -933,10 +935,10 @@ static double surf_cpu_ti_integrate_trace_simple_point(surf_cpu_ti_trace_t
 *
 * Here, amount can span multiple trace periods
 *
-* \param trace 	CPU trace structure
-* \param a				Initial time
-* \param amount	Amount to be executed
-* \return	End time
+* \param trace   CPU trace structure
+* \param a        Initial time
+* \param amount  Amount to be executed
+* \return  End time
 */
 static double surf_cpu_ti_solve_trace(surf_cpu_ti_tgmr_t trace, double a,
                                       double amount)
@@ -985,8 +987,8 @@ static double surf_cpu_ti_solve_trace(surf_cpu_ti_tgmr_t trace, double a,
 
 /* Now solve for new_amount which is <= trace_total */
 /*
-	 fprintf(stderr,"reduced_a = %.2f\n",reduced_a);
-	 fprintf(stderr,"reduced_amount = %.2f\n",reduced_amount);
+   fprintf(stderr,"reduced_a = %.2f\n",reduced_a);
+   fprintf(stderr,"reduced_amount = %.2f\n",reduced_amount);
  */
   reduced_b =
       surf_cpu_ti_solve_trace_somewhat_simple(trace, reduced_a,
@@ -1016,7 +1018,7 @@ static double surf_cpu_ti_solve_trace_somewhat_simple(surf_cpu_ti_tgmr_t
   amount_till_end =
       surf_cpu_ti_integrate_trace(trace, a, trace->last_time);
 /*
-	 fprintf(stderr,"amount_till_end=%.2f\n",amount_till_end);
+   fprintf(stderr,"amount_till_end=%.2f\n",amount_till_end);
  */
 
   if (amount_till_end > amount) {
@@ -1031,10 +1033,10 @@ static double surf_cpu_ti_solve_trace_somewhat_simple(surf_cpu_ti_tgmr_t
 
 /**
  * \brief Auxiliary function to solve integral.
- *	It returns the date when the requested amount of flops is available
- * \param trace		Trace structure
- * \param a				Initial point
- * \param amount	Amount of flops 
+ *  It returns the date when the requested amount of flops is available
+ * \param trace    Trace structure
+ * \param a        Initial point
+ * \param amount  Amount of flops 
  * \return The date when amount is available.
 */
 static double surf_cpu_ti_solve_trace_simple(surf_cpu_ti_trace_t trace,
@@ -1060,11 +1062,11 @@ static double surf_cpu_ti_solve_trace_simple(surf_cpu_ti_trace_t trace,
 
 /**
  * \brief Binary search in array.
- *	It returns the first point of the interval in which "a" is. 
- * \param array		Array
- * \param a				Value to search
- * \param low 		Low bound to search in array
- * \param high		Upper bound to search in array
+ *  It returns the first point of the interval in which "a" is. 
+ * \param array    Array
+ * \param a        Value to search
+ * \param low     Low bound to search in array
+ * \param high    Upper bound to search in array
  * \return Index of point
 */
 static int surf_cpu_ti_binary_search(double *array, double a, int low,
