@@ -30,17 +30,15 @@ static int storage_selective_update = 0;
 static xbt_swag_t
     storage_running_action_set_that_does_not_need_being_checked = NULL;
 
+static xbt_dynar_t storage_list;
+
 #define GENERIC_LMM_ACTION(action) action->generic_lmm_action
 #define GENERIC_ACTION(action) GENERIC_LMM_ACTION(action).generic_action
 
-typedef struct surf_storage {
-  s_surf_resource_t generic_resource;
-  const char* type;
-  const char* content; /*should be a dict */
-} s_surf_storage_t, *surf_storage_t;
-
+static xbt_dict_t parse_storage_content(char *filename, unsigned long *used_size);
 static void storage_action_state_set(surf_action_t action, e_surf_action_state_t state);
-static surf_action_t storage_action_execute (void *storage, size_t size, e_surf_action_storage_type_t type);
+static surf_action_t storage_action_execute (void *storage, double size, e_surf_action_storage_type_t type);
+static void free_storage_content(void *p);
 
 static surf_action_t storage_action_stat(void *storage, surf_file_t stream)
 {
@@ -50,20 +48,88 @@ static surf_action_t storage_action_stat(void *storage, surf_file_t stream)
   return action;
 }
 
-static surf_action_t storage_action_open(void *storage, const char* path, const char* mode)
+static surf_action_t storage_action_ls(void *storage, const char* path)
+{
+  surf_action_t action = storage_action_execute(storage,0, LS);
+  action->ls_dict = NULL;
+  xbt_dict_t ls_dict = xbt_dict_new();
+
+  char* key;
+  surf_stat_t data = NULL;
+  xbt_dict_cursor_t cursor = NULL;
+
+  xbt_dynar_t dyn = NULL;
+  char* file = NULL;
+
+  // foreach file int the storage
+  xbt_dict_foreach(((storage_t)storage)->content,cursor,key,data){
+    // Search if file start with the prefix 'path'
+    if(xbt_str_start_with(key,path)){
+      file = &key[strlen(path)];
+
+      // Split file with '/'
+      dyn = xbt_str_split(file,"/");
+      file = xbt_dynar_get_as(dyn,0,char*);
+
+      // file
+      if(xbt_dynar_length(dyn) == 1){
+        xbt_dict_set(ls_dict,file,&(data->stat),NULL);
+      }
+      // Directory
+      else
+      {
+        // if directory does not exist yet in dict
+        if(!xbt_dict_get_or_null(ls_dict,file))
+          xbt_dict_set(ls_dict,file,NULL,NULL);
+      }
+      xbt_dynar_free(&dyn);
+    }
+  }
+
+  action->ls_dict = ls_dict;
+  return action;
+}
+
+static surf_action_t storage_action_unlink(void *storage, surf_file_t stream)
+{
+  surf_action_t action = storage_action_execute(storage,0, UNLINK);
+
+  // Add memory to storage
+  ((storage_t)storage)->used_size -= stream->content->stat.size;
+
+  // Remove the file from storage
+  xbt_dict_t content_dict = ((storage_t)storage)->content;
+  xbt_dict_remove(content_dict,stream->name);
+
+  free(stream->name);
+  stream->content = NULL;
+  xbt_free(stream);
+
+  return action;
+}
+
+static surf_action_t storage_action_open(void *storage, const char* mount, const char* path, const char* mode)
 {
   XBT_DEBUG("\tOpen file '%s'",path);
-  char *storage_type_id = xbt_lib_get_or_null(
-      storage_lib,
-      ((storage_t)storage)->generic_resource.name,
-      ROUTING_STORAGE_LEVEL);
-  storage_type_t storage_type = xbt_lib_get_or_null(storage_type_lib, storage_type_id,ROUTING_STORAGE_TYPE_LEVEL);
-  xbt_dict_t content_dict = storage_type->content;
-  surf_stat_t content = xbt_dict_get(content_dict,path);
+  xbt_dict_t content_dict = ((storage_t)storage)->content;
+  surf_stat_t content = xbt_dict_get_or_null(content_dict,path);
 
+  // if file does not exist create an empty file
+  if(!content){
+    content = xbt_new0(s_surf_stat_t,1);
+    content->stat.date = xbt_strdup("");
+    content->stat.group = xbt_strdup("");
+    content->stat.size = 0;
+    content->stat.time = xbt_strdup("");
+    content->stat.user = xbt_strdup("");
+    content->stat.user_rights = xbt_strdup("");
+    xbt_dict_set(content_dict,path,content,NULL);
+    XBT_DEBUG("File '%s' was not found, file created.",path);
+  }
   surf_file_t file = xbt_new0(s_surf_file_t,1);
   file->name = xbt_strdup(path);
   file->content = content;
+  file->storage = mount;
 
   surf_action_t action = storage_action_execute(storage,0, OPEN);
   action->file = (void *)file;
@@ -73,19 +139,17 @@ static surf_action_t storage_action_open(void *storage, const char* path, const
 static surf_action_t storage_action_close(void *storage, surf_file_t fp)
 {
   char *filename = fp->name;
+  XBT_DEBUG("\tClose file '%s' size '%f'",filename,fp->content->stat.size);
   free(fp->name);
   fp->content = NULL;
   xbt_free(fp);
   surf_action_t action = storage_action_execute(storage,0, CLOSE);
-  XBT_DEBUG("\tClose file '%s'",filename);
   return action;
 }
 
-static surf_action_t storage_action_read(void *storage, void* ptr, size_t size, size_t nmemb, surf_file_t stream)
+static surf_action_t storage_action_read(void *storage, void* ptr, double size, size_t nmemb, surf_file_t stream)
 {
-  char *filename = stream->name;
   surf_stat_t content = stream->content;
-  XBT_DEBUG("\tRead file '%s' size '%zu/%zu'",filename,size,content->stat.size);
   if(size > content->stat.size)
     size = content->stat.size;
   surf_action_t action = storage_action_execute(storage,size,READ);
@@ -96,35 +160,46 @@ static surf_action_t storage_action_write(void *storage, const void* ptr, size_t
 {
   char *filename = stream->name;
   surf_stat_t content = stream->content;
-  XBT_DEBUG("\tWrite file '%s' size '%zu/%zu'",filename,size,content->stat.size);
+  XBT_DEBUG("\tWrite file '%s' size '%zu/%f'",filename,size,content->stat.size);
 
   surf_action_t action = storage_action_execute(storage,size,WRITE);
-  content->stat.size += size;
+  action->file = stream;
+
+  // If the storage is full
+  if(((storage_t)storage)->used_size==((storage_t)storage)->size) {
+    storage_action_state_set((surf_action_t) action, SURF_ACTION_FAILED);
+  }
   return action;
 }
 
-static surf_action_t storage_action_execute (void *storage, size_t size, e_surf_action_storage_type_t type)
+static surf_action_t storage_action_execute (void *storage, double size, e_surf_action_storage_type_t type)
 {
   surf_action_storage_t action = NULL;
   storage_t STORAGE = storage;
 
-  XBT_IN("(%s,%zu)", surf_resource_name(STORAGE), size);
+  XBT_IN("(%s,%f)", surf_resource_name(STORAGE), size);
   action =
       surf_action_new(sizeof(s_surf_action_storage_t), size, surf_storage_model,
           STORAGE->state_current != SURF_RESOURCE_ON);
 
+  // Save the storage on action
+  action->storage = storage;
   GENERIC_LMM_ACTION(action).suspended = 0;     /* Should be useless because of the
                                                    calloc but it seems to help valgrind... */
 
   GENERIC_LMM_ACTION(action).variable =
       lmm_variable_new(storage_maxmin_system, action, 1.0, -1.0 , 3);
 
+  // Must be less than the max bandwidth for all actions
+  lmm_expand(storage_maxmin_system, STORAGE->constraint,
+             GENERIC_LMM_ACTION(action).variable, 1.0);
+
   switch(type) {
   case OPEN:
   case CLOSE:
   case STAT:
-    lmm_expand(storage_maxmin_system, STORAGE->constraint,
-               GENERIC_LMM_ACTION(action).variable, 1.0);
+  case UNLINK:
+  case LS:
     break;
   case READ:
     lmm_expand(storage_maxmin_system, STORAGE->constraint_read,
@@ -133,14 +208,15 @@ static surf_action_t storage_action_execute (void *storage, size_t size, e_surf_
   case WRITE:
     lmm_expand(storage_maxmin_system, STORAGE->constraint_write,
                GENERIC_LMM_ACTION(action).variable, 1.0);
+    xbt_dynar_push(((storage_t)storage)->write_actions,&action);
     break;
   }
-
+  action->type = type;
   XBT_OUT();
   return (surf_action_t) action;
 }
 
-static void* storage_create_resource(const char* id, const char* model,const char* type_id)
+static void* storage_create_resource(const char* id, const char* model,const char* type_id,const char* content_name)
 {
   storage_t storage = NULL;
 
@@ -151,25 +227,33 @@ static void* storage_create_resource(const char* id, const char* model,const cha
           surf_storage_model, id,NULL);
 
   storage->state_current = SURF_RESOURCE_ON;
+  storage->used_size = 0;
+  storage->size = 0;
+  storage->write_actions = xbt_dynar_new(sizeof(char *),NULL);
 
   storage_type_t storage_type = xbt_lib_get_or_null(storage_type_lib, type_id,ROUTING_STORAGE_TYPE_LEVEL);
-  size_t Bread  = atof(xbt_dict_get(storage_type->properties,"Bread"));
-  size_t Bwrite = atof(xbt_dict_get(storage_type->properties,"Bwrite"));
-  size_t Bconnection   = atof(xbt_dict_get(storage_type->properties,"Bconnection"));
-  XBT_INFO("Create resource with Bconnection '%zu' Bread '%zu' Bwrite '%zu'",Bconnection,Bread,Bwrite);
+  double Bread  = atof(xbt_dict_get(storage_type->properties,"Bread"));
+  double Bwrite = atof(xbt_dict_get(storage_type->properties,"Bwrite"));
+  double Bconnection   = atof(xbt_dict_get(storage_type->properties,"Bconnection"));
+  XBT_DEBUG("Create resource with Bconnection '%f' Bread '%f' Bwrite '%f' and Size '%ld'",Bconnection,Bread,Bwrite,storage_type->size);
   storage->constraint       = lmm_constraint_new(storage_maxmin_system, storage, Bconnection);
   storage->constraint_read  = lmm_constraint_new(storage_maxmin_system, storage, Bread);
   storage->constraint_write = lmm_constraint_new(storage_maxmin_system, storage, Bwrite);
+  storage->content = parse_storage_content((char*)content_name,&(storage->used_size));
+  storage->size = storage_type->size;
 
   xbt_lib_set(storage_lib, id, SURF_STORAGE_LEVEL, storage);
 
-  XBT_DEBUG("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' \n\t\tmodel '%s' \n\t\tproperties '%p'\n\t\tBread '%zu'\n",
+  XBT_DEBUG("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' \n\t\tmodel '%s' \n\t\tproperties '%p'\n\t\tBread '%f'\n",
       id,
       model,
       type_id,
       storage_type->properties,
       Bread);
 
+  if(!storage_list) storage_list=xbt_dynar_new(sizeof(char *),NULL);
+  xbt_dynar_push(storage_list,&storage);
+
   return storage;
 }
 
@@ -181,6 +265,9 @@ static void storage_finalize(void)
   surf_model_exit(surf_storage_model);
   surf_storage_model = NULL;
 
+  if(storage_list)
+    xbt_dynar_free(&storage_list);
+
   xbt_swag_free
       (storage_running_action_set_that_does_not_need_being_checked);
   storage_running_action_set_that_does_not_need_being_checked = NULL;
@@ -192,17 +279,40 @@ static void storage_update_actions_state(double now, double delta)
   surf_action_storage_t next_action = NULL;
   xbt_swag_t running_actions = surf_storage_model->states.running_action_set;
 
+  // Update the disk usage
+  // Update the file size
+  // Foreach action of type write
   xbt_swag_foreach_safe(action, next_action, running_actions) {
+    if(action->type == WRITE)
+    {
+      double rate = lmm_variable_getvalue(GENERIC_LMM_ACTION(action).variable);
+      ((storage_t)(action->storage))->used_size += delta * rate; // disk usage
+      ((surf_action_t)action)->file->content->stat.size += delta * rate; // file size
+    }
+  }
+
+  xbt_swag_foreach_safe(action, next_action, running_actions) {
+
     double_update(&(GENERIC_ACTION(action).remains),
                   lmm_variable_getvalue(GENERIC_LMM_ACTION(action).variable) * delta);
+
     if (GENERIC_LMM_ACTION(action).generic_action.max_duration != NO_MAX_DURATION)
       double_update(&(GENERIC_ACTION(action).max_duration), delta);
-    if ((GENERIC_ACTION(action).remains <= 0) &&
-        (lmm_get_variable_weight(GENERIC_LMM_ACTION(action).variable) > 0)) {
+
+    if(GENERIC_ACTION(action).remains > 0 &&
+        lmm_get_variable_weight(GENERIC_LMM_ACTION(action).variable) > 0 &&
+        ((storage_t)action->storage)->used_size == ((storage_t)action->storage)->size)
+    {
+      GENERIC_ACTION(action).finish = surf_get_clock();
+      storage_action_state_set((surf_action_t) action, SURF_ACTION_FAILED);
+    } else if ((GENERIC_ACTION(action).remains <= 0) &&
+        (lmm_get_variable_weight(GENERIC_LMM_ACTION(action).variable) > 0))
+    {
       GENERIC_ACTION(action).finish = surf_get_clock();
       storage_action_state_set((surf_action_t) action, SURF_ACTION_DONE);
     } else if ((GENERIC_ACTION(action).max_duration != NO_MAX_DURATION) &&
-               (GENERIC_ACTION(action).max_duration <= 0)) {
+               (GENERIC_ACTION(action).max_duration <= 0))
+    {
       GENERIC_ACTION(action).finish = surf_get_clock();
       storage_action_state_set((surf_action_t) action, SURF_ACTION_DONE);
     }
@@ -215,9 +325,29 @@ static double storage_share_resources(double NOW)
 {
   XBT_DEBUG("storage_share_resources %f",NOW);
   s_surf_action_storage_t action;
-  return generic_maxmin_share_resources(surf_storage_model->states.running_action_set,
+  unsigned int i,j;
+  storage_t storage;
+  surf_action_storage_t write_action;
+
+  double min_completion = generic_maxmin_share_resources(surf_storage_model->states.running_action_set,
       xbt_swag_offset(action, generic_lmm_action.variable),
       storage_maxmin_system, lmm_solve);
+
+  double rate;
+  // Foreach disk
+  xbt_dynar_foreach(storage_list,i,storage)
+  {
+    rate = 0;
+    // Foreach write action on disk
+    xbt_dynar_foreach(storage->write_actions,j,write_action)
+    {
+      rate += lmm_variable_getvalue(write_action->generic_lmm_action.variable);
+    }
+    if(rate > 0)
+      min_completion = MIN(min_completion, (storage->size-storage->used_size)/rate);
+  }
+
+  return min_completion;
 }
 
 static int storage_resource_used(void *resource_id)
@@ -299,16 +429,24 @@ static void parse_storage_init(sg_platf_storage_cbarg_t storage)
                                     storage->type_id,
                                     ROUTING_STORAGE_TYPE_LEVEL);
   if(!stype) xbt_die("No storage type '%s'",storage->type_id);
-//  XBT_INFO("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' \n\t\tmodel '%s' \n\t\tcontent '%s'\n\t\tproperties '%p'\n",
-//      storage->id,
-//      ((storage_type_t) stype)->model,
-//      ((storage_type_t) stype)->type_id,
-//      ((storage_type_t) stype)->content,
-//      ((storage_type_t) stype)->properties);
 
- storage_create_resource(storage->id,
+  // if storage content is not specified use the content of storage_type if exist
+  if(!strcmp(storage->content,"") && strcmp(((storage_type_t) stype)->content,"")){
+    storage->content = ((storage_type_t) stype)->content;
+    XBT_DEBUG("For disk '%s' content is empty, use the content of storage type '%s'",storage->id,((storage_type_t) stype)->type_id);
+  }
+
+  XBT_DEBUG("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' \n\t\tmodel '%s' \n\t\tcontent '%s'\n\t\tproperties '%p'\n",
+      storage->id,
+      ((storage_type_t) stype)->model,
+      ((storage_type_t) stype)->type_id,
+      storage->content,
+      ((storage_type_t) stype)->properties);
+
+  storage_create_resource(storage->id,
      ((storage_type_t) stype)->model,
-     ((storage_type_t) stype)->type_id);
+     ((storage_type_t) stype)->type_id,
+     storage->content);
 }
 
 static void parse_mstorage_init(sg_platf_mstorage_cbarg_t mstorage)
@@ -366,6 +504,8 @@ static void surf_storage_model_init_internal(void)
   surf_storage_model->extension.storage.read = storage_action_read;
   surf_storage_model->extension.storage.write = storage_action_write;
   surf_storage_model->extension.storage.stat = storage_action_stat;
+  surf_storage_model->extension.storage.unlink = storage_action_unlink;
+  surf_storage_model->extension.storage.ls = storage_action_ls;
   surf_storage_model->extension.storage.create_resource = storage_create_resource;
 
   if (!storage_maxmin_system) {
@@ -393,9 +533,10 @@ static void storage_parse_storage(sg_platf_storage_cbarg_t storage)
 #endif
   xbt_assert(storage_type,"Reading a storage, type id \"%s\" does not exists", storage->type_id);
 
-  XBT_DEBUG("ROUTING Create a storage name '%s' with type_id '%s'",
+  XBT_DEBUG("ROUTING Create a storage name '%s' with type_id '%s' and content '%s'",
       storage->id,
-      storage->type_id);
+      storage->type_id,
+      storage->content);
 
   xbt_lib_set(storage_lib,
       storage->id,
@@ -414,8 +555,9 @@ static void free_storage_content(void *p)
   free(content);
 }
 
-static xbt_dict_t parse_storage_content(const char *filename)
+static xbt_dict_t parse_storage_content(char *filename, unsigned long *used_size)
 {
+  *used_size = 0;
   if ((!filename) || (strcmp(filename, "") == 0))
     return NULL;
 
@@ -435,19 +577,21 @@ static xbt_dict_t parse_storage_content(const char *filename)
   char date[12];
   char time[12];
   char path[1024];
-  int nb, size;
+  int nb;
+  unsigned long size;
 
   surf_stat_t content;
 
   while ((read = getline(&line, &len, file)) != -1) {
     content = xbt_new0(s_surf_stat_t,1);
-    if(sscanf(line,"%s %d %s %s %d %s %s %s",user_rights,&nb,user,group,&size,date,time,path)==8) {
+    if(sscanf(line,"%s %d %s %s %ld %s %s %s",user_rights,&nb,user,group,&size,date,time,path)==8) {
       content->stat.date = xbt_strdup(date);
       content->stat.group = xbt_strdup(group);
       content->stat.size = size;
       content->stat.time = xbt_strdup(time);
       content->stat.user = xbt_strdup(user);
       content->stat.user_rights = xbt_strdup(user_rights);
+      *used_size += content->stat.size;
       xbt_dict_set(parse_content,path,content,NULL);
     } else {
       xbt_die("Be sure of passing a good format for content file.\n");
@@ -457,7 +601,6 @@ static xbt_dict_t parse_storage_content(const char *filename)
   }
   if (line)
       free(line);
-
   fclose(file);
   return parse_content;
 }
@@ -470,14 +613,14 @@ static void storage_parse_storage_type(sg_platf_storage_type_cbarg_t storage_typ
   storage_type_t stype = xbt_new0(s_storage_type_t, 1);
   stype->model = xbt_strdup(storage_type->model);
   stype->properties = storage_type->properties;
-  stype->content = parse_storage_content(storage_type->content);
+  stype->content = xbt_strdup(storage_type->content);
   stype->type_id = xbt_strdup(storage_type->id);
+  stype->size = storage_type->size * 1000000000; /* storage_type->size is in Gbytes and stype->sizeis in bytes */
 
-  XBT_DEBUG("ROUTING Create a storage type id '%s' with model '%s' content '%s' and properties '%p'",
+  XBT_DEBUG("ROUTING Create a storage type id '%s' with model '%s' content '%s'",
       stype->type_id,
       stype->model,
-      storage_type->content,
-      stype->properties);
+      storage_type->content);
 
   xbt_lib_set(storage_type_lib,
       stype->type_id,
@@ -534,11 +677,21 @@ static XBT_INLINE void routing_storage_type_free(void *r)
   storage_type_t stype = r;
   free(stype->model);
   free(stype->type_id);
+  free(stype->content);
   xbt_dict_free(&(stype->properties));
-  xbt_dict_free(&(stype->content));
   free(stype);
 }
 
+static XBT_INLINE void surf_storage_resource_free(void *r)
+{
+  // specific to storage
+  storage_t storage = r;
+  xbt_dict_free(&storage->content);
+  xbt_dynar_free(&storage->write_actions);
+  // generic resource
+  surf_resource_free(r);
+}
+
 static XBT_INLINE void routing_storage_host_free(void *r)
 {
   xbt_dynar_t dyn = r;
@@ -550,7 +703,7 @@ void storage_register_callbacks() {
   ROUTING_STORAGE_LEVEL = xbt_lib_add_level(storage_lib,xbt_free);
   ROUTING_STORAGE_HOST_LEVEL = xbt_lib_add_level(storage_lib,routing_storage_host_free);
   ROUTING_STORAGE_TYPE_LEVEL = xbt_lib_add_level(storage_type_lib,routing_storage_type_free);
-  SURF_STORAGE_LEVEL = xbt_lib_add_level(storage_lib,surf_resource_free);
+  SURF_STORAGE_LEVEL = xbt_lib_add_level(storage_lib,surf_storage_resource_free);
 
   sg_platf_storage_add_cb(storage_parse_storage);
   sg_platf_mstorage_add_cb(storage_parse_mstorage);
