@@ -6,12 +6,14 @@
 
 /* surf_config: configuration infrastructure for the simulation world       */
 
+#include "xbt/misc.h"
 #include "xbt/config.h"
 #include "xbt/log.h"
 #include "xbt/str.h"
 #include "surf/surf_private.h"
-#include "surf/surf_routing.h"	/* COORD_HOST_LEVEL and COORD_ASR_LEVEL */
+#include "surf/surf_routing.h"  /* COORD_HOST_LEVEL and COORD_ASR_LEVEL */
 #include "simgrid/simix.h"
+#include "mc/mc.h" /* configuration callbacks of model-checking */
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_config, surf,
                                 "About the configuration of surf (and the rest of the simulation)");
@@ -38,6 +40,9 @@ static void surf_config_cmd_line(int *argc, char **argv)
       xbt_cfg_help(_surf_cfg_set);
       printf(
 "\n"
+"Each of these configurations can be used by adding\n"
+"    --cfg=<option name>:<option value>\n"
+"to the command line.\n"
 "You can also use --help-models to see the details of all models known by this simulator.\n"
 #ifdef HAVE_TRACING
 "\n"
@@ -81,16 +86,17 @@ static void surf_config_cmd_line(int *argc, char **argv)
 }
 
 
-int _surf_init_status = 0;      /* 0: beginning of time;
-                                   1: pre-inited (cfg_set created);
-                                   2: inited (running) */
+int _surf_init_status = 0;      /* 0: beginning of time (config cannot be changed yet);
+                                   1: initialized: cfg_set created (config can now be changed);
+                                   2: configured: command line parsed and config part of platform file was integrated also, platform construction ongoing or done.
+                                      (Config cannot be changed anymore!) */
 
 /* callback of the workstation/model variable */
 static void _surf_cfg_cb__workstation_model(const char *name, int pos)
 {
   char *val;
 
-  xbt_assert(_surf_init_status < 2,
+  xbt_assert(_surf_init_status == 1,
               "Cannot change the model after the initialization");
 
   val = xbt_cfg_get_string(_surf_cfg_set, name);
@@ -109,7 +115,7 @@ static void _surf_cfg_cb__cpu_model(const char *name, int pos)
 {
   char *val;
 
-  xbt_assert(_surf_init_status < 2,
+  xbt_assert(_surf_init_status == 1,
               "Cannot change the model after the initialization");
 
   val = xbt_cfg_get_string(_surf_cfg_set, name);
@@ -128,7 +134,7 @@ static void _surf_cfg_cb__optimization_mode(const char *name, int pos)
 {
   char *val;
 
-  xbt_assert(_surf_init_status < 2,
+  xbt_assert(_surf_init_status == 1,
               "Cannot change the model after the initialization");
 
   val = xbt_cfg_get_string(_surf_cfg_set, name);
@@ -147,7 +153,7 @@ static void _surf_cfg_cb__storage_mode(const char *name, int pos)
 {
   char *val;
 
-  xbt_assert(_surf_init_status < 2,
+  xbt_assert(_surf_init_status == 1,
               "Cannot change the model after the initialization");
 
   val = xbt_cfg_get_string(_surf_cfg_set, name);
@@ -166,7 +172,7 @@ static void _surf_cfg_cb__network_model(const char *name, int pos)
 {
   char *val;
 
-  xbt_assert(_surf_init_status < 2,
+  xbt_assert(_surf_init_status == 1,
               "Cannot change the model after the initialization");
 
   val = xbt_cfg_get_string(_surf_cfg_set, name);
@@ -227,11 +233,11 @@ static void _surf_cfg_cb_model_check(const char *name, int pos)
 {
   _surf_do_model_check = xbt_cfg_get_int(_surf_cfg_set, name);
 
+#ifndef HAVE_MC
   if (_surf_do_model_check) {
-    /* Tell modules using mallocators that they shouldn't. MC don't like them */
-    xbt_fifo_preinit();
-    xbt_dict_preinit();
+    xbt_die("You tried to activate the model-checking from the command line, but it was not compiled in. Change your settings in cmake, recompile and try again");
   }
+#endif
 }
 
 extern int _surf_do_verbose_exit;
@@ -242,8 +248,7 @@ static void _surf_cfg_cb_verbose_exit(const char *name, int pos)
 }
 
 
-static void _surf_cfg_cb_context_factory(const char *name, int pos)
-{
+static void _surf_cfg_cb_context_factory(const char *name, int pos) {
   smx_context_factory_name = xbt_cfg_get_string(_surf_cfg_set, name);
 }
 
@@ -331,8 +336,6 @@ void surf_config_init(int *argc, char **argv)
 
   /* Create the configuration support */
   if (_surf_init_status == 0) { /* Only create stuff if not already inited */
-    _surf_init_status = 1;
-
     sprintf(description,
             "The model to use for the CPU. Possible values: ");
     p = description;
@@ -373,6 +376,23 @@ void surf_config_init(int *argc, char **argv)
                      &default_value, 1, 1, &_surf_cfg_cb__storage_mode,
                      NULL);
 
+    /* ********************************************************************* */
+    /* TUTORIAL: New model                                                   */
+    sprintf(description,
+            "The model to use for the New model. Possible values: ");
+    p = description;
+    while (*(++p) != '\0');
+    for (i = 0; surf_new_model_description[i].name; i++)
+      p += sprintf(p, "%s%s", (i == 0 ? "" : ", "),
+                   surf_new_model_description[i].name);
+    sprintf(p,
+            ".\n       (use 'help' as a value to see the long description of each model)");
+    default_value = xbt_strdup("default");
+    xbt_cfg_register(&_surf_cfg_set, "new_model/model", description, xbt_cfgelm_string,
+                     &default_value, 1, 1, &_surf_cfg_cb__storage_mode,
+                     NULL);
+    /* ********************************************************************* */
+
     sprintf(description,
             "The model to use for the network. Possible values: ");
     p = description;
@@ -468,18 +488,36 @@ void surf_config_init(int *argc, char **argv)
                      xbt_cfgelm_int, &default_value_int, 0, 1,
                      NULL, NULL);
 
-    /* do model-check */
+#ifdef HAVE_MC
+    /* do model-checking */
     default_value_int = 0;
     xbt_cfg_register(&_surf_cfg_set, "model-check",
-                     "Activate the model-checking of the \"simulated\" system (EXPERIMENTAL -- msg only for now)",
+                     "Verify the system through model-checking instead of simulating it (EXPERIMENTAL)",
                      xbt_cfgelm_int, &default_value_int, 0, 1,
                      _surf_cfg_cb_model_check, NULL);
+
+    /* do stateful model-checking */
+    default_value_int = 0;
+    xbt_cfg_register(&_surf_cfg_set, "model-check/checkpoint",
+                     "Specify the amount of steps between checkpoints during stateful model-checking (default: 0 => stateless verification). "
+                     "If value=1, one checkpoint is saved for each step => faster verification, but huge memory consumption; higher values are good compromises between speed and memory consumption.",
+                     xbt_cfgelm_int, &default_value_int, 0, 1,
+                     _mc_cfg_cb_checkpoint, NULL);
     
-    /*
-       FIXME: this function is not setting model-check to it's default value because
-       internally it calls to variable->cb_set that in this case is the function 
-       _surf_cfg_cb_model_check which sets it's value to 1 (instead of the default value 0)
-       xbt_cfg_set_int(_surf_cfg_set, "model-check", default_value_int); */
+    /* do liveness model-checking */
+    default_value = xbt_strdup("");
+    xbt_cfg_register(&_surf_cfg_set, "model-check/property",
+                     "Specify the name of the file containing the property. It must be the result of the ltl2ba program.",
+                     xbt_cfgelm_string, &default_value, 0, 1,
+                     _mc_cfg_cb_property, NULL);
+
+    /* Specify the kind of model-checking reduction */
+    default_value = xbt_strdup("unset");
+    xbt_cfg_register(&_surf_cfg_set, "model-check/reduction",
+                     "Specify the kind of exploration reduction (either none or DPOR)",
+                     xbt_cfgelm_string, &default_value, 0, 1,
+                     _mc_cfg_cb_reduce, NULL);
+#endif
 
     /* do verbose-exit */
     default_value_int = 1;
@@ -588,11 +626,17 @@ void surf_config_init(int *argc, char **argv)
                      xbt_cfgelm_double, &default_threshold, 1, 1, NULL,
                      NULL);
 
+    int default_small_messages_threshold = 0;
+    xbt_cfg_register(&_surf_cfg_set, "smpi/async_small_thres",
+                     "Maximal size of messages that are to be sent asynchronously, without waiting for the receiver",
+                     xbt_cfgelm_int, &default_small_messages_threshold, 1, 1, NULL,
+                     NULL);
+
     //For smpi/bw_factor and smpi/lat_factor
     //Default value have to be "threshold0:value0;threshold1:value1;...;thresholdN:valueN"
     //test is if( size >= thresholdN ) return valueN;
     //Values can be modified with command line --cfg=smpi/bw_factor:"threshold0:value0;threshold1:value1;...;thresholdN:valueN"
-    //	or with tag config put line <prop id="smpi/bw_factor" value="threshold0:value0;threshold1:value1;...;thresholdN:valueN"></prop>
+    //  or with tag config put line <prop id="smpi/bw_factor" value="threshold0:value0;threshold1:value1;...;thresholdN:valueN"></prop>
     xbt_cfg_register(&_surf_cfg_set, "smpi/bw_factor",
                      "Bandwidth factors for smpi.",
                      xbt_cfgelm_string, NULL, 1, 1, NULL,
@@ -617,8 +661,10 @@ void surf_config_init(int *argc, char **argv)
       xbt_cfg_setdefault_string(_surf_cfg_set, "path", initial_path);
     }
 
+    _surf_init_status = 1;
 
     surf_config_cmd_line(argc, argv);
+
   } else {
     XBT_WARN("Call to surf_config_init() after initialization ignored");
   }
@@ -655,14 +701,14 @@ void surf_config_models_setup()
    */
 
   if((!xbt_cfg_is_default_value(_surf_cfg_set, "network/model") ||
-	  !xbt_cfg_is_default_value(_surf_cfg_set, "cpu/model")) &&
-	  xbt_cfg_is_default_value(_surf_cfg_set, "workstation/model"))
+    !xbt_cfg_is_default_value(_surf_cfg_set, "cpu/model")) &&
+    xbt_cfg_is_default_value(_surf_cfg_set, "workstation/model"))
   {
-	    const char *val = "compound";
-	    XBT_INFO
-	        ("Switching workstation model to compound since you changed the network and/or cpu model(s)");
-	    xbt_cfg_set_string(_surf_cfg_set, "workstation/model", val);
-	    workstation_model_name = (char *) "compound";
+      const char *val = "compound";
+      XBT_INFO
+          ("Switching workstation model to compound since you changed the network and/or cpu model(s)");
+      xbt_cfg_set_string(_surf_cfg_set, "workstation/model", val);
+      workstation_model_name = (char *) "compound";
   }
 
   XBT_DEBUG("Workstation model: %s", workstation_model_name);
@@ -695,4 +741,14 @@ void surf_config_models_setup()
   XBT_DEBUG("Call storage_model_init");
   storage_id = find_model_description(surf_storage_model_description, storage_model_name);
   surf_storage_model_description[storage_id].model_init_preparse();
+
+  /* ********************************************************************* */
+  /* TUTORIAL: New model                                                   */
+  int new_model_id = -1;
+  char *new_model_name = NULL;
+  new_model_name = xbt_cfg_get_string(_surf_cfg_set, "new_model/model");
+  XBT_DEBUG("Call new model_init");
+  new_model_id = find_model_description(surf_new_model_description, new_model_name);
+  surf_new_model_description[new_model_id].model_init_preparse();
+  /* ********************************************************************* */
 }
