@@ -30,6 +30,7 @@ extern double sg_gtnets_jitter;
 extern int sg_gtnets_jitter_seed;
 #endif
 
+extern xbt_dict_t watched_hosts_lib;
 
 extern const char *surf_action_state_names[6];
 
@@ -62,7 +63,7 @@ double generic_maxmin_share_resources(xbt_swag_t running_actions,
                                       size_t offset,
                                       lmm_system_t sys,
                                       void (*solve) (lmm_system_t));
-void generic_update_action_remaining_lazy( surf_action_t action, double now);
+double generic_share_resources_lazy(double now, surf_model_t model);
 
 /* Generic functions common to all models */
 void surf_action_init(void);
@@ -92,6 +93,9 @@ void surf_action_set_category(surf_action_t action,
                                     const char *category);
 #endif
 double surf_action_get_remains(surf_action_t action);
+void generic_update_action_remaining_lazy( surf_action_lmm_t action, double now);
+void generic_update_actions_state_lazy(double now, double delta, surf_model_t model);
+void generic_update_actions_state_full(double now, double delta, surf_model_t model);
 
 FILE *surf_fopen(const char *name, const char *mode);
 
@@ -128,7 +132,7 @@ typedef struct s_routing_edge {
   e_surf_network_element_type_t rc_type;
   int id;
   char *name;
-} s_network_element_t;
+} s_routing_edge_t;
 
 /*
  * Link of lenght 1, alongside with its source and destination. This is mainly usefull in the bindings to gtnets and ns3
@@ -150,12 +154,6 @@ typedef struct s_model_type {
   void (*end) (AS_t as);
 } s_routing_model_description_t, *routing_model_description_t;
 
-typedef struct s_route {
-  xbt_dynar_t link_list;
-  sg_routing_edge_t src_gateway;
-  sg_routing_edge_t dst_gateway;
-} s_route_t, *route_t;
-
 /* This enum used in the routing structure helps knowing in which situation we are. */
 typedef enum {
   SURF_ROUTING_NULL = 0,   /**< Undefined type                                   */
@@ -165,7 +163,7 @@ typedef enum {
 
 typedef struct s_as {
   xbt_dynar_t index_network_elm;
-  xbt_dict_t bypassRoutes;		/* store bypass routes */
+  xbt_dict_t bypassRoutes;    /* store bypass routes */
   routing_model_description_t model_desc;
   e_surf_routing_hierarchy_t hierarchy;
   char *name;
@@ -174,10 +172,10 @@ typedef struct s_as {
   sg_routing_edge_t net_elem;
   xbt_dynar_t link_up_down_list;
 
-  void (*get_route_and_latency) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, route_t into, double *latency);
+  void (*get_route_and_latency) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, sg_platf_route_cbarg_t into, double *latency);
 
   xbt_dynar_t(*get_onelink_routes) (AS_t as);
-  route_t(*get_bypass_route) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, double *lat);
+  sg_platf_route_cbarg_t(*get_bypass_route) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, double *lat);
   void (*finalize) (AS_t as);
 
 
@@ -187,12 +185,10 @@ typedef struct s_as {
    * Of course, only the routing model of this AS is informed, not every ones */
   int (*parse_PU) (AS_t as, sg_routing_edge_t elm); /* A host or a router, whatever */
   int (*parse_AS) (AS_t as, sg_routing_edge_t elm);
-  void (*parse_route) (AS_t as, const char *src,
-                     const char *dst, route_t route);
-  void (*parse_ASroute) (AS_t as, const char *src,
-                       const char *dst, route_t route);
-  void (*parse_bypassroute) (AS_t as, const char *src,
-                           const char *dst, route_t e_route);
+  void (*parse_route) (AS_t as, sg_platf_route_cbarg_t route);
+  void (*parse_ASroute) (AS_t as, sg_platf_route_cbarg_t route);
+  void (*parse_bypassroute) (AS_t as, sg_platf_route_cbarg_t e_route);
+
 } s_as_t;
 
 struct s_routing_platf {
@@ -205,8 +201,13 @@ struct s_routing_platf {
 XBT_PUBLIC(void) routing_model_create(void *loopback);
 XBT_PUBLIC(void) routing_exit(void);
 XBT_PUBLIC(void) storage_register_callbacks(void);
+/* ***************************************** */
+/* TUTORIAL: New TAG                         */
+XBT_PUBLIC(void) gpu_register_callbacks(void);
+/* ***************************************** */
+
 XBT_PUBLIC(void) routing_register_callbacks(void);
-XBT_PUBLIC(void) generic_free_route(route_t route); // FIXME rename to routing_route_free
+XBT_PUBLIC(void) generic_free_route(sg_platf_route_cbarg_t route); // FIXME rename to routing_route_free
  // FIXME: make previous function private to routing again?
 
 
@@ -216,8 +217,6 @@ XBT_PUBLIC(void) routing_get_route_and_latency(sg_routing_edge_t src, sg_routing
 /**
  * Resource protected methods
  */
-static XBT_INLINE xbt_dict_t surf_resource_properties(const void *resource);
-
 XBT_PUBLIC(void) surfxml_bufferstack_push(int new);
 XBT_PUBLIC(void) surfxml_bufferstack_pop(int new);
 
