@@ -4,14 +4,14 @@
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include <pcre.h>               /* regular expression library */
-
 #include "simgrid/platf_interface.h"    // platform creation API internal interface
 
 #include "surf_routing_private.h"
 #include "surf/surf_routing.h"
 #include "surf/surfxml_parse_values.h"
 
+#include <pcre.h>               /* regular expression library */
+
 /**
  * @ingroup SURF_build_api
  * @brief A library containing all known workstations
@@ -27,6 +27,8 @@ int SD_HOST_LEVEL;              //Simdag level
 int COORD_HOST_LEVEL=0;         //Coordinates level
 int NS3_HOST_LEVEL;             //host node for ns3
 
+xbt_dict_t watched_hosts_lib;
+
 /**
  * @ingroup SURF_build_api
  * @brief A library containing all known links
@@ -40,6 +42,7 @@ xbt_lib_t as_router_lib;
 int ROUTING_ASR_LEVEL;          //Routing level
 int COORD_ASR_LEVEL;            //Coordinates level
 int NS3_ASR_LEVEL;              //host node for ns3
+int ROUTING_PROP_ASR_LEVEL;     //Where the properties are stored
 
 static xbt_dict_t random_value = NULL;
 
@@ -58,14 +61,8 @@ routing_platf_t routing_platf = NULL;
 AS_t current_routing = NULL;
 
 /* global parse functions */
-xbt_dynar_t parsed_link_list = NULL;   /* temporary store of current list link of a route */
 extern xbt_dynar_t mount_list;
 
-static const char *src = NULL;  /* temporary store the source name of a route */
-static const char *dst = NULL;  /* temporary store the destination name of a route */
-static char *gw_src = NULL;     /* temporary store the gateway source name of a route */
-static char *gw_dst = NULL;     /* temporary store the gateway destination name of a route */
-
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_route, surf, "Routing part of surf");
 
 static void routing_parse_peer(sg_platf_peer_cbarg_t peer);     /* peer bypass */
@@ -109,6 +106,37 @@ struct s_model_type routing_models[] = {
   {NULL, NULL, NULL, NULL}
 };
 
+/**
+ * \brief Add a "host_link" to the network element list
+ */
+static void parse_S_host_link(sg_platf_host_link_cbarg_t host)
+{
+  sg_routing_edge_t info = NULL;
+  info = xbt_lib_get_or_null(host_lib, host->id, ROUTING_HOST_LEVEL);
+  xbt_assert(info, "Host '%s' not found!",host->id);
+  xbt_assert(current_routing->model_desc == &routing_models[SURF_MODEL_CLUSTER] ||
+      current_routing->model_desc == &routing_models[SURF_MODEL_VIVALDI],
+      "You have to be in model Cluster to use tag host_link!");
+
+  s_surf_parsing_link_up_down_t link_up_down;
+  link_up_down.link_up = xbt_lib_get_or_null(link_lib, host->link_up, SURF_LINK_LEVEL);
+  link_up_down.link_down = xbt_lib_get_or_null(link_lib, host->link_down, SURF_LINK_LEVEL);
+
+  xbt_assert(link_up_down.link_up, "Link '%s' not found!",host->link_up);
+  xbt_assert(link_up_down.link_down, "Link '%s' not found!",host->link_down);
+
+  if(!current_routing->link_up_down_list)
+    current_routing->link_up_down_list = xbt_dynar_new(sizeof(s_surf_parsing_link_up_down_t),NULL);
+
+  // If dynar is is greater than edge id and if the host_link is already defined
+  if(xbt_dynar_length(current_routing->link_up_down_list) > info->id &&
+      xbt_dynar_get_as(current_routing->link_up_down_list,info->id,void*))
+    xbt_die("Host_link for '%s' is already defined!",host->id);
+
+  XBT_DEBUG("Push Host_link for host '%s' to position %d",info->name,info->id);
+  xbt_dynar_set_as(current_routing->link_up_down_list,info->id,s_surf_parsing_link_up_down_t,link_up_down);
+}
+
 /**
  * \brief Add a "host" to the network element list
  */
@@ -120,7 +148,7 @@ static void parse_S_host(sg_platf_host_cbarg_t host)
   xbt_assert(!xbt_lib_get_or_null(host_lib, host->id, ROUTING_HOST_LEVEL),
              "Reading a host, processing unit \"%s\" already exists", host->id);
 
-  info = xbt_new0(s_network_element_t, 1);
+  info = xbt_new0(s_routing_edge_t, 1);
   info->rc_component = current_routing;
   info->rc_type = SURF_NETWORK_ELEMENT_HOST;
   info->name = xbt_strdup(host->id);
@@ -165,7 +193,7 @@ static void parse_S_router(sg_platf_router_cbarg_t router)
              "Reading a router, processing unit \"%s\" already exists",
              router->id);
 
-  info = xbt_new0(s_network_element_t, 1);
+  info = xbt_new0(s_routing_edge_t, 1);
   info->rc_component = current_routing;
   info->rc_type = SURF_NETWORK_ELEMENT_ROUTER;
   info->name = xbt_strdup(router->id);
@@ -173,7 +201,7 @@ static void parse_S_router(sg_platf_router_cbarg_t router)
   xbt_lib_set(as_router_lib, router->id, ROUTING_ASR_LEVEL, (void *) info);
   XBT_DEBUG("Having set name '%s' id '%d'",router->id,info->id);
 
-  if (strcmp(router->coord, "")) {
+  if (router->coord && strcmp(router->coord, "")) {
     unsigned int cursor;
     char*str;
 
@@ -193,173 +221,108 @@ static void parse_S_router(sg_platf_router_cbarg_t router)
   }
 }
 
-
-/**
- * \brief Set the end points for a route
- */
-static void routing_parse_S_route(void)
-{
-  src = A_surfxml_route_src;
-  dst = A_surfxml_route_dst;
-  xbt_assert(strlen(src) > 0 || strlen(dst) > 0,
-             "Missing end-points while defining route \"%s\"->\"%s\"",
-             src, dst);
-  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
-}
-
-/**
- * \brief Set the end points and gateways for a ASroute
- */
-static void routing_parse_S_ASroute(void)
-{
-  src = A_surfxml_ASroute_src;
-  dst = A_surfxml_ASroute_dst;
-  gw_src = A_surfxml_ASroute_gw_src;
-  gw_dst = A_surfxml_ASroute_gw_dst;
-  xbt_assert(strlen(src) > 0 || strlen(dst) > 0 || strlen(gw_src) > 0 || strlen(gw_dst) > 0,
-             "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
-             src, dst,gw_src,gw_dst);
-  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
-}
-
-/**
- * \brief Set the end points for a bypassRoute
- */
-static void routing_parse_S_bypassRoute(void)
-{
-  src = A_surfxml_bypassRoute_src;
-  dst = A_surfxml_bypassRoute_dst;
-  gw_src = NULL;
-  gw_dst = NULL;
-  xbt_assert(strlen(src) > 0 || strlen(dst) > 0 || strlen(gw_src) > 0 || strlen(gw_dst) > 0,
-             "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
-             src, dst,gw_src,gw_dst);
-  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
-}
-
-/**
- * \brief Set the end points for a bypassASroute
- */
-static void routing_parse_S_bypassASroute(void)
-{
-  src = A_surfxml_bypassASroute_src;
-  dst = A_surfxml_bypassASroute_dst;
-  gw_src = A_surfxml_bypassASroute_gw_src;
-  gw_dst = A_surfxml_bypassASroute_gw_dst;
-  xbt_assert(strlen(src) > 0 || strlen(dst) > 0 || strlen(gw_src) > 0 || strlen(gw_dst) > 0,
-             "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
-             src, dst,gw_src,gw_dst);
-  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
-}
-/**
- * \brief Set a new link on the actual list of link for a route or ASroute from XML
- */
-
-static void routing_parse_link_ctn(void)
-{
-  char *link_id;
-  switch (A_surfxml_link_ctn_direction) {
-  case AU_surfxml_link_ctn_direction:
-  case A_surfxml_link_ctn_direction_NONE:
-    link_id = xbt_strdup(A_surfxml_link_ctn_id);
-    break;
-  case A_surfxml_link_ctn_direction_UP:
-    link_id = bprintf("%s_UP", A_surfxml_link_ctn_id);
-    break;
-  case A_surfxml_link_ctn_direction_DOWN:
-    link_id = bprintf("%s_DOWN", A_surfxml_link_ctn_id);
-    break;
-  }
-  xbt_dynar_push(parsed_link_list, &link_id);
-}
-
 /**
  * \brief Store the route by calling the set_route function of the current routing component
  */
-static void routing_parse_E_route(void)
+static void parse_E_route(sg_platf_route_cbarg_t route)
 {
-  route_t route = xbt_new0(s_route_t, 1);
-  route->link_list = parsed_link_list;
   xbt_assert(current_routing->parse_route,
              "no defined method \"set_route\" in \"%s\"",
              current_routing->name);
-  current_routing->parse_route(current_routing, src, dst, route);
-  generic_free_route(route);
-  parsed_link_list = NULL;
-  src = NULL;
-  dst = NULL;
+
+  current_routing->parse_route(current_routing, route);
 }
 
 /**
  * \brief Store the ASroute by calling the set_ASroute function of the current routing component
  */
-static void routing_parse_E_ASroute(void)
+static void parse_E_ASroute(sg_platf_route_cbarg_t ASroute)
 {
-  route_t e_route = xbt_new0(s_route_t, 1);
-  e_route->link_list = parsed_link_list;
-
-  if (!strcmp(current_routing->model_desc->name,"RuleBased")) {
-    // DIRTY PERL HACK AHEAD: with the rulebased routing, the {src,dst}_gateway fields
-    // store the provided name instead of the entity directly (model_rulebased_parse_ASroute knows)
-    //
-    // This is because the user will provide something like "^AS_(.*)$" instead of the proper name of a given entity
-    e_route->src_gateway = (sg_routing_edge_t) gw_src;
-    e_route->dst_gateway = (sg_routing_edge_t) gw_dst;
-  } else {
-    e_route->src_gateway = sg_routing_edge_by_name_or_null(gw_src);
-    e_route->dst_gateway = sg_routing_edge_by_name_or_null(gw_dst);
-  }
   xbt_assert(current_routing->parse_ASroute,
              "no defined method \"set_ASroute\" in \"%s\"",
              current_routing->name);
-  current_routing->parse_ASroute(current_routing, src, dst, e_route);
-  generic_free_route(e_route);
-  parsed_link_list = NULL;
-  src = NULL;
-  dst = NULL;
-  gw_src = NULL;
-  gw_dst = NULL;
+  current_routing->parse_ASroute(current_routing, ASroute);
 }
 
 /**
  * \brief Store the bypass route by calling the set_bypassroute function of the current routing component
  */
-static void routing_parse_E_bypassRoute(void)
+static void parse_E_bypassRoute(sg_platf_route_cbarg_t route)
 {
-  route_t e_route = xbt_new0(s_route_t, 1);
-  e_route->link_list = parsed_link_list;
-
   xbt_assert(current_routing->parse_bypassroute,
              "Bypassing mechanism not implemented by routing '%s'",
              current_routing->name);
 
-  current_routing->parse_bypassroute(current_routing, src, dst, e_route);
-  parsed_link_list = NULL;
-  src = NULL;
-  dst = NULL;
-  gw_src = NULL;
-  gw_dst = NULL;
+  current_routing->parse_bypassroute(current_routing, route);
 }
+
 /**
  * \brief Store the bypass route by calling the set_bypassroute function of the current routing component
  */
-static void routing_parse_E_bypassASroute(void)
+static void parse_E_bypassASroute(sg_platf_route_cbarg_t ASroute)
 {
-  route_t e_route = xbt_new0(s_route_t, 1);
-  e_route->link_list = parsed_link_list;
-  e_route->src_gateway = sg_routing_edge_by_name_or_null(gw_src);
-  e_route->dst_gateway = sg_routing_edge_by_name_or_null(gw_dst);
   xbt_assert(current_routing->parse_bypassroute,
              "Bypassing mechanism not implemented by routing '%s'",
              current_routing->name);
-  current_routing->parse_bypassroute(current_routing, src, dst, e_route);
-  parsed_link_list = NULL;
-  src = NULL;
-  dst = NULL;
-  gw_src = NULL;
-  gw_dst = NULL;
+  current_routing->parse_bypassroute(current_routing, ASroute);
 }
 
+static void routing_parse_trace(sg_platf_trace_cbarg_t trace)
+{
+  tmgr_trace_t tmgr_trace;
+  if (!trace->file || strcmp(trace->file, "") != 0) {
+    tmgr_trace = tmgr_trace_new_from_file(trace->file);
+  } else if (strcmp(trace->pc_data, "") == 0) {
+    tmgr_trace = NULL;
+  } else {
+    tmgr_trace =
+          tmgr_trace_new_from_string(trace->id, trace->pc_data,
+                                     trace->periodicity);
+  }
+  xbt_dict_set(traces_set_list, trace->id, (void *) tmgr_trace, NULL);
+}
+
+static void routing_parse_trace_connect(sg_platf_trace_connect_cbarg_t trace_connect)
+{
+  xbt_assert(xbt_dict_get_or_null
+              (traces_set_list, trace_connect->trace),
+              "Cannot connect trace %s to %s: trace unknown",
+              trace_connect->trace,
+              trace_connect->element);
+
+  switch (trace_connect->kind) {
+  case SURF_TRACE_CONNECT_KIND_HOST_AVAIL:
+    xbt_dict_set(trace_connect_list_host_avail,
+        trace_connect->trace,
+        xbt_strdup(trace_connect->element), NULL);
+    break;
+  case SURF_TRACE_CONNECT_KIND_POWER:
+    xbt_dict_set(trace_connect_list_power, trace_connect->trace,
+        xbt_strdup(trace_connect->element), NULL);
+    break;
+  case SURF_TRACE_CONNECT_KIND_LINK_AVAIL:
+    xbt_dict_set(trace_connect_list_link_avail,
+        trace_connect->trace,
+        xbt_strdup(trace_connect->element), NULL);
+    break;
+  case SURF_TRACE_CONNECT_KIND_BANDWIDTH:
+    xbt_dict_set(trace_connect_list_bandwidth,
+        trace_connect->trace,
+        xbt_strdup(trace_connect->element), NULL);
+    break;
+  case SURF_TRACE_CONNECT_KIND_LATENCY:
+    xbt_dict_set(trace_connect_list_latency, trace_connect->trace,
+        xbt_strdup(trace_connect->element), NULL);
+    break;
+  default:
+    xbt_die("Cannot connect trace %s to %s: kind of trace unknown",
+        trace_connect->trace, trace_connect->element);
+    break;
+  }
+}
+
+extern int _surf_init_status; /* yay, this is an horrible hack */
+
 /**
  * \brief Make a new routing component to the platform
  *
@@ -373,38 +336,40 @@ static void routing_parse_E_bypassASroute(void)
  * @param AS_id name of this autonomous system. Must be unique in the platform
  * @param wanted_routing_type one of Full, Floyd, Dijkstra or similar. Full list in the variable routing_models, in src/surf/surf_routing.c
  */
-void routing_AS_begin(const char *AS_id, const char *wanted_routing_type)
+void routing_AS_begin(sg_platf_AS_cbarg_t AS)
 {
+  XBT_DEBUG("routing_AS_begin");
   AS_t new_as;
   routing_model_description_t model = NULL;
-  int cpt;
 
   xbt_assert(!xbt_lib_get_or_null
-             (as_router_lib, AS_id, ROUTING_ASR_LEVEL),
-             "The AS \"%s\" already exists", AS_id);
+             (as_router_lib, AS->id, ROUTING_ASR_LEVEL),
+             "The AS \"%s\" already exists", AS->id);
+
+  _surf_init_status = 2; /* horrible hack: direct access to the global controlling the level of configuration to prevent any further config */
 
   /* search the routing model */
-  for (cpt = 0; routing_models[cpt].name; cpt++)
-    if (!strcmp(wanted_routing_type, routing_models[cpt].name))
-      model = &routing_models[cpt];
-  /* if its not exist, error */
-  if (model == NULL) {
-    fprintf(stderr, "Routing model %s not found. Existing models:\n",
-            wanted_routing_type);
-    for (cpt = 0; routing_models[cpt].name; cpt++)
-      fprintf(stderr, "   %s: %s\n", routing_models[cpt].name,
-              routing_models[cpt].desc);
-    xbt_die("dying");
+  switch(AS->routing){
+    case A_surfxml_AS_routing_Cluster:       model = &routing_models[SURF_MODEL_CLUSTER];break;
+    case A_surfxml_AS_routing_Dijkstra:      model = &routing_models[SURF_MODEL_DIJKSTRA];break;
+    case A_surfxml_AS_routing_DijkstraCache: model = &routing_models[SURF_MODEL_DIJKSTRACACHE];break;
+    case A_surfxml_AS_routing_Floyd:         model = &routing_models[SURF_MODEL_FLOYD];break;
+    case A_surfxml_AS_routing_Full:          model = &routing_models[SURF_MODEL_FULL];break;
+    case A_surfxml_AS_routing_None:          model = &routing_models[SURF_MODEL_NONE];break;
+    case A_surfxml_AS_routing_RuleBased:     model = &routing_models[SURF_MODEL_RULEBASED];break;
+    case A_surfxml_AS_routing_Vivaldi:       model = &routing_models[SURF_MODEL_VIVALDI];break;
+    default: xbt_die("Not a valid model!!!");
+    break;
   }
 
   /* make a new routing component */
   new_as = (AS_t) model->create();
   new_as->model_desc = model;
   new_as->hierarchy = SURF_ROUTING_NULL;
-  new_as->name = xbt_strdup(AS_id);
+  new_as->name = xbt_strdup(AS->id);
 
   sg_routing_edge_t info = NULL;
-  info = xbt_new0(s_network_element_t, 1);
+  info = xbt_new0(s_routing_edge_t, 1);
 
   if (current_routing == NULL && routing_platf->root == NULL) {
 
@@ -415,15 +380,15 @@ void routing_AS_begin(const char *AS_id, const char *wanted_routing_type)
   } else if (current_routing != NULL && routing_platf->root != NULL) {
 
     xbt_assert(!xbt_dict_get_or_null
-               (current_routing->routing_sons, AS_id),
-               "The AS \"%s\" already exists", AS_id);
+               (current_routing->routing_sons, AS->id),
+               "The AS \"%s\" already exists", AS->id);
     /* it is a part of the tree */
     new_as->routing_father = current_routing;
     /* set the father behavior */
     if (current_routing->hierarchy == SURF_ROUTING_NULL)
       current_routing->hierarchy = SURF_ROUTING_RECURSIVE;
     /* add to the sons dictionary */
-    xbt_dict_set(current_routing->routing_sons, AS_id,
+    xbt_dict_set(current_routing->routing_sons, AS->id,
                  (void *) new_as, NULL);
     /* add to the father element list */
     info->id = current_routing->parse_AS(current_routing, (void *) info);
@@ -435,7 +400,7 @@ void routing_AS_begin(const char *AS_id, const char *wanted_routing_type)
   info->rc_type = SURF_NETWORK_ELEMENT_AS;
   info->name = new_as->name;
 
-  xbt_lib_set(as_router_lib, new_as->name, ROUTING_ASR_LEVEL,
+  xbt_lib_set(as_router_lib, info->name, ROUTING_ASR_LEVEL,
               (void *) info);
   XBT_DEBUG("Having set name '%s' id '%d'",new_as->name,info->id);
 
@@ -456,7 +421,7 @@ void routing_AS_begin(const char *AS_id, const char *wanted_routing_type)
  * even if you add stuff to a closed AS
  *
  */
-void routing_AS_end()
+void routing_AS_end(sg_platf_AS_cbarg_t AS)
 {
 
   if (current_routing == NULL) {
@@ -557,11 +522,11 @@ static void elements_father(sg_routing_edge_t src, sg_routing_edge_t dst,
 static void _get_route_and_latency(sg_routing_edge_t src, sg_routing_edge_t dst,
                                    xbt_dynar_t * links, double *latency)
 {
-  s_route_t route;
+  s_sg_platf_route_cbarg_t route;
   memset(&route,0,sizeof(route));
 
-  XBT_DEBUG("Solve route/latency  \"%s\" to \"%s\"", src->name, dst->name);
   xbt_assert(src && dst, "bad parameters for \"_get_route_latency\" method");
+  XBT_DEBUG("Solve route/latency  \"%s\" to \"%s\"", src->name, dst->name);
 
   /* Find how src and dst are interconnected */
   AS_t common_father, src_father, dst_father;
@@ -570,7 +535,7 @@ static void _get_route_and_latency(sg_routing_edge_t src, sg_routing_edge_t dst,
       common_father->name,src_father->name,dst_father->name);
 
   /* Check whether a direct bypass is defined */
-  route_t e_route_bypass = NULL;
+  sg_platf_route_cbarg_t e_route_bypass = NULL;
   if (common_father->get_bypass_route)
     e_route_bypass = common_father->get_bypass_route(common_father, src, dst, latency);
 
@@ -585,6 +550,7 @@ static void _get_route_and_latency(sg_routing_edge_t src, sg_routing_edge_t dst,
   if (src_father == dst_father) {       /* SURF_ROUTING_BASE */
     route.link_list = *links;
     common_father->get_route_and_latency(common_father, src, dst, &route,latency);
+    // if vivaldi latency+=vivaldi(src,dst)
     return;
   }
 
@@ -599,21 +565,22 @@ static void _get_route_and_latency(sg_routing_edge_t src, sg_routing_edge_t dst,
                                        src_father_net_elm, dst_father_net_elm,
                                        &route, latency);
 
-  xbt_assert((route.src_gateway != NULL) && (route.dst_gateway != NULL),
+  xbt_assert((route.gw_src != NULL) && (route.gw_dst != NULL),
       "bad gateways for route from \"%s\" to \"%s\"", src->name, dst->name);
 
-  sg_routing_edge_t src_gateway_net_elm = route.src_gateway;
-  sg_routing_edge_t dst_gateway_net_elm = route.dst_gateway;
+  sg_routing_edge_t src_gateway_net_elm = route.gw_src;
+  sg_routing_edge_t dst_gateway_net_elm = route.gw_dst;
 
   /* If source gateway is not our source, we have to recursively find our way up to this point */
   if (src != src_gateway_net_elm)
     _get_route_and_latency(src, src_gateway_net_elm, links, latency);
-
   xbt_dynar_merge(links, &route.link_list);
 
   /* If dest gateway is not our destination, we have to recursively find our way from this point */
   if (dst_gateway_net_elm != dst)
     _get_route_and_latency(dst_gateway_net_elm, dst, links, latency);
+
+  // if vivaldi latency+=vivaldi(src_gateway,dst_gateway)
 }
 
 /**
@@ -745,6 +712,83 @@ void routing_model_create( void *loopback)
 /* ************************************************************************** */
 /* ************************* GENERIC PARSE FUNCTIONS ************************ */
 
+void routing_cluster_add_backbone(void* bb) {
+  xbt_assert(current_routing->model_desc == &routing_models[SURF_MODEL_CLUSTER],
+        "You have to be in model Cluster to use tag backbone!");
+  xbt_assert(!((as_cluster_t)current_routing)->backbone,"The backbone link is already defined!");
+  ((as_cluster_t)current_routing)->backbone = bb;
+  XBT_DEBUG("Add a backbone to AS '%s'",current_routing->name);
+}
+
+static void routing_parse_cabinet(sg_platf_cabinet_cbarg_t cabinet)
+{
+  int start, end, i;
+  char *groups , *host_id , *link_id = NULL;
+  unsigned int iter;
+  xbt_dynar_t radical_elements;
+  xbt_dynar_t radical_ends;
+
+  //Make all hosts
+  radical_elements = xbt_str_split(cabinet->radical, ",");
+  xbt_dynar_foreach(radical_elements, iter, groups) {
+
+    radical_ends = xbt_str_split(groups, "-");
+    start = surf_parse_get_int(xbt_dynar_get_as(radical_ends, 0, char *));
+
+    switch (xbt_dynar_length(radical_ends)) {
+    case 1:
+      end = start;
+      break;
+    case 2:
+      end = surf_parse_get_int(xbt_dynar_get_as(radical_ends, 1, char *));
+      break;
+    default:
+      surf_parse_error("Malformed radical");
+      break;
+    }
+    s_sg_platf_host_cbarg_t host;
+    memset(&host, 0, sizeof(host));
+    host.initial_state = SURF_RESOURCE_ON;
+    host.power_peak = cabinet->power;
+    host.power_scale = 1.0;
+    host.core_amount = 1;
+
+    s_sg_platf_link_cbarg_t link;
+    memset(&link, 0, sizeof(link));
+    link.state = SURF_RESOURCE_ON;
+    link.policy = SURF_LINK_FULLDUPLEX;
+    link.latency = cabinet->lat;
+    link.bandwidth = cabinet->bw;
+
+    s_sg_platf_host_link_cbarg_t host_link;
+    memset(&host_link, 0, sizeof(host_link));
+
+    for (i = start; i <= end; i++) {
+      host_id = bprintf("%s%d%s",cabinet->prefix,i,cabinet->suffix);
+      link_id = bprintf("link_%s%d%s",cabinet->prefix,i,cabinet->suffix);
+      host.id = host_id;
+      link.id = link_id;
+      sg_platf_new_host(&host);
+      sg_platf_new_link(&link);
+
+      char* link_up = bprintf("%s_UP",link_id);
+      char* link_down = bprintf("%s_DOWN",link_id);
+      host_link.id = host_id;
+      host_link.link_up = link_up;
+      host_link.link_down= link_down;
+      sg_platf_new_host_link(&host_link);
+
+      free(host_id);
+      free(link_id);
+      free(link_up);
+      free(link_down);
+    }
+
+    xbt_dynar_free(&radical_ends);
+  }
+  xbt_dynar_free(&radical_elements);
+}
+
 static void routing_parse_cluster(sg_platf_cluster_cbarg_t cluster)
 {
   char *host_id, *groups, *link_id = NULL;
@@ -758,8 +802,8 @@ static void routing_parse_cluster(sg_platf_cluster_cbarg_t cluster)
   xbt_dynar_t radical_elements;
   xbt_dynar_t radical_ends;
 
-  if (strcmp(cluster->availability_trace, "")
-      || strcmp(cluster->state_trace, "")) {
+  if ((cluster->availability_trace && strcmp(cluster->availability_trace, ""))
+      || (cluster->state_trace && strcmp(cluster->state_trace, ""))) {
     patterns = xbt_dict_new_homogeneous(xbt_free_f);
     xbt_dict_set(patterns, "id", xbt_strdup(cluster->id), NULL);
     xbt_dict_set(patterns, "prefix", xbt_strdup(cluster->prefix), NULL);
@@ -767,7 +811,10 @@ static void routing_parse_cluster(sg_platf_cluster_cbarg_t cluster)
   }
 
   XBT_DEBUG("<AS id=\"%s\"\trouting=\"Cluster\">", cluster->id);
-  sg_platf_new_AS_begin(cluster->id, "Cluster");
+  s_sg_platf_AS_cbarg_t AS = SG_PLATF_AS_INITIALIZER;
+  AS.id = cluster->id;
+  AS.routing = A_surfxml_AS_routing_Cluster;
+  sg_platf_new_AS_begin(&AS);
 
   current_routing->link_up_down_list
             = xbt_dynar_new(sizeof(s_surf_parsing_link_up_down_t),NULL);
@@ -799,20 +846,20 @@ static void routing_parse_cluster(sg_platf_cluster_cbarg_t cluster)
 
       memset(&host, 0, sizeof(host));
       host.id = host_id;
-      if (strcmp(cluster->availability_trace, "")) {
+      if (cluster->availability_trace && strcmp(cluster->availability_trace, "")) {
         xbt_dict_set(patterns, "radical", bprintf("%d", i), NULL);
         char *avail_file = xbt_str_varsubst(cluster->availability_trace, patterns);
         XBT_DEBUG("\tavailability_file=\"%s\"", avail_file);
-        host.power_trace = tmgr_trace_new(avail_file);
+        host.power_trace = tmgr_trace_new_from_file(avail_file);
         xbt_free(avail_file);
       } else {
         XBT_DEBUG("\tavailability_file=\"\"");
       }
 
-      if (strcmp(cluster->state_trace, "")) {
+      if (cluster->state_trace && strcmp(cluster->state_trace, "")) {
         char *avail_file = xbt_str_varsubst(cluster->state_trace, patterns);
         XBT_DEBUG("\tstate_file=\"%s\"", avail_file);
-        host.state_trace = tmgr_trace_new(avail_file);
+        host.state_trace = tmgr_trace_new_from_file(avail_file);
         xbt_free(avail_file);
       } else {
         XBT_DEBUG("\tstate_file=\"\"");
@@ -892,7 +939,7 @@ static void routing_parse_cluster(sg_platf_cluster_cbarg_t cluster)
 
     sg_platf_new_link(&link);
 
-    surf_routing_cluster_add_backbone(current_routing, xbt_lib_get_or_null(link_lib, link_backbone, SURF_LINK_LEVEL));
+    routing_cluster_add_backbone(xbt_lib_get_or_null(link_lib, link_backbone, SURF_LINK_LEVEL));
 
     free(link_backbone);
   }
@@ -909,28 +956,12 @@ static void routing_parse_postparse(void) {
 
 static void routing_parse_peer(sg_platf_peer_cbarg_t peer)
 {
-  static int AX_ptr = 0;
   char *host_id = NULL;
-  char *router_id, *link_router, *link_backbone, *link_id_up, *link_id_down;
-
-  static unsigned int surfxml_buffer_stack_stack_ptr = 1;
-  static unsigned int surfxml_buffer_stack_stack[1024];
-
-  surfxml_buffer_stack_stack[0] = 0;
-
-  surfxml_bufferstack_push(1);
-
-  XBT_DEBUG("<AS id=\"%s\"\trouting=\"Full\">", peer->id);
-  sg_platf_new_AS_begin(peer->id, "Full");
+  char *link_id;
 
   XBT_DEBUG(" ");
   host_id = HOST_PEER(peer->id);
-  router_id = ROUTER_PEER(peer->id);
-  link_id_up = LINK_UP_PEER(peer->id);
-  link_id_down = LINK_DOWN_PEER(peer->id);
-
-  link_router = bprintf("%s_link_router", peer->id);
-  link_backbone = bprintf("%s_backbone", peer->id);
+  link_id = LINK_PEER(peer->id);
 
   XBT_DEBUG("<host\tid=\"%s\"\tpower=\"%f\"/>", host_id, peer->power);
   s_sg_platf_host_cbarg_t host;
@@ -945,80 +976,41 @@ static void routing_parse_peer(sg_platf_peer_cbarg_t peer)
   host.coord = peer->coord;
   sg_platf_new_host(&host);
 
-
-  XBT_DEBUG("<router id=\"%s\"\tcoordinates=\"%s\"/>", router_id, peer->coord);
-  s_sg_platf_router_cbarg_t router;
-  memset(&router, 0, sizeof(router));
-  router.id = router_id;
-  router.coord = peer->coord;
-  sg_platf_new_router(&router);
-
-  XBT_DEBUG("<link\tid=\"%s\"\tbw=\"%f\"\tlat=\"%f\"/>", link_id_up,
-            peer->bw_in, peer->lat);
   s_sg_platf_link_cbarg_t link;
   memset(&link, 0, sizeof(link));
   link.state = SURF_RESOURCE_ON;
   link.policy = SURF_LINK_SHARED;
-  link.id = link_id_up;
-  link.bandwidth = peer->bw_in;
   link.latency = peer->lat;
-  sg_platf_new_link(&link);
 
-  // FIXME: dealing with full duplex is not the role of this piece of code, I'd say [Mt]
-  // Instead, it should be created fullduplex, and the models will do what's needed in this case
-  XBT_DEBUG("<link\tid=\"%s\"\tbw=\"%f\"\tlat=\"%f\"/>", link_id_down,
+  char* link_up = bprintf("%s_UP",link_id);
+  XBT_DEBUG("<link\tid=\"%s\"\tbw=\"%f\"\tlat=\"%f\"/>", link_up,
             peer->bw_out, peer->lat);
-  link.id = link_id_down;
+  link.id = link_up;
+  link.bandwidth = peer->bw_out;
   sg_platf_new_link(&link);
 
-  XBT_DEBUG(" ");
+  char* link_down = bprintf("%s_DOWN",link_id);
+  XBT_DEBUG("<link\tid=\"%s\"\tbw=\"%f\"\tlat=\"%f\"/>", link_down,
+            peer->bw_in, peer->lat);
+  link.id = link_down;
+  link.bandwidth = peer->bw_in;
+  sg_platf_new_link(&link);
 
-  // begin here
-  XBT_DEBUG("<route\tsrc=\"%s\"\tdst=\"%s\"", host_id, router_id);
-  XBT_DEBUG("symmetrical=\"NO\">");
-  SURFXML_BUFFER_SET(route_src, host_id);
-  SURFXML_BUFFER_SET(route_dst, router_id);
-  A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_NO;
-  SURFXML_START_TAG(route);
+  XBT_DEBUG("<host_link\tid=\"%s\"\tup=\"%s\"\tdown=\"%s\" />", host_id,link_up,link_down);
+  s_sg_platf_host_link_cbarg_t host_link;
+  memset(&host_link, 0, sizeof(host_link));
+  host_link.id = host_id;
+  host_link.link_up = link_up;
+  host_link.link_down= link_down;
+  sg_platf_new_host_link(&host_link);
 
-  XBT_DEBUG("<link_ctn\tid=\"%s\"/>", link_id_up);
-  SURFXML_BUFFER_SET(link_ctn_id, link_id_up);
-  A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
-  SURFXML_START_TAG(link_ctn);
-  SURFXML_END_TAG(link_ctn);
-
-  XBT_DEBUG("</route>");
-  SURFXML_END_TAG(route);
-
-  //Opposite Route
-  XBT_DEBUG("<route\tsrc=\"%s\"\tdst=\"%s\"", router_id, host_id);
-  XBT_DEBUG("symmetrical=\"NO\">");
-  SURFXML_BUFFER_SET(route_src, router_id);
-  SURFXML_BUFFER_SET(route_dst, host_id);
-  A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_NO;
-  SURFXML_START_TAG(route);
-
-  XBT_DEBUG("<link_ctn\tid=\"%s\"/>", link_id_down);
-  SURFXML_BUFFER_SET(link_ctn_id, link_id_down);
-  A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
-  SURFXML_START_TAG(link_ctn);
-  SURFXML_END_TAG(link_ctn);
-
-  XBT_DEBUG("</route>");
-  SURFXML_END_TAG(route);
-
-  XBT_DEBUG("</AS>");
-  sg_platf_new_AS_end();
   XBT_DEBUG(" ");
 
   //xbt_dynar_free(&tab_elements_num);
   free(host_id);
-  free(router_id);
-  free(link_router);
-  free(link_backbone);
-  free(link_id_up);
-  free(link_id_down);
-  surfxml_bufferstack_pop(1);
+  free(link_id);
+  free(link_up);
+  free(link_down);
 }
 
 static void routing_parse_Srandom(void)
@@ -1145,26 +1137,14 @@ void routing_register_callbacks()
 {
   sg_platf_host_add_cb(parse_S_host);
   sg_platf_router_add_cb(parse_S_router);
-
-  surfxml_add_callback(STag_surfxml_random_cb_list, &routing_parse_Srandom);
-
-  surfxml_add_callback(STag_surfxml_route_cb_list, &routing_parse_S_route);
-  surfxml_add_callback(STag_surfxml_ASroute_cb_list, &routing_parse_S_ASroute);
-  surfxml_add_callback(STag_surfxml_bypassRoute_cb_list,
-                       &routing_parse_S_bypassRoute);
-  surfxml_add_callback(STag_surfxml_bypassASroute_cb_list,
-                       &routing_parse_S_bypassASroute);
-
-  surfxml_add_callback(ETag_surfxml_link_ctn_cb_list, &routing_parse_link_ctn);
-
-  surfxml_add_callback(ETag_surfxml_route_cb_list, &routing_parse_E_route);
-  surfxml_add_callback(ETag_surfxml_ASroute_cb_list, &routing_parse_E_ASroute);
-  surfxml_add_callback(ETag_surfxml_bypassRoute_cb_list,
-                       &routing_parse_E_bypassRoute);
-  surfxml_add_callback(ETag_surfxml_bypassASroute_cb_list,
-                       &routing_parse_E_bypassASroute);
+  sg_platf_host_link_add_cb(parse_S_host_link);
+  sg_platf_route_add_cb(parse_E_route);
+  sg_platf_ASroute_add_cb(parse_E_ASroute);
+  sg_platf_bypassRoute_add_cb(parse_E_bypassRoute);
+  sg_platf_bypassASroute_add_cb(parse_E_bypassASroute);
 
   sg_platf_cluster_add_cb(routing_parse_cluster);
+  sg_platf_cabinet_add_cb(routing_parse_cabinet);
 
   sg_platf_peer_add_cb(routing_parse_peer);
   sg_platf_postparse_add_cb(routing_parse_postparse);
@@ -1173,6 +1153,9 @@ void routing_register_callbacks()
   sg_platf_AS_end_add_cb(routing_AS_end);
   sg_platf_AS_begin_add_cb(routing_AS_begin);
 
+  sg_platf_trace_add_cb(routing_parse_trace);
+  sg_platf_trace_connect_add_cb(routing_parse_trace_connect);
+
 #ifdef HAVE_TRACING
   instr_routing_define_callbacks();
 #endif
