@@ -22,21 +22,23 @@ int ETag_surfxml_include_state(void);
 
 char* surf_parsed_filename = NULL; // to locate parse error messages
 
+xbt_dynar_t parsed_link_list = NULL;   /* temporary store of current list link of a route */
+extern AS_t current_routing;
 /*
  * Helping functions
  */
 void surf_parse_error(const char *fmt, ...) {
-	va_list va;
-	va_start(va,fmt);
-	char *msg = bvprintf(fmt,va);
-	va_end(va);
-	xbt_die("Parse error at %s:%d: %s", surf_parsed_filename, surf_parse_lineno, msg);
+  va_list va;
+  va_start(va,fmt);
+  char *msg = bvprintf(fmt,va);
+  va_end(va);
+  xbt_die("Parse error at %s:%d: %s", surf_parsed_filename, surf_parse_lineno, msg);
 }
 void surf_parse_warn(const char *fmt, ...) {
-	va_list va;
-	va_start(va,fmt);
-	char *msg = bvprintf(fmt,va);
-	va_end(va);
+  va_list va;
+  va_start(va,fmt);
+  char *msg = bvprintf(fmt,va);
+  va_end(va);
     XBT_WARN("%s:%d: %s", surf_parsed_filename, surf_parse_lineno, msg);
     free(msg);
 }
@@ -64,50 +66,19 @@ int surf_parse_get_int(const char *string) {
  */
 
 /* make sure these symbols are defined as strong ones in this file so that the linker can resolve them */
-xbt_dynar_t STag_surfxml_route_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_route_cb_list = NULL;
-xbt_dynar_t STag_surfxml_link_ctn_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_link_ctn_cb_list = NULL;
-xbt_dynar_t STag_surfxml_process_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_process_cb_list = NULL;
-xbt_dynar_t STag_surfxml_argument_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_argument_cb_list = NULL;
-xbt_dynar_t STag_surfxml_prop_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_prop_cb_list = NULL;
-xbt_dynar_t STag_surfxml_peer_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_peer_cb_list = NULL;
-xbt_dynar_t STag_surfxml_trace_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_trace_cb_list = NULL;
-xbt_dynar_t STag_surfxml_trace_connect_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_trace_connect_cb_list = NULL;
-xbt_dynar_t STag_surfxml_random_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_random_cb_list = NULL;
-xbt_dynar_t STag_surfxml_ASroute_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_ASroute_cb_list = NULL;
-xbt_dynar_t STag_surfxml_bypassRoute_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_bypassRoute_cb_list = NULL;
-xbt_dynar_t STag_surfxml_bypassASroute_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_bypassASroute_cb_list = NULL;
-xbt_dynar_t STag_surfxml_include_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_include_cb_list = NULL;
-
-xbt_dynar_t STag_surfxml_storage_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_storage_cb_list = NULL;
-xbt_dynar_t STag_surfxml_storage_type_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_storage_type_cb_list = NULL;
-xbt_dynar_t STag_surfxml_mount_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_mount_cb_list = NULL;
-xbt_dynar_t STag_surfxml_mstorage_cb_list = NULL;
-xbt_dynar_t ETag_surfxml_mstorage_cb_list = NULL;
 
 /* The default current property receiver. Setup in the corresponding opening callbacks. */
 xbt_dict_t current_property_set = NULL;
+xbt_dict_t as_current_property_set = NULL;
+int AS_TAG = 0;
+char* as_name_tab[1024];
+void* as_dict_tab[1024];
+int as_prop_nb = 0;
+
+
 /* dictionary of random generator data */
 xbt_dict_t random_data_list = NULL;
 
-/* Call all the callbacks of a specific SAX event */
-static XBT_INLINE void surfxml_call_cb_functions(xbt_dynar_t);
-
 YY_BUFFER_STATE surf_input_buffer;
 FILE *surf_file_to_parse = NULL;
 
@@ -119,7 +90,9 @@ static void add_randomness(void);
  */
 void STag_surfxml_storage(void)
 {
+  AS_TAG = 0;
   XBT_DEBUG("STag_surfxml_storage");
+  xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
 void ETag_surfxml_storage(void)
 {
@@ -128,10 +101,14 @@ void ETag_surfxml_storage(void)
 
   storage.id = A_surfxml_storage_id;
   storage.type_id = A_surfxml_storage_typeId;
+  storage.content = A_surfxml_storage_content;
+  storage.properties = current_property_set;
   sg_platf_new_storage(&storage);
+  current_property_set = NULL;
 }
 void STag_surfxml_storage_type(void)
 {
+  AS_TAG = 0;
   XBT_DEBUG("STag_surfxml_storage_type");
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
@@ -144,6 +121,7 @@ void ETag_surfxml_storage_type(void)
   storage_type.id = A_surfxml_storage_type_id;
   storage_type.model = A_surfxml_storage_type_model;
   storage_type.properties = current_property_set;
+  storage_type.size = surf_parse_get_int(A_surfxml_storage_type_size);
   sg_platf_new_storage_type(&storage_type);
   current_property_set = NULL;
 }
@@ -221,7 +199,7 @@ int ETag_surfxml_include_state(void)
   XBT_DEBUG("ETag_surfxml_include_state '%s'",A_surfxml_include_file);
 
   if(xbt_dynar_is_empty(surf_input_buffer_stack)) // nope, that's a true premature EOF. Let the parser die verbosely.
-	  return 0;
+    return 0;
 
   // Yeah, we were in an <include> Restore state and proceed.
   fclose(surf_file_to_parse);
@@ -239,112 +217,21 @@ int ETag_surfxml_include_state(void)
 
 void surf_parse_init_callbacks(void)
 {
-	  sg_platf_init(); // FIXME: move to a proper place?
-
-	  STag_surfxml_route_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_route_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_link_ctn_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_link_ctn_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_process_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_process_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_argument_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_argument_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_prop_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_prop_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_trace_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_trace_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_trace_connect_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_trace_connect_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_random_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_random_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_ASroute_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_ASroute_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_bypassRoute_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_bypassRoute_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      STag_surfxml_bypassASroute_cb_list =
-          xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      ETag_surfxml_bypassASroute_cb_list =
-          xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_peer_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_peer_cb_list =
-	      xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  STag_surfxml_include_cb_list =
-			  xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_include_cb_list =
-			  xbt_dynar_new(sizeof(void_f_void_t), NULL);
-
-	  STag_surfxml_storage_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-	  ETag_surfxml_storage_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      STag_surfxml_storage_type_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      ETag_surfxml_storage_type_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      STag_surfxml_mount_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      ETag_surfxml_mount_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      STag_surfxml_mstorage_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
-      ETag_surfxml_mstorage_cb_list = xbt_dynar_new(sizeof(void_f_void_t), NULL);
+    sg_platf_init();
 }
 
 void surf_parse_reset_callbacks(void)
 {
-	surf_parse_free_callbacks();
-	surf_parse_init_callbacks();
+  surf_parse_free_callbacks();
+  surf_parse_init_callbacks();
 }
 
 void surf_parse_free_callbacks(void)
 {
-  sg_platf_exit(); // FIXME: better place?
-
-  xbt_dynar_free(&STag_surfxml_route_cb_list);
-  xbt_dynar_free(&ETag_surfxml_route_cb_list);
-  xbt_dynar_free(&STag_surfxml_link_ctn_cb_list);
-  xbt_dynar_free(&ETag_surfxml_link_ctn_cb_list);
-  xbt_dynar_free(&STag_surfxml_process_cb_list);
-  xbt_dynar_free(&ETag_surfxml_process_cb_list);
-  xbt_dynar_free(&STag_surfxml_argument_cb_list);
-  xbt_dynar_free(&ETag_surfxml_argument_cb_list);
-  xbt_dynar_free(&STag_surfxml_prop_cb_list);
-  xbt_dynar_free(&ETag_surfxml_prop_cb_list);
-  xbt_dynar_free(&STag_surfxml_trace_cb_list);
-  xbt_dynar_free(&ETag_surfxml_trace_cb_list);
-  xbt_dynar_free(&STag_surfxml_trace_connect_cb_list);
-  xbt_dynar_free(&ETag_surfxml_trace_connect_cb_list);
-  xbt_dynar_free(&STag_surfxml_random_cb_list);
-  xbt_dynar_free(&ETag_surfxml_random_cb_list);
-  xbt_dynar_free(&STag_surfxml_ASroute_cb_list);
-  xbt_dynar_free(&ETag_surfxml_ASroute_cb_list);
-  xbt_dynar_free(&STag_surfxml_bypassRoute_cb_list);
-  xbt_dynar_free(&ETag_surfxml_bypassRoute_cb_list);
-  xbt_dynar_free(&STag_surfxml_bypassASroute_cb_list);
-  xbt_dynar_free(&ETag_surfxml_bypassASroute_cb_list);
-  xbt_dynar_free(&STag_surfxml_peer_cb_list);
-  xbt_dynar_free(&ETag_surfxml_peer_cb_list);
-  xbt_dynar_free(&STag_surfxml_include_cb_list);
-  xbt_dynar_free(&ETag_surfxml_include_cb_list);
-
-  xbt_dynar_free(&STag_surfxml_storage_cb_list);
-  xbt_dynar_free(&ETag_surfxml_storage_cb_list);
-  xbt_dynar_free(&STag_surfxml_mstorage_cb_list);
-  xbt_dynar_free(&ETag_surfxml_mstorage_cb_list);
-  xbt_dynar_free(&STag_surfxml_mount_cb_list);
-  xbt_dynar_free(&ETag_surfxml_mount_cb_list);
-  xbt_dynar_free(&STag_surfxml_storage_type_cb_list);
-  xbt_dynar_free(&ETag_surfxml_storage_type_cb_list);
+  sg_platf_exit();
 }
 
 /* Stag and Etag parse functions */
-void ETag_surfxml_router(void)  { /* ignored -- do not add content here */ }
 
 void STag_surfxml_platform(void) {
   _XBT_GNUC_UNUSED double version = surf_parse_get_double(A_surfxml_platform_version);
@@ -376,9 +263,30 @@ void ETag_surfxml_platform(void){
 }
 
 void STag_surfxml_host(void){
+  AS_TAG = 0;
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
 
+void STag_surfxml_prop(void)
+{
+  if(AS_TAG){
+    if (!as_current_property_set){
+      xbt_assert(as_prop_nb < 1024, "Number of AS property reach the limit!!!");
+      as_current_property_set = xbt_dict_new_homogeneous(xbt_free_f); // Maybe, it should raise an error
+      as_name_tab[as_prop_nb] = xbt_strdup(A_surfxml_AS_id);
+      as_dict_tab[as_prop_nb] = as_current_property_set;
+      XBT_DEBUG("PUSH prop %p for AS '%s'",as_dict_tab[as_prop_nb],as_name_tab[as_prop_nb]);
+      as_prop_nb++;
+    }
+    xbt_dict_set(as_current_property_set, A_surfxml_prop_id, xbt_strdup(A_surfxml_prop_value), NULL);
+  }
+  else{
+    if (!current_property_set)
+      current_property_set = xbt_dict_new_homogeneous(xbt_free_f); // Maybe, it should raise an error
+    xbt_dict_set(current_property_set, A_surfxml_prop_id, xbt_strdup(A_surfxml_prop_value), NULL);
+  }
+}
+
 void ETag_surfxml_host(void)    {
   s_sg_platf_host_cbarg_t host;
   memset(&host,0,sizeof(host));
@@ -389,8 +297,8 @@ void ETag_surfxml_host(void)    {
   host.power_peak = get_cpu_power(A_surfxml_host_power);
   host.power_scale = surf_parse_get_double( A_surfxml_host_availability);
   host.core_amount = surf_parse_get_int(A_surfxml_host_core);
-  host.power_trace = tmgr_trace_new(A_surfxml_host_availability_file);
-  host.state_trace = tmgr_trace_new(A_surfxml_host_state_file);
+  host.power_trace = tmgr_trace_new_from_file(A_surfxml_host_availability_file);
+  host.state_trace = tmgr_trace_new_from_file(A_surfxml_host_state_file);
   xbt_assert((A_surfxml_host_state == A_surfxml_host_state_ON) ||
         (A_surfxml_host_state == A_surfxml_host_state_OFF), "Invalid state");
   if (A_surfxml_host_state == A_surfxml_host_state_ON)
@@ -403,32 +311,43 @@ void ETag_surfxml_host(void)    {
   current_property_set = NULL;
 }
 
+void STag_surfxml_host_link(void){
+  XBT_DEBUG("Create a Host_link for %s",A_surfxml_host_link_id);
+  s_sg_platf_host_link_cbarg_t host_link;
+  memset(&host_link,0,sizeof(host_link));
+
+  host_link.id = A_surfxml_host_link_id;
+  host_link.link_up = A_surfxml_host_link_up;
+  host_link.link_down = A_surfxml_host_link_down;
+  sg_platf_new_host_link(&host_link);
+}
+
 void STag_surfxml_router(void){
   s_sg_platf_router_cbarg_t router;
   memset(&router, 0, sizeof(router));
 
-	router.id = A_surfxml_router_id;
-	router.coord = A_surfxml_router_coordinates;
+  router.id = A_surfxml_router_id;
+  router.coord = A_surfxml_router_coordinates;
 
-	sg_platf_new_router(&router);
+  sg_platf_new_router(&router);
 }
 
 void STag_surfxml_cluster(void){
   s_sg_platf_cluster_cbarg_t cluster;
   memset(&cluster,0,sizeof(cluster));
-	cluster.id = A_surfxml_cluster_id;
-	cluster.prefix = A_surfxml_cluster_prefix;
-	cluster.suffix = A_surfxml_cluster_suffix;
-	cluster.radical = A_surfxml_cluster_radical;
-	cluster.power= surf_parse_get_double(A_surfxml_cluster_power);
-	cluster.core_amount = surf_parse_get_int(A_surfxml_cluster_core);
-	cluster.bw =   surf_parse_get_double(A_surfxml_cluster_bw);
-	cluster.lat =  surf_parse_get_double(A_surfxml_cluster_lat);
-	if(strcmp(A_surfxml_cluster_bb_bw,""))
-	  cluster.bb_bw = surf_parse_get_double(A_surfxml_cluster_bb_bw);
-	if(strcmp(A_surfxml_cluster_bb_lat,""))
-	  cluster.bb_lat = surf_parse_get_double(A_surfxml_cluster_bb_lat);
-	cluster.router_id = A_surfxml_cluster_router_id;
+  cluster.id = A_surfxml_cluster_id;
+  cluster.prefix = A_surfxml_cluster_prefix;
+  cluster.suffix = A_surfxml_cluster_suffix;
+  cluster.radical = A_surfxml_cluster_radical;
+  cluster.power= surf_parse_get_double(A_surfxml_cluster_power);
+  cluster.core_amount = surf_parse_get_int(A_surfxml_cluster_core);
+  cluster.bw =   surf_parse_get_double(A_surfxml_cluster_bw);
+  cluster.lat =  surf_parse_get_double(A_surfxml_cluster_lat);
+  if(strcmp(A_surfxml_cluster_bb_bw,""))
+    cluster.bb_bw = surf_parse_get_double(A_surfxml_cluster_bb_bw);
+  if(strcmp(A_surfxml_cluster_bb_lat,""))
+    cluster.bb_lat = surf_parse_get_double(A_surfxml_cluster_bb_lat);
+  cluster.router_id = A_surfxml_cluster_router_id;
 
   switch (AX_surfxml_cluster_sharing_policy) {
   case A_surfxml_cluster_sharing_policy_SHARED:
@@ -458,35 +377,45 @@ void STag_surfxml_cluster(void){
     break;
   }
 
-	cluster.availability_trace = A_surfxml_cluster_availability_file;
-	cluster.state_trace = A_surfxml_cluster_state_file;
-	sg_platf_new_cluster(&cluster);
+  cluster.availability_trace = A_surfxml_cluster_availability_file;
+  cluster.state_trace = A_surfxml_cluster_state_file;
+  sg_platf_new_cluster(&cluster);
 }
-void ETag_surfxml_cluster(void){
-  /* nothing I can think of */
+
+void STag_surfxml_cabinet(void){
+  s_sg_platf_cabinet_cbarg_t cabinet;
+  memset(&cabinet,0,sizeof(cabinet));
+  cabinet.id = A_surfxml_cabinet_id;
+  cabinet.prefix = A_surfxml_cabinet_prefix;
+  cabinet.suffix = A_surfxml_cabinet_suffix;
+  cabinet.power = surf_parse_get_double(A_surfxml_cabinet_power);
+  cabinet.bw = surf_parse_get_double(A_surfxml_cabinet_bw);
+  cabinet.lat = surf_parse_get_double(A_surfxml_cabinet_lat);
+  cabinet.radical = A_surfxml_cabinet_radical;
+
+  sg_platf_new_cabinet(&cabinet);
 }
 
 void STag_surfxml_peer(void){
   s_sg_platf_peer_cbarg_t peer;
   memset(&peer,0,sizeof(peer));
-	peer.id = A_surfxml_peer_id;
-	peer.power = surf_parse_get_double(A_surfxml_peer_power);
-	peer.bw_in = surf_parse_get_double(A_surfxml_peer_bw_in);
-	peer.bw_out = surf_parse_get_double(A_surfxml_peer_bw_out);
-	peer.lat = surf_parse_get_double(A_surfxml_peer_lat);
-	peer.coord = A_surfxml_peer_coordinates;
-	peer.availability_trace = tmgr_trace_new(A_surfxml_peer_availability_file);
-	peer.state_trace = tmgr_trace_new(A_surfxml_peer_state_file);
+  peer.id = A_surfxml_peer_id;
+  peer.power = surf_parse_get_double(A_surfxml_peer_power);
+  peer.bw_in = surf_parse_get_double(A_surfxml_peer_bw_in);
+  peer.bw_out = surf_parse_get_double(A_surfxml_peer_bw_out);
+  peer.lat = surf_parse_get_double(A_surfxml_peer_lat);
+  peer.coord = A_surfxml_peer_coordinates;
+  peer.availability_trace = tmgr_trace_new_from_file(A_surfxml_peer_availability_file);
+  peer.state_trace = tmgr_trace_new_from_file(A_surfxml_peer_state_file);
 
-	surfxml_call_cb_functions(STag_surfxml_peer_cb_list);
-	sg_platf_new_peer(&peer);
-}
-void ETag_surfxml_peer(void){
-  /* nothing to do here */
+  sg_platf_new_peer(&peer);
 }
+
 void STag_surfxml_link(void){
+  AS_TAG = 0;
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
+
 void ETag_surfxml_link(void){
   s_sg_platf_link_cbarg_t link;
   memset(&link,0,sizeof(link));
@@ -495,9 +424,9 @@ void ETag_surfxml_link(void){
 
   link.id = A_surfxml_link_id;
   link.bandwidth = surf_parse_get_double(A_surfxml_link_bandwidth);
-  link.bandwidth_trace = tmgr_trace_new(A_surfxml_link_bandwidth_file);
+  link.bandwidth_trace = tmgr_trace_new_from_file(A_surfxml_link_bandwidth_file);
   link.latency = surf_parse_get_double(A_surfxml_link_latency);
-  link.latency_trace = tmgr_trace_new(A_surfxml_link_latency_file);
+  link.latency_trace = tmgr_trace_new_from_file(A_surfxml_link_latency_file);
 
   switch (A_surfxml_link_state) {
   case A_surfxml_link_state_ON:
@@ -510,7 +439,7 @@ void ETag_surfxml_link(void){
     surf_parse_error("invalid state for link %s", link.id);
     break;
   }
-  link.state_trace = tmgr_trace_new(A_surfxml_link_state_file);
+  link.state_trace = tmgr_trace_new_from_file(A_surfxml_link_state_file);
 
   switch (A_surfxml_link_sharing_policy) {
   case A_surfxml_link_sharing_policy_SHARED:
@@ -532,45 +461,248 @@ void ETag_surfxml_link(void){
   current_property_set = NULL;
 }
 
-void STag_surfxml_route(void){
-	surfxml_call_cb_functions(STag_surfxml_route_cb_list);
-}
 void STag_surfxml_link_ctn(void){
-	surfxml_call_cb_functions(STag_surfxml_link_ctn_cb_list);
+
+  char *link_id;
+  switch (A_surfxml_link_ctn_direction) {
+  case AU_surfxml_link_ctn_direction:
+  case A_surfxml_link_ctn_direction_NONE:
+    link_id = xbt_strdup(A_surfxml_link_ctn_id);
+    break;
+  case A_surfxml_link_ctn_direction_UP:
+    link_id = bprintf("%s_UP", A_surfxml_link_ctn_id);
+    break;
+  case A_surfxml_link_ctn_direction_DOWN:
+    link_id = bprintf("%s_DOWN", A_surfxml_link_ctn_id);
+    break;
+  }
+
+  // FIXME we should push the surf link object but it don't
+  // work because of model rulebased
+  xbt_dynar_push(parsed_link_list, &link_id);
 }
-void STag_surfxml_process(void){
-	surfxml_call_cb_functions(STag_surfxml_process_cb_list);
+
+void ETag_surfxml_backbone(void){
+  s_sg_platf_link_cbarg_t link;
+  memset(&link,0,sizeof(link));
+
+  link.properties = NULL;
+
+  link.id = A_surfxml_backbone_id;
+  link.bandwidth = surf_parse_get_double(A_surfxml_backbone_bandwidth);
+  link.latency = surf_parse_get_double(A_surfxml_backbone_latency);
+  link.state = SURF_RESOURCE_ON;
+  link.policy = SURF_LINK_SHARED;
+
+  sg_platf_new_link(&link);
+  routing_cluster_add_backbone(xbt_lib_get_or_null(link_lib, A_surfxml_backbone_id, SURF_LINK_LEVEL));
 }
-void STag_surfxml_argument(void){
-	surfxml_call_cb_functions(STag_surfxml_argument_cb_list);
+
+void STag_surfxml_route(void){
+  xbt_assert(strlen(A_surfxml_route_src) > 0 || strlen(A_surfxml_route_dst) > 0,
+      "Missing end-points while defining route \"%s\"->\"%s\"",
+      A_surfxml_route_src, A_surfxml_route_dst);
+  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
-void STag_surfxml_prop(void){
-	surfxml_call_cb_functions(STag_surfxml_prop_cb_list);
+
+void STag_surfxml_ASroute(void){
+  xbt_assert(strlen(A_surfxml_ASroute_src) > 0 || strlen(A_surfxml_ASroute_dst) > 0
+      || strlen(A_surfxml_ASroute_gw_src) > 0 || strlen(A_surfxml_ASroute_gw_dst) > 0,
+      "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
+      A_surfxml_ASroute_src, A_surfxml_ASroute_dst,
+      A_surfxml_ASroute_gw_src,A_surfxml_ASroute_gw_dst);
+  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
-void STag_surfxml_trace(void){
-	surfxml_call_cb_functions(STag_surfxml_trace_cb_list);
+
+void STag_surfxml_bypassRoute(void){
+  xbt_assert(strlen(A_surfxml_bypassRoute_src) > 0 || strlen(A_surfxml_bypassRoute_dst) > 0,
+      "Missing end-points while defining bupass route \"%s\"->\"%s\"",
+      A_surfxml_bypassRoute_src, A_surfxml_bypassRoute_dst);
+  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
+
+void STag_surfxml_bypassASroute(void){
+  xbt_assert(strlen(A_surfxml_bypassASroute_src) > 0 || strlen(A_surfxml_bypassASroute_dst) > 0
+      || strlen(A_surfxml_bypassASroute_gw_src) > 0 || strlen(A_surfxml_bypassASroute_gw_dst) > 0,
+      "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
+      A_surfxml_bypassASroute_src, A_surfxml_bypassASroute_dst,
+      A_surfxml_bypassASroute_gw_src,A_surfxml_bypassASroute_gw_dst);
+  parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
+}
+
+void ETag_surfxml_route(void){
+  s_sg_platf_route_cbarg_t route;
+  memset(&route,0,sizeof(route));
+
+  route.src = A_surfxml_route_src;
+  route.dst = A_surfxml_route_dst;
+  route.gw_src = NULL;
+  route.gw_dst = NULL;
+  route.link_list = parsed_link_list;
+
+  switch (A_surfxml_route_symmetrical) {
+  case AU_surfxml_route_symmetrical:
+  case A_surfxml_route_symmetrical_YES:
+    route.symmetrical = TRUE;
+    break;
+  case A_surfxml_route_symmetrical_NO:
+    route.symmetrical = FALSE;;
+    break;
+  }
+
+  sg_platf_new_route(&route);
+  parsed_link_list = NULL;
+}
+
+void ETag_surfxml_ASroute(void){
+  s_sg_platf_route_cbarg_t ASroute;
+  memset(&ASroute,0,sizeof(ASroute));
+
+  ASroute.src = A_surfxml_ASroute_src;
+  ASroute.dst = A_surfxml_ASroute_dst;
+
+  if (!strcmp(current_routing->model_desc->name,"RuleBased")) {
+    // DIRTY PERL HACK AHEAD: with the rulebased routing, the {src,dst}_gateway fields
+    // store the provided name instead of the entity directly (model_rulebased_parse_ASroute knows)
+    //
+    // This is because the user will provide something like "^AS_(.*)$" instead of the proper name of a given entity
+    ASroute.gw_src = (sg_routing_edge_t) A_surfxml_ASroute_gw_src;
+    ASroute.gw_dst = (sg_routing_edge_t) A_surfxml_ASroute_gw_dst;
+  } else {
+    ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw_src);
+    ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw_dst);
+  }
+
+  ASroute.link_list = parsed_link_list;
+
+  switch (A_surfxml_ASroute_symmetrical) {
+  case AU_surfxml_ASroute_symmetrical:
+  case A_surfxml_ASroute_symmetrical_YES:
+    ASroute.symmetrical = TRUE;
+    break;
+  case A_surfxml_ASroute_symmetrical_NO:
+    ASroute.symmetrical = FALSE;
+    break;
+  }
+
+  sg_platf_new_ASroute(&ASroute);
+  parsed_link_list = NULL;
+}
+
+void ETag_surfxml_bypassRoute(void){
+  s_sg_platf_route_cbarg_t route;
+  memset(&route,0,sizeof(route));
+
+  route.src = A_surfxml_bypassRoute_src;
+  route.dst = A_surfxml_bypassRoute_dst;
+  route.gw_src = NULL;
+  route.gw_dst = NULL;
+  route.link_list = parsed_link_list;
+  route.symmetrical = FALSE;
+
+  sg_platf_new_bypassRoute(&route);
+  parsed_link_list = NULL;
+}
+
+void ETag_surfxml_bypassASroute(void){
+  s_sg_platf_route_cbarg_t ASroute;
+  memset(&ASroute,0,sizeof(ASroute));
+
+  ASroute.src = A_surfxml_bypassASroute_src;
+  ASroute.dst = A_surfxml_bypassASroute_dst;
+  ASroute.link_list = parsed_link_list;
+  ASroute.symmetrical = FALSE;
+
+  if (!strcmp(current_routing->model_desc->name,"RuleBased")) {
+    // DIRTY PERL HACK AHEAD: with the rulebased routing, the {src,dst}_gateway fields
+    // store the provided name instead of the entity directly (model_rulebased_parse_ASroute knows)
+    //
+    // This is because the user will provide something like "^AS_(.*)$" instead of the proper name of a given entity
+    ASroute.gw_src = (sg_routing_edge_t) A_surfxml_bypassASroute_gw_src;
+    ASroute.gw_dst = (sg_routing_edge_t) A_surfxml_bypassASroute_gw_dst;
+  } else {
+    ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw_src);
+    ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw_dst);
+  }
+
+  sg_platf_new_bypassASroute(&ASroute);
+  parsed_link_list = NULL;
+}
+
+void ETag_surfxml_trace(void){
+  s_sg_platf_trace_cbarg_t trace;
+  memset(&trace,0,sizeof(trace));
+
+  trace.id = A_surfxml_trace_id;
+  trace.file = A_surfxml_trace_file;
+  trace.periodicity = surf_parse_get_double(A_surfxml_trace_periodicity);
+  trace.pc_data = surfxml_pcdata;
+
+  sg_platf_new_trace(&trace);
+}
+
 void STag_surfxml_trace_connect(void){
-	surfxml_call_cb_functions(STag_surfxml_trace_connect_cb_list);
+  s_sg_platf_trace_connect_cbarg_t trace_connect;
+  memset(&trace_connect,0,sizeof(trace_connect));
+
+  trace_connect.element = A_surfxml_trace_connect_element;
+  trace_connect.trace = A_surfxml_trace_connect_trace;
+
+  switch (A_surfxml_trace_connect_kind) {
+  case AU_surfxml_trace_connect_kind:
+  case A_surfxml_trace_connect_kind_POWER:
+    trace_connect.kind =  SURF_TRACE_CONNECT_KIND_POWER;
+    break;
+  case A_surfxml_trace_connect_kind_BANDWIDTH:
+    trace_connect.kind =  SURF_TRACE_CONNECT_KIND_BANDWIDTH;
+    break;
+  case A_surfxml_trace_connect_kind_HOST_AVAIL:
+    trace_connect.kind =  SURF_TRACE_CONNECT_KIND_HOST_AVAIL;
+    break;
+  case A_surfxml_trace_connect_kind_LATENCY:
+    trace_connect.kind =  SURF_TRACE_CONNECT_KIND_LATENCY;
+    break;
+  case A_surfxml_trace_connect_kind_LINK_AVAIL:
+    trace_connect.kind =  SURF_TRACE_CONNECT_KIND_LINK_AVAIL;
+    break;
+  }
+  sg_platf_new_trace_connect(&trace_connect);
 }
+
 void STag_surfxml_AS(void){
-  sg_platf_new_AS_begin(A_surfxml_AS_id,A_surfxml_AS_routing);
+  AS_TAG = 1;
+  s_sg_platf_AS_cbarg_t AS = SG_PLATF_AS_INITIALIZER;
+  AS.id = A_surfxml_AS_id;
+  AS.routing = (int)A_surfxml_AS_routing;
+
+  as_current_property_set = NULL;
+
+  sg_platf_new_AS_begin(&AS);
 }
 void ETag_surfxml_AS(void){
+  if(as_prop_nb){
+    char *name = as_name_tab[as_prop_nb-1];
+    xbt_dict_t dict = as_dict_tab[as_prop_nb-1];
+    as_prop_nb--;
+    XBT_DEBUG("POP prop %p for AS '%s'",dict,name);
+    xbt_lib_set(as_router_lib,
+        name,
+      ROUTING_PROP_ASR_LEVEL,
+      dict);
+    xbt_free(name);
+  }
   sg_platf_new_AS_end();
 }
-void STag_surfxml_ASroute(void){
-	surfxml_call_cb_functions(STag_surfxml_ASroute_cb_list);
-}
-void STag_surfxml_bypassRoute(void){
-	surfxml_call_cb_functions(STag_surfxml_bypassRoute_cb_list);
-}
-void STag_surfxml_bypassASroute(void){
-  surfxml_call_cb_functions(STag_surfxml_bypassASroute_cb_list);
-}
+
+extern int _surf_init_status; /* FIXME: find a proper way to export this at some point */
+
 void STag_surfxml_config(void){
-  XBT_DEBUG("START configuration name = %s",A_surfxml_config_id);
+  AS_TAG = 0;
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
+  XBT_DEBUG("START configuration name = %s",A_surfxml_config_id);
+  if (_surf_init_status == 2) {
+    surf_parse_error("All <config> tags must be given before any platform elements (such as <AS>, <host>, <cluster>, <link>, etc).");
+  }
 }
 void ETag_surfxml_config(void){
   xbt_dict_cursor_t cursor = NULL;
@@ -587,25 +719,83 @@ void ETag_surfxml_config(void){
   }
   XBT_DEBUG("End configuration name = %s",A_surfxml_config_id);
   xbt_dict_free(&current_property_set);
-}
-void STag_surfxml_random(void){
-	surfxml_call_cb_functions(STag_surfxml_random_cb_list);
+  current_property_set = NULL;
 }
 
-#define parse_method(type,name) \
-void type##Tag_surfxml_##name(void) \
-{ surfxml_call_cb_functions(type##Tag_surfxml_##name##_cb_list); }
-parse_method(E, route)
-parse_method(E, link_ctn)
-parse_method(E, process)
-parse_method(E, argument)
-parse_method(E, prop)
-parse_method(E, trace)
-parse_method(E, trace_connect)
-parse_method(E, random)
-parse_method(E, ASroute)
-parse_method(E, bypassRoute)
-parse_method(E, bypassASroute)
+static int argc;
+static char **argv;
+
+void STag_surfxml_process(void){
+  AS_TAG = 0;
+  argc = 1;
+  argv = xbt_new(char *, 1);
+  argv[0] = xbt_strdup(A_surfxml_process_function);
+  xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
+}
+
+void ETag_surfxml_process(void){
+  s_sg_platf_process_cbarg_t process;
+  memset(&process,0,sizeof(process));
+
+  process.argc = argc;
+  process.argv = (const char **)argv;
+  process.properties = current_property_set;
+  process.host = A_surfxml_process_host;
+  process.function = A_surfxml_process_function;
+  process.start_time = surf_parse_get_double(A_surfxml_process_start_time);
+  process.kill_time = surf_parse_get_double(A_surfxml_process_kill_time);
+
+  switch (A_surfxml_process_on_failure) {
+  case AU_surfxml_process_on_failure:
+  case A_surfxml_process_on_failure_DIE:
+    process.on_failure =  SURF_PROCESS_ON_FAILURE_DIE;
+    break;
+  case A_surfxml_process_on_failure_RESTART:
+    process.on_failure =  SURF_PROCESS_ON_FAILURE_RESTART;
+    break;
+  }
+
+  sg_platf_new_process(&process);
+  current_property_set = NULL;
+}
+
+void STag_surfxml_argument(void){
+  argc++;
+  argv = xbt_realloc(argv, (argc) * sizeof(char *));
+  argv[(argc) - 1] = xbt_strdup(A_surfxml_argument_value);
+}
+
+/* ***************************************** */
+/* TUTORIAL: New TAG                         */
+void STag_surfxml_gpu(void)
+{
+  XBT_DEBUG("STag_surfxml_gpu");
+}
+void ETag_surfxml_gpu(void)
+{
+  s_sg_platf_gpu_cbarg_t gpu;
+  memset(&gpu,0,sizeof(gpu));
+
+  gpu.name = A_surfxml_gpu_name;
+
+  sg_platf_new_gpu(&gpu);
+}
+/* ***************************************** */
+
+/* nothing to do in those functions */
+void ETag_surfxml_prop(void){}
+void STag_surfxml_random(void){}
+void ETag_surfxml_random(void){}
+void ETag_surfxml_trace_connect(void){}
+void STag_surfxml_trace(void){}
+void ETag_surfxml_router(void){}
+void ETag_surfxml_host_link(void){}
+void ETag_surfxml_cluster(void){}
+void ETag_surfxml_cabinet(void){}
+void ETag_surfxml_peer(void){}
+void STag_surfxml_backbone(void){}
+void ETag_surfxml_link_ctn(void){}
+void ETag_surfxml_argument(void){}
 
 /* Open and Close parse file */
 
@@ -661,56 +851,11 @@ static int _surf_parse(void) {
 int_f_void_t surf_parse = _surf_parse;
 
 
-/* Aux parse functions */
-
-void surfxml_add_callback(xbt_dynar_t cb_list, void_f_void_t function)
-{
-  xbt_dynar_push(cb_list, &function);
-}
-
-void surfxml_del_callback(xbt_dynar_t cb_list, void_f_void_t function)
-{
-  xbt_ex_t e;
-  unsigned int it=0;
-  void_f_void_t null_f=NULL;
-
-  TRY {
-    it = xbt_dynar_search(cb_list,&function);
-  }
-  CATCH(e) {
-    if (e.category == not_found_error) {
-      xbt_ex_free(e);
-      xbt_die("Trying to remove a callback that is not here! This should not happen");
-    }
-    RETHROW;
-  }
-
-  xbt_dynar_replace(cb_list, it,&null_f);
-}
-
-static XBT_INLINE void surfxml_call_cb_functions(xbt_dynar_t cb_list)
-{
-  unsigned int iterator;
-  void_f_void_t fun;
-  xbt_dynar_foreach(cb_list, iterator, fun) {
-    if (fun) fun();
-  }
-}
-
-
 /* Prop tag functions */
 
 /**
  * With XML parser
  */
-void parse_properties(void)
-{
-  if (!current_property_set)
-    current_property_set = xbt_dict_new_homogeneous(xbt_free_f); // Maybe, it should raise an error
-
-  xbt_dict_set(current_property_set, A_surfxml_prop_id, xbt_strdup(A_surfxml_prop_value), NULL);
-}
-
 
 /* Random tag functions */
 
@@ -778,3 +923,9 @@ static void add_randomness(void)
                &xbt_free_ref);
 }
 
+
+xbt_dict_t get_as_router_properties(const char* name)
+{
+  return xbt_lib_get_or_null(as_router_lib, name, ROUTING_PROP_ASR_LEVEL);
+}
+
