@@ -6,15 +6,13 @@
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#define DJB2_HASH_FUNCTION
-//#define FNV_HASH_FUNCTION
-
 #include <string.h>
 #include <stdio.h>
 #include "xbt/ex.h"
 #include "xbt/log.h"
 #include "xbt/mallocator.h"
 #include "xbt_modinter.h"
+#include "xbt/str.h"
 #include "dict_private.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(xbt_dict, xbt,
@@ -105,85 +103,6 @@ XBT_INLINE unsigned int xbt_dict_size(xbt_dict_t dict)
   return (dict ? (unsigned int) dict->count : (unsigned int) 0);
 }
 
-/**
- * Returns the hash code of a string.
- */
-static XBT_INLINE unsigned int xbt_dict_hash_ext(const char *str,
-                                                 int str_len)
-{
-
-
-#ifdef DJB2_HASH_FUNCTION
-  /* fast implementation of djb2 algorithm */
-  int c;
-  register unsigned int hash = 5381;
-
-  while (str_len--) {
-    c = *str++;
-    hash = ((hash << 5) + hash) + c;    /* hash * 33 + c */
-  }
-# elif defined(FNV_HASH_FUNCTION)
-  register unsigned int hash = 0x811c9dc5;
-  unsigned char *bp = (unsigned char *) str;    /* start of buffer */
-  unsigned char *be = bp + str_len;     /* beyond end of buffer */
-
-  while (bp < be) {
-    /* multiply by the 32 bit FNV magic prime mod 2^32 */
-    hash +=
-        (hash << 1) + (hash << 4) + (hash << 7) + (hash << 8) +
-        (hash << 24);
-
-    /* xor the bottom with the current octet */
-    hash ^= (unsigned int) *bp++;
-  }
-
-# else
-  register unsigned int hash = 0;
-
-  while (str_len--) {
-    hash += (*str) * (*str);
-    str++;
-  }
-#endif
-
-  return hash;
-}
-
-static XBT_INLINE unsigned int xbt_dict_hash(const char *str)
-{
-#ifdef DJB2_HASH_FUNCTION
-  /* fast implementation of djb2 algorithm */
-  int c;
-  register unsigned int hash = 5381;
-
-  while ((c = *str++)) {
-    hash = ((hash << 5) + hash) + c;    /* hash * 33 + c */
-  }
-
-# elif defined(FNV_HASH_FUNCTION)
-  register unsigned int hash = 0x811c9dc5;
-
-  while (*str) {
-    /* multiply by the 32 bit FNV magic prime mod 2^32 */
-    hash +=
-        (hash << 1) + (hash << 4) + (hash << 7) + (hash << 8) +
-        (hash << 24);
-
-    /* xor the bottom with the current byte */
-    hash ^= (unsigned int) *str++;
-  }
-
-# else
-  register unsigned int hash = 0;
-
-  while (*str) {
-    hash += (*str) * (*str);
-    str++;
-  }
-#endif
-  return hash;
-}
-
 /* Expend the size of the dict */
 static void xbt_dict_rehash(xbt_dict_t dict)
 {
@@ -247,7 +166,7 @@ XBT_INLINE void xbt_dict_set_ext(xbt_dict_t dict,
                                  void *data, void_f_pvoid_t free_ctn)
 {
 
-  unsigned int hash_code = xbt_dict_hash_ext(key, key_len);
+  unsigned int hash_code = xbt_str_hash_ext(key, key_len);
 
   xbt_dictelm_t current, previous = NULL;
   xbt_assert(dict);
@@ -318,7 +237,7 @@ XBT_INLINE void *xbt_dict_get_ext(xbt_dict_t dict, const char *key,
 {
 
 
-  unsigned int hash_code = xbt_dict_hash_ext(key, key_len);
+  unsigned int hash_code = xbt_str_hash_ext(key, key_len);
   xbt_dictelm_t current;
 
   xbt_assert(dict);
@@ -343,7 +262,7 @@ void *xbt_dict_get_or_null_ext(xbt_dict_t dict, const char *key,
                                int key_len)
 {
 
-  unsigned int hash_code = xbt_dict_hash_ext(key, key_len);
+  unsigned int hash_code = xbt_str_hash_ext(key, key_len);
   xbt_dictelm_t current;
 
   xbt_assert(dict);
@@ -398,7 +317,7 @@ char *xbt_dict_get_key(xbt_dict_t dict, const void *data)
 XBT_INLINE void *xbt_dict_get(xbt_dict_t dict, const char *key)
 {
 
-  unsigned int hash_code = xbt_dict_hash(key);
+  unsigned int hash_code = xbt_str_hash(key);
   xbt_dictelm_t current;
 
   xbt_assert(dict);
@@ -419,7 +338,7 @@ XBT_INLINE void *xbt_dict_get(xbt_dict_t dict, const char *key)
  */
 XBT_INLINE void *xbt_dict_get_or_null(xbt_dict_t dict, const char *key)
 {
-  unsigned int hash_code = xbt_dict_hash(key);
+  unsigned int hash_code = xbt_str_hash(key);
   xbt_dictelm_t current;
 
   xbt_assert(dict);
@@ -450,7 +369,7 @@ XBT_INLINE void xbt_dict_remove_ext(xbt_dict_t dict, const char *key,
 {
 
 
-  unsigned int hash_code = xbt_dict_hash_ext(key, key_len);
+  unsigned int hash_code = xbt_str_hash_ext(key, key_len);
   xbt_dictelm_t current, previous = NULL;
 
   xbt_assert(dict);
@@ -689,12 +608,6 @@ void xbt_dict_dump_sizes(xbt_dict_t dict)
  */
 void xbt_dict_preinit(void)
 {
-  if (dict_elm_mallocator != NULL) {
-    /* Already created. I guess we want to switch to MC mode, so kill the previously created mallocator */
-    xbt_mallocator_free(dict_elm_mallocator);
-    xbt_mallocator_free(dict_het_elm_mallocator);
-  }
-
   dict_elm_mallocator = xbt_mallocator_new(256,
                                            dict_elm_mallocator_new_f,
                                            dict_elm_mallocator_free_f,
