@@ -11,6 +11,7 @@
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "mmprivate.h"
+#include "xbt/ex.h"
 
 /* Return memory to the heap.
    Like `mfree' but don't call a mfree_hook if there is one.  */
@@ -19,13 +20,15 @@
 void mfree(struct mdesc *mdp, void *ptr)
 {
   int type;
-  size_t block;
+  size_t block, frag_nb;
   register size_t i;
-  struct list *prev, *next;
   int it;
 
+  mmalloc_paranoia(mdp);
+//  fprintf(stderr,"free(%p)\n",ptr);
+
   if (ptr == NULL)
-	  return;
+    return;
 
   block = BLOCK(ptr);
 
@@ -34,16 +37,22 @@ void mfree(struct mdesc *mdp, void *ptr)
     abort();
   }
 
-
   type = mdp->heapinfo[block].type;
 
   switch (type) {
   case -1: /* Already free */
-    fprintf(stderr,"Asked to free a fragment in a block that is already free. I'm puzzled\n");
-    abort();
+    UNLOCK(mdp);
+    THROWF(system_error, 0, "Asked to free a fragment in a block that is already free. I'm puzzled\n");
     break;
-
+    
   case 0:
+    /* Get as many statistics as early as we can.  */
+    mdp -> heapstats.chunks_used--;
+    mdp -> heapstats.bytes_used -=
+      mdp -> heapinfo[block].busy_block.size * BLOCKSIZE;
+    mdp -> heapstats.bytes_free +=
+      mdp -> heapinfo[block].busy_block.size * BLOCKSIZE;
+
     /* Find the free cluster previous to this one in the free list.
        Start searching at the last block referenced; this may benefit
        programs with locality of allocation.  */
@@ -67,12 +76,12 @@ void mfree(struct mdesc *mdp, void *ptr)
       mdp->heapinfo[i].free_block.size += mdp->heapinfo[block].busy_block.size;
       /* Mark all my ex-blocks as free */
       for (it=0; it<mdp->heapinfo[block].busy_block.size; it++) {
-    	  if (mdp->heapinfo[block+it].type <0) {
-      		fprintf(stderr,"Internal Error: Asked to free a block already marked as free (block=%lu it=%d type=%lu). Please report this bug.\n",
-      				(unsigned long)block,it,(unsigned long)mdp->heapinfo[block].type);
-      		abort();
-    	  }
-    	  mdp->heapinfo[block+it].type = -1;
+        if (mdp->heapinfo[block+it].type <0) {
+          fprintf(stderr,"Internal Error: Asked to free a block already marked as free (block=%lu it=%d type=%lu). Please report this bug.\n",
+                  (unsigned long)block,it,(unsigned long)mdp->heapinfo[block].type);
+          abort();
+        }
+        mdp->heapinfo[block+it].type = -1;
       }
 
       block = i;
@@ -84,14 +93,15 @@ void mfree(struct mdesc *mdp, void *ptr)
       mdp->heapinfo[block].free_block.prev = i;
       mdp->heapinfo[i].free_block.next = block;
       mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.prev = block;
+      mdp -> heapstats.chunks_free++;
       /* Mark all my ex-blocks as free */
       for (it=0; it<mdp->heapinfo[block].free_block.size; it++) {
-    	  if (mdp->heapinfo[block+it].type <0) {
-      		fprintf(stderr,"Internal error: Asked to free a block already marked as free (block=%lu it=%d/%lu type=%lu). Please report this bug.\n",
-      				(unsigned long)block,it,(unsigned long)mdp->heapinfo[block].free_block.size,(unsigned long)mdp->heapinfo[block].type);
-      		abort();
-    	  }
-    	  mdp->heapinfo[block+it].type = -1;
+        if (mdp->heapinfo[block+it].type <0) {
+          fprintf(stderr,"Internal error: Asked to free a block already marked as free (block=%lu it=%d/%lu type=%lu). Please report this bug.\n",
+                  (unsigned long)block,it,(unsigned long)mdp->heapinfo[block].free_block.size,(unsigned long)mdp->heapinfo[block].type);
+          abort();
+        }
+        mdp->heapinfo[block+it].type = -1;
       }
     }
 
@@ -101,83 +111,83 @@ void mfree(struct mdesc *mdp, void *ptr)
     if (block + mdp->heapinfo[block].free_block.size ==
         mdp->heapinfo[block].free_block.next) {
       mdp->heapinfo[block].free_block.size
-          += mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.size;
+        += mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.size;
       mdp->heapinfo[block].free_block.next
-          = mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.next;
+        = mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.next;
       mdp->heapinfo[mdp->heapinfo[block].free_block.next].free_block.prev = block;
+      mdp -> heapstats.chunks_free--;
     }
 
     /* Now see if we can return stuff to the system.  */
     /*    blocks = mdp -> heapinfo[block].free.size;
-       if (blocks >= FINAL_FREE_BLOCKS && block + blocks == mdp -> heaplimit
-       && mdp -> morecore (mdp, 0) == ADDRESS (block + blocks))
-       {
-       register size_t bytes = blocks * BLOCKSIZE;
-       mdp -> heaplimit -= blocks;
-       mdp -> morecore (mdp, -bytes);
-       mdp -> heapinfo[mdp -> heapinfo[block].free.prev].free.next
-       = mdp -> heapinfo[block].free.next;
-       mdp -> heapinfo[mdp -> heapinfo[block].free.next].free.prev
-       = mdp -> heapinfo[block].free.prev;
-       block = mdp -> heapinfo[block].free.prev;
-       mdp -> heapstats.chunks_free--;
-       mdp -> heapstats.bytes_free -= bytes;
-       } */
+          if (blocks >= FINAL_FREE_BLOCKS && block + blocks == mdp -> heaplimit
+          && mdp -> morecore (mdp, 0) == ADDRESS (block + blocks))
+          {
+          register size_t bytes = blocks * BLOCKSIZE;
+          mdp -> heaplimit -= blocks;
+          mdp -> morecore (mdp, -bytes);
+          mdp -> heapinfo[mdp -> heapinfo[block].free.prev].free.next
+          = mdp -> heapinfo[block].free.next;
+          mdp -> heapinfo[mdp -> heapinfo[block].free.next].free.prev
+          = mdp -> heapinfo[block].free.prev;
+          block = mdp -> heapinfo[block].free.prev;
+          mdp -> heapstats.chunks_free--;
+          mdp -> heapstats.bytes_free -= bytes;
+          } */
 
-    /* Set the next search to begin at this block.  */
+    /* Set the next search to begin at this block.  
+       This is probably important to the trick where realloc returns the block to 
+       the system before reasking for the same block with a bigger size.  */
     mdp->heapindex = block;
     break;
 
   default:
-    /* Get the address of the first free fragment in this block.  */
-    prev = (struct list *)
-        ((char *) ADDRESS(block) +
-         (mdp->heapinfo[block].busy_frag.first << type));
+    /* Do some of the statistics.  */
+    mdp -> heapstats.chunks_used--;
+    mdp -> heapstats.bytes_used -= 1 << type;
+    mdp -> heapstats.chunks_free++;
+    mdp -> heapstats.bytes_free += 1 << type;
 
+    frag_nb = RESIDUAL(ptr, BLOCKSIZE) >> type;
+
+    if( mdp->heapinfo[block].busy_frag.frag_size[frag_nb] == -1){
+      UNLOCK(mdp);
+      THROWF(system_error, 0, "Asked to free a fragment that is already free. I'm puzzled\n");
+    }
+
+    /* Set size used in the fragment to -1 */
+    mdp->heapinfo[block].busy_frag.frag_size[frag_nb] = -1;
+
+//    fprintf(stderr,"nfree:%zu capa:%d\n", mdp->heapinfo[block].busy_frag.nfree,(BLOCKSIZE >> type));
     if (mdp->heapinfo[block].busy_frag.nfree ==
         (BLOCKSIZE >> type) - 1) {
-      /* If all fragments of this block are free, remove them
-         from the fragment list and free the whole block.  */
-      next = prev;
-      for (i = 1; i < (size_t) (BLOCKSIZE >> type); ++i) {
-        next = next->next;
-      }
-      prev->prev->next = next;
-      if (next != NULL) {
-        next->prev = prev->prev;
-      }
+      /* If all fragments of this block are free, remove this block from its swag and free the whole block.  */
+      xbt_swag_remove(&mdp->heapinfo[block],&mdp->fraghead[type]);
+
       /* pretend that this block is used and free it so that it gets properly coalesced with adjacent free blocks */
       mdp->heapinfo[block].type = 0;
       mdp->heapinfo[block].busy_block.size = 1;
       mdp->heapinfo[block].busy_block.busy_size = 0;
-
+      
+      /* Keep the statistics accurate.  */
+      mdp -> heapstats.chunks_used++;
+      mdp -> heapstats.bytes_used += BLOCKSIZE;
+      mdp -> heapstats.chunks_free -= BLOCKSIZE >> type;
+      mdp -> heapstats.bytes_free -= BLOCKSIZE;
+      
       mfree((void *) mdp, (void *) ADDRESS(block));
     } else if (mdp->heapinfo[block].busy_frag.nfree != 0) {
-      /* If some fragments of this block are free, link this
-         fragment into the fragment list after the first free
-         fragment of this block. */
-      next = (struct list *) ptr;
-      next->next = prev->next;
-      next->prev = prev;
-      prev->next = next;
-      if (next->next != NULL) {
-        next->next->prev = next;
-      }
+      /* If some fragments of this block are free, you know what? I'm already happy. */
       ++mdp->heapinfo[block].busy_frag.nfree;
     } else {
       /* No fragments of this block were free before the one we just released,
-       * so link this fragment into the fragment list and announce that
-         it is the first free fragment of this block. */
-      prev = (struct list *) ptr;
+       * so add this block to the swag and announce that
+       it is the first free fragment of this block. */
       mdp->heapinfo[block].busy_frag.nfree = 1;
-      mdp->heapinfo[block].busy_frag.first =
-          RESIDUAL(ptr, BLOCKSIZE) >> type;
-      prev->next = mdp->fraghead[type].next;
-      prev->prev = &mdp->fraghead[type];
-      prev->prev->next = prev;
-      if (prev->next != NULL) {
-        prev->next->prev = prev;
-      }
+      mdp->heapinfo[block].freehook.prev = NULL;
+      mdp->heapinfo[block].freehook.next = NULL;
+
+      xbt_swag_insert(&mdp->heapinfo[block],&mdp->fraghead[type]);
     }
     break;
   }
