@@ -23,16 +23,21 @@
 XBT_LOG_NEW_CATEGORY(xbt, "All XBT categories (simgrid toolbox)");
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(module, xbt, "module handling");
 
+XBT_LOG_NEW_CATEGORY(smpi, "All SMPI categories"); /* lives here even if that's a bit odd to solve linking issues: this is used in xbt_log_file_appender to detect whether SMPI is used (and thus whether we should unbench the writing to disk) */
+
+
 char *xbt_binary_name = NULL;   /* Mandatory to retrieve neat backtraces */
 int xbt_initialized = 0;
 
-int _surf_do_model_check = 0;   /* this variable is used accros the libraries, and must be declared in XBT so that it's also defined in GRAS (not only in libsimgrid) */
+int _surf_do_model_check = 0;
+int _surf_mc_checkpoint=0;
+char* _surf_mc_property_file=NULL;
 
 /* Declare xbt_preinit and xbt_postexit as constructor/destructor of the library.
  * This is crude and rather compiler-specific, unfortunately.
  */
 static void xbt_preinit(void) _XBT_GNUC_CONSTRUCTOR(200);
-static void xbt_postexit(void) _XBT_GNUC_DESTRUCTOR(200);
+static void xbt_postexit(void);
 
 #ifdef _XBT_WIN32
 # undef _XBT_NEED_INIT_PRAGMA
@@ -40,7 +45,6 @@ static void xbt_postexit(void) _XBT_GNUC_DESTRUCTOR(200);
 
 #ifdef _XBT_NEED_INIT_PRAGMA
 #pragma init (xbt_preinit)
-#pragma fini (xbt_postexit)
 #endif
 
 #ifdef _XBT_WIN32
@@ -62,13 +66,12 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason,
                     LPVOID lpvReserved)
 {
   if (fdwReason == DLL_PROCESS_ATTACH
-		  && xbt_dll_process_is_attached == 0) {
-	  xbt_dll_process_is_attached = 1;
-	  xbt_preinit();
+      && xbt_dll_process_is_attached == 0) {
+    xbt_dll_process_is_attached = 1;
+    xbt_preinit();
   } else if (fdwReason == DLL_PROCESS_DETACH
-		  && xbt_dll_process_is_attached == 1) {
-	  xbt_dll_process_is_attached = 0;
-      xbt_postexit();
+      && xbt_dll_process_is_attached == 1) {
+    xbt_dll_process_is_attached = 0;
   }
   return 1;
 }
@@ -89,6 +92,8 @@ static void xbt_preinit(void)
   xbt_dict_preinit();
   xbt_datadesc_preinit();
   xbt_trp_preinit();
+
+  atexit(xbt_postexit);
 }
 
 static void xbt_postexit(void)
