@@ -74,8 +74,7 @@ static xbt_running_ctx_t *_os_thread_get_running_ctx(void)
 static void _os_thread_ex_terminate(xbt_ex_t * e)
 {
   xbt_ex_display(e);
-
-  abort();
+  xbt_abort();
   /* FIXME: there should be a configuration variable to choose to kill everyone or only this one */
 }
 
@@ -134,11 +133,12 @@ void xbt_os_thread_mod_postexit(void)
   __xbt_ex_terminate = &__xbt_ex_terminate_default;
 }
 
+/* this function is critical to tesh+mmalloc, don't mess with it */
 int xbt_os_thread_atfork(void (*prepare)(void),
                          void (*parent)(void), void (*child)(void))
 {
 #ifdef WIN32
-	THROW_UNIMPLEMENTED; //pthread_atfork is not implemented in pthread.h on windows
+  THROW_UNIMPLEMENTED; //pthread_atfork is not implemented in pthread.h on windows
 #else
   return pthread_atfork(prepare, parent, child);
 #endif
@@ -642,7 +642,7 @@ typedef struct xbt_os_thread_ {
 #endif
 
 /* the default size of the stack of the threads (in bytes)*/
-#define XBT_DEFAULT_THREAD_STACK_SIZE	4096
+#define XBT_DEFAULT_THREAD_STACK_SIZE  4096
 
 /* key to the TLS containing the xbt_os_thread_t structure */
 static unsigned long xbt_self_thread_key;
