@@ -28,7 +28,7 @@ typedef struct s_xbt_thread_ {
   void *userparam;
   void *father_data;
   /* stuff to allow other people to wait on me with xbt_thread_join */
-  int joinable:1, done:1;
+  unsigned joinable:1, done:1;
   xbt_cond_t cond;
   xbt_mutex_t mutex;
 } s_xbt_thread_t;
@@ -66,7 +66,7 @@ xbt_thread_t xbt_thread_create(const char *name, void_f_pvoid_t code,
   simcall_process_create(&res->s_process, name,
                            xbt_thread_create_wrapper, res,
                            SIMIX_host_self_get_name(), -1.0, 0, NULL,
-                           /*props */ NULL);
+                           /*props */ NULL,0);
   res->joinable = joinable;
   res->done = 0;
   res->cond = xbt_cond_init();
