@@ -23,7 +23,7 @@ static int send(int argc, char *argv[])
 static int receive(int argc, char *argv[])
 {
   XBT_INFO("Receiving");
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   MSG_task_receive_with_timeout(&task, MSG_host_get_name(MSG_host_self()), DBL_MAX);
   xbt_assert(MSG_task_get_sender(task), "No sender received");
   XBT_INFO("Got a message sent by '%s'",
@@ -34,9 +34,9 @@ static int receive(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
   /*   Application deployment */
   MSG_function_register("send", &send);
@@ -45,7 +45,7 @@ int main(int argc, char *argv[])
   MSG_create_environment(argv[1]);
   MSG_launch_application(argv[1]);
   res = MSG_main();
-  MSG_clean();
+
   if (res == MSG_OK)
     return 0;
   else
